/*! @file    main.c
 *  @brief   I2C中断访问EEPROM例程
 *  @version 2023-04-20  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_gpio.h"
#include "nccm0670_i2c.h"
#include "nccm0670_rcc.h"
#include "systick.h"

/* AT24C02 地址 */
#define SLAVE_ADDRESS  0xA0
/* I2C数据帧最大长度 */
#define MAX_NBYTE_SIZE 255U
/* 测试数据长度，AT24C02最大只支持8字节连续写入 */
#define I2C_DATA_LEN   8

void rcc_config(void);
void gpio_config(void);
void i2c_config(void);
uint8_t i2c_mem_write_it(I2C_Type *i2cx,
                         uint16_t dev_address,
                         uint16_t mem_address,
                         uint16_t mem_address_size,
                         uint8_t *data,
                         uint16_t size);
uint8_t i2c_mem_read_it(I2C_Type *i2cx,
                        uint16_t dev_address,
                        uint16_t mem_address,
                        uint16_t mem_address_size,
                        uint8_t *data,
                        uint16_t size);
/* 发送数据起始值 */
uint8_t base = 0;
/* 保存返回值 */
uint8_t result;
/* I2C传输数据指针 */
uint8_t *buf_ptr;
/* I2C传输计数器 */
volatile uint16_t xfer_count;
/* I2C传输长度 */
volatile uint16_t xfer_size;
/* I2C传输模式 */
volatile uint16_t xfer_mode;
/* I2C读操作标志 */
uint16_t read_mode;
/* I2C设备地址 */
uint16_t i2c_device_address;
/* I2C寄存器地址 */
uint16_t i2c_mem_address;
/* I2C寄存器地址长度 */
uint8_t i2c_mem_address_size;
/* I2C发送数据缓存 */
uint8_t i2c_tx_data[I2C_DATA_LEN];
/* I2C接收数据缓存 */
uint8_t i2c_rx_data[I2C_DATA_LEN];

int main(void)
{
    /* 时钟配置 */
    rcc_config();
    /* 引脚配置 */
    gpio_config();
    /* I2C配置 */
    i2c_config();
    /* 调试串口配置 */
    ncc_eval_com_init();
    /* 系统定时器配置 */
    systick_init();
    /* 打开I2C中断 */
    NVIC_EnableIRQ(I2C1_IRQn);

    while (1)
    {
        /* 修改写入的数据 */
        base++;
        for (size_t i = 0; i < I2C_DATA_LEN; i++)
        {
            i2c_tx_data[i] = base + i;
        }

        printf("write start\r\n");
        /* 写入数据 */
        result = i2c_mem_write_it(I2C1, SLAVE_ADDRESS, 0, 1, i2c_tx_data, I2C_DATA_LEN);
        if (result != 0U)
        {
            printf("write error : %d\r\n", result);
        }
        else
        {
            printf("write success\r\n");
        }

        delay_ms(1000);

        printf("read start\r\n");
        /* 读出数据 */
        result = i2c_mem_read_it(I2C1, SLAVE_ADDRESS, 0, 1, i2c_rx_data, I2C_DATA_LEN);
        if (result != 0U)
        {
            printf("read error : %d\r\n", result);
        }
        else
        {
            printf("read data : ");
            for (size_t i = 0; i < I2C_DATA_LEN; i++)
            {
                printf("0x%02x ", i2c_rx_data[i]);
            }
            printf("\r\n");
        }
    }
}

void rcc_config(void)
{
    RCC_EnablePeripheralClock(RCC_GPIOB);
    RCC_EnablePeripheralClock(RCC_I2C1);
}

void gpio_config(void)
{
    gpio_config_t gpio_cfg = {0};

    gpio_cfg.pin           = GPIO_PIN_6 | GPIO_PIN_7;
    gpio_cfg.mode          = GPIO_MODE_ALTERNATE;
    gpio_cfg.output_type   = GPIO_OUTPUT_OPENDRAIN;
    gpio_cfg.pull          = GPIO_PULLUP;
    gpio_cfg.speed         = GPIO_SPEED_FREQ_HIGH;
    gpio_cfg.alternate     = GPIO_AF0_I2C1;
    GPIO_Init(GPIOB, &gpio_cfg);
}

void i2c_config(void)
{
    i2c_config_t i2c_cfg      = {0};

    /* PCLK=4M, I2C 波特率:100K */
    i2c_cfg.timing            = I2C_CONVERT_TIMINGS(0x0, 4, 2, 0xF, 0x13);
    /* 主机地址为7位 */
    i2c_cfg.addressing_mode   = I2C_ADDRESSING_MODE_7BIT;
    /* 禁止双地址模式 */
    i2c_cfg.dual_address_mode = I2C_DUAL_ADDRESS_DISABLE;
    /* 禁止广播模式 */
    i2c_cfg.general_call_mode = I2C_GENERAL_CALL_DISABLE;
    /* 禁止关闭时钟延长 */
    i2c_cfg.no_stretch_mode   = I2C_NO_STRETCH_DISABLE;
    I2C_Init(I2C1, &i2c_cfg);
}

/**
 * @brief 中断方式写入从设备数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[in] mem_address      寄存器地址
 * @param[in] mem_address_size 寄存器地址长度
 * @param[in] data             数据缓存
 * @param[in] size             数据长度
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_mem_write_it(I2C_Type *i2cx,
                         uint16_t dev_address,
                         uint16_t mem_address,
                         uint16_t mem_address_size,
                         uint8_t *data,
                         uint16_t size)
{
    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 总线忙 */
    if (I2C_GetStatusFlag(i2cx, I2C_FLAG_BUSY))
    {
        return 2;
    }
    buf_ptr              = data;
    xfer_count           = size;
    i2c_device_address   = dev_address;
    i2c_mem_address      = mem_address;
    i2c_mem_address_size = mem_address_size;

    xfer_mode            = 1;
    read_mode            = 0;
    I2C_HandleTransfer(i2cx, dev_address, mem_address_size, I2C_MODE_RELOAD, I2C_GENERATE_START_WRITE);

    I2C_SetIntMode(i2cx, I2C_INT_TCIE, ENABLE);
    I2C_SetIntMode(i2cx, I2C_INT_TXIE, ENABLE);

    return 0;
}

/**
 * @brief 中断方式读取从设备数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[in] mem_address      寄存器地址
 * @param[in] mem_address_size 寄存器地址长度
 * @param[out] data            数据缓存
 * @param[in] size             数据长度
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_mem_read_it(I2C_Type *i2cx,
                        uint16_t dev_address,
                        uint16_t mem_address,
                        uint16_t mem_address_size,
                        uint8_t *data,
                        uint16_t size)
{
    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 总线忙 */
    if (I2C_GetStatusFlag(i2cx, I2C_FLAG_BUSY))
    {
        return 2;
    }

    buf_ptr              = data;
    xfer_count           = size;
    i2c_device_address   = dev_address;
    i2c_mem_address      = mem_address;
    i2c_mem_address_size = mem_address_size;

    xfer_mode            = 1;
    read_mode            = 1;
    I2C_HandleTransfer(i2cx, dev_address, mem_address_size, I2C_MODE_SOFTEND, I2C_GENERATE_START_WRITE);

    I2C_SetIntMode(i2cx, I2C_INT_RXIE, ENABLE);
    I2C_SetIntMode(i2cx, I2C_INT_TCIE, ENABLE);
    I2C_SetIntMode(i2cx, I2C_INT_TXIE, ENABLE);

    return 0;
}

void I2C1_IRQHandler(void)
{
    if (I2C_GetIntFlag(I2C1, I2C_FLAG_RXNE))
    {
        /* 保存接收到的数据 */
        *buf_ptr = I2C_ReceiveData8(I2C1);

        buf_ptr++;
        xfer_size--;
        xfer_count--;
    }
    else if (I2C_GetIntFlag(I2C1, I2C_FLAG_TXIS))
    {
        /* 发送16位寄存器高地址或8位寄存器地址 */
        if (xfer_mode == 1)
        {
            /* 如果寄存器地址为8位 */
            if (i2c_mem_address_size == 1U)
            {
                /* 发送寄存器地址 */
                I2C_TransmitData8(I2C1, i2c_mem_address);
                /* 防止再次进入TXIS中断 */
                I2C_SetIntMode(I2C1, I2C_INT_TXIE, DISABLE);
                xfer_mode = 3;
            }
            else
            {
                /* 发送寄存器地址高8位 */
                I2C_TransmitData8(I2C1, i2c_mem_address >> 8U);
                xfer_mode = 2;
            }
        }
        /* 发送16位寄存器低地址 */
        else if (xfer_mode == 2)
        {
            /* 发送寄存器地址低8位 */
            I2C_TransmitData8(I2C1, i2c_mem_address);
            /* 防止再次进入TXIS中断 */
            I2C_SetIntMode(I2C1, I2C_INT_TXIE, DISABLE);
            xfer_mode = 3;
        }
        /* 发送数据 */
        else
        {
            I2C_TransmitData8(I2C1, *buf_ptr);

            buf_ptr++;
            xfer_size--;
            xfer_count--;
            if (xfer_size == 0)
            {
                I2C_SetIntMode(I2C1, I2C_INT_TXIE, DISABLE);
            }
        }
    }
    else if (I2C_GetIntFlag(I2C1, I2C_FLAG_TCR))
    {
        if ((xfer_count != 0U) && (xfer_size == 0U))
        {
            if (xfer_count > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                I2C_HandleTransfer(I2C1, i2c_device_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_NOSTARTSTOP);
            }
            else
            {
                xfer_size = xfer_count;
                I2C_HandleTransfer(I2C1, i2c_device_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_NOSTARTSTOP);
            }
            I2C_SetIntMode(I2C1, I2C_INT_TXIE, ENABLE);
        }
    }
    else if (I2C_GetIntFlag(I2C1, I2C_FLAG_TC))
    {
        I2C_SetIntMode(I2C1, I2C_INT_TCIE, DISABLE);

        if (read_mode)
        {
            /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
            if (xfer_count > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE,并发送START和读请求 */
                I2C_HandleTransfer(I2C1, i2c_device_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_START_READ);
            }
            else
            {
                xfer_size = xfer_count;
                /* 传输结果自动发送STOP, 并发送START和读请求 */
                I2C_HandleTransfer(I2C1, i2c_device_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_START_READ);
            }
        }
    }
    else
    {
    }
}
