/*! @file    main.c
 *  @brief   LPUART低功耗唤醒例程
 *  @version 2023-05-09  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_exti.h"
#include "nccm0670_gpio.h"
#include "nccm0670_lpuart.h"
#include "nccm0670_pwr.h"
#include "nccm0670_rcc.h"
#include "systick.h"

void lpuart1_config(void);
void enter_deepsleep_mode(void);
void lpuart_puts(const char *s);

int main(void)
{
    /* 初始化lpuart */
    lpuart1_config();
    /* 进入低功耗模式 */
    enter_deepsleep_mode();
    /* 初始化系统定时器 */
    systick_init();

    while (1)
    {
        lpuart_puts("You will see this message if wakeup successfully\n");
        delay_ms(500);
    }
    return 0;
}

void lpuart1_config(void)
{
    gpio_config_t gpio_cfg     = {0};
    lpuart_config_t lpuart_cfg = {0};

    RCC_EnablePeripheralClock(RCC_GPIOA);
    RCC_EnablePeripheralClock(RCC_LPUART1);

    RCC_EnableOSC(RCC_OSC_LSE);
    RCC_SetLpuart1ClockSource(RCC_LPUART_SRC_LSE);

    gpio_cfg.pin       = GPIO_PIN_6 | GPIO_PIN_7;
    gpio_cfg.mode      = GPIO_MODE_ALTERNATE;
    gpio_cfg.pull      = GPIO_NOPULL;
    gpio_cfg.speed     = GPIO_SPEED_FREQ_LOW;
    gpio_cfg.alternate = GPIO_AF1_LPUART1;
    GPIO_Init(GPIOB, &gpio_cfg);

    /* 通信波特率,帧长度,1位停止位(0.5和1相同、1.5和2相同),奇偶校验位,传输方向，打开停止模式唤醒 */
    lpuart_cfg.baudrate           = 9600;
    lpuart_cfg.datawidth          = LPUART_DATAWIDTH_8B;
    lpuart_cfg.stopbits           = LPUART_STOPBITS_1;
    lpuart_cfg.parity             = LPUART_PARITY_NONE;
    lpuart_cfg.transfer_direction = LPUART_DIRECTION_TX_RX;
    lpuart_cfg.stop_mode          = LPUART_STOP_MODE_ENABLE;
    /* 反初始化lpuart */
    LPUART_DeInit(LPUART1);
    /* 初始化lpuart */
    LPUART_Init(LPUART1, &lpuart_cfg);
    /* 使能唤醒中断 */
    LPUART_SetIntMode(LPUART1, LPUART_INT_WUF, ENABLE);
    /* 使能lpuart */
    LPUART_Enable(LPUART1);
    NVIC_SetPriority(LPUART1_IRQn, 5);
    NVIC_EnableIRQ(LPUART1_IRQn);
}

void LPUART1_2_IRQHandler(void)
{
    if (LPUART_GetIntFlag(LPUART1, LPUART_FLAG_WUF))
    {
        LPUART_ClearStatusFlag(LPUART1, LPUART_FLAG_WUF);
    }
}

void enter_deepsleep_mode(void)
{
    /* 进入休眠前打印信息 */
    lpuart_puts("Enter deepsleep mode!\r\n");
    /* 打开PWR外设时钟 */
    RCC_EnablePeripheralClock(RCC_PWR);
    /* 进入STOP模式后， 关闭bandgap */
    PWR_SetBandgapPowerDown(ENABLE);

    exti_config_t cfg = {0};
    /* 选择EXTI 中断模式 */
    cfg.mode          = EXTI_MODE_INTERRUPT;
    /* 选择EXTI 下降沿 */
    cfg.trigger       = EXTI_TRIGGER_FALLING;
    EXTI_SetConfigLine(EXTI_LINE_20, &cfg);

    /* 进入STOP模式，并将LDO关闭 */
    PWR_EnterStopMode(PWR_LOWPOWERREGULATOR_ON, PWR_LOWPOWER_ENTRY_WFI);

    /* 唤醒后打印信息 */
    lpuart_puts("Wakeup!\r\n");
}

void lpuart_puts(const char *s)
{
    while (*s)
    {
        while (!LPUART_GetStatusFlag(LPUART1, LPUART_FLAG_TXE))
            ;
        LPUART_TransmitData(LPUART1, *s++);
    }
}