/*! @file    main.c
 *  @brief   USART例程
 *  @version 2021-06-29  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_gpio.h"
#include "nccm0670_rcc.h"
#include "nccm0670_usart.h"

/* 接收超时标志 */
static volatile uint8_t rx_timeout = 0;

void rcc_config(void);
void gpio_config(void);
void usart_config(void);

int main(void)
{
    /* 初始化时钟 */
    rcc_config();
    /* 初始化GPIO */
    gpio_config();
    /* 初始化USART */
    usart_config();

    while (1)
    {
        if (rx_timeout)
        {
            rx_timeout = 0;
            printf("Receive time out\n");
        }

    }
}

void rcc_config(void)
{
    /* 将系统时钟设置为48MHz */
    RCC_SetHsiDiv(RCC_HSI_DIV1);
    /* 使能GPIOA时钟 */
    RCC_EnablePeripheralClock(RCC_GPIOA);
    /* 使能USART1时钟 */
    RCC_EnablePeripheralClock(RCC_USART1);
}

void gpio_config(void)
{
    gpio_config_t gpio_cfg = {0};

    /* 配置PA2和PA3引脚复用为USART1 */
    gpio_cfg.pin           = GPIO_PIN_4 | GPIO_PIN_5;
    gpio_cfg.mode          = GPIO_MODE_ALTERNATE;
    gpio_cfg.pull          = GPIO_NOPULL;
    gpio_cfg.speed         = GPIO_SPEED_FREQ_LOW;
    gpio_cfg.alternate     = GPIO_AF0_USART1;
    GPIO_Init(GPIOA, &gpio_cfg);
}

void usart_config(void)
{
    usart_base_cfg_t usart_cfg      = {0};
    /* 通信波特率 */
    usart_cfg.baudrate              = 115200;
    /* 帧长度 */
    usart_cfg.datawidth             = USART_DATAWIDTH_8B;
    /* 1位停止位(0.5和1相同、1.5和2相同) */
    usart_cfg.stopbits              = USART_STOPBITS_1;
    /* 奇偶校验位 */
    usart_cfg.parity                = USART_PARITY_NONE;
    /* 传输方向 */
    usart_cfg.transfer_direction    = USART_DIRECTION_TX_RX;
    /* 设置接收超时时间为3秒 */
    USART_SetRxTimeout(USART1, 115200 * 3);
    /* 开启接收超时模式 */
    USART_SetRxTimeoutMode(USART1, ENABLE);
    /* 初始化串口 */
    USART_Init(USART1, &usart_cfg);

    /* 使能串口接收非空中断 */
    USART_SetIntMode(USART1, USART_INT_RXNE, ENABLE);
		
    /* 配置USART1中断优级 */
    NVIC_SetPriority(USART1_IRQn, 5);
    /* 使能USART1中断 */
    NVIC_EnableIRQ(USART1_IRQn);
}

void USART1_IRQHandler(void)
{
    if (USART_GetIntFlag(USART1, USART_FLAG_RXNE))
    {
        USART_ClearStatusFlag(USART1, USART_FLAG_RXNE);

        /* 接收数据并发送 */
        USART_TransmitData(USART1, USART_ReceiveData(USART1));
        /* 开启串口发送中断 */
        USART_SetIntMode(USART1, USART_INT_TXE, ENABLE);
        
        USART_SetIntMode(USART1, USART_INT_RTO, ENABLE);
    }

    if (USART_GetIntFlag(USART1, USART_FLAG_RTOF))
    {
        /* 清除标志并打印 */
        USART_ClearStatusFlag(USART1, USART_FLAG_RTOF);
        USART_SetIntMode(USART1, USART_INT_RTO, DISABLE);
        rx_timeout = 1;
    }

    if (USART_GetIntFlag(USART1, USART_FLAG_TXE))
    {
        /* 发送结束数据清除标志并关闭发送中断 */
        USART_ClearStatusFlag(USART1, USART_FLAG_TXE);
        USART_SetIntMode(USART1, USART_INT_TXE, DISABLE);
    }
}