/*! @file    main.c
 *  @brief   USART 同步模式例程
 *  @version 2021-06-29  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "nccm0670_eval.h"
#include "nccm0670_gpio.h"
#include "nccm0670_rcc.h"
#include "nccm0670_spi.h"
#include "nccm0670_usart.h"
#include "systick.h"

uint8_t tx_buf[4]     = {0x12, 0x34, 0x56, 0x78};
uint8_t tx_spi_buf[4] = {0x22, 0x33, 0x55, 0x77};
uint8_t rx_buf[4]     = {0};
uint8_t rx_spi_buf[4] = {0};
uint8_t a = 0, b = 0, c = 0;

void usart_sync_config(void);
void spi_config(void);

int main(void)
{
    /* 将系统时钟设置为48MHz */
    RCC_SetHsiDiv(RCC_HSI_DIV1);
    SystemCoreClockUpdate();
    systick_init();
    ncc_eval_com_init();
    /* 初始化串口 */
    usart_sync_config();
    /* 初始化SPI */
    spi_config();

    for (int i = 0U; i < 4; i++)
    {
        /* USART发送数据 */
        USART_TransmitData(USART1, tx_buf[i]);
    }

    delay_ms(1000);

    printf("usart receive data:\n%x\n%x\n%x\n%x\n", rx_buf[0], rx_buf[1], rx_buf[2], rx_buf[3]);
    delay_ms(100);
    printf("spi receive data:\n%x\n%x\n%x\n%x\n", rx_spi_buf[0], rx_spi_buf[1], rx_spi_buf[2], rx_spi_buf[3]);

    while (1)
    {
    }
}
void usart_sync_config(void)
{

    gpio_config_t gpio_cfg      = {0};
    usart_clock_cfg_t clock_cfg = {0};
    usart_base_cfg_t usart_cfg  = {0};
    /* 使能GPIOC时钟*/
    RCC_EnablePeripheralClock(RCC_GPIOC);
    /* 使能USART1时钟 */
    RCC_EnablePeripheralClock(RCC_USART1);
    /* 配置PC10,PC11和PC12引脚复用为USART1 */
    gpio_cfg.pin       = GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_11;
    gpio_cfg.mode      = GPIO_MODE_ALTERNATE;
    gpio_cfg.pull      = GPIO_PULLUP;
    gpio_cfg.speed     = GPIO_SPEED_FREQ_LOW;
    gpio_cfg.alternate = GPIO_AF1_USART1;
    GPIO_Init(GPIOC, &gpio_cfg);

    /* 通信波特率*/
    usart_cfg.baudrate              = 380000;
    /* 帧长度 */
    usart_cfg.datawidth             = USART_DATAWIDTH_8B;
    /* 1位停止位(0.5和1相同、1.5和2相同） */
    usart_cfg.stopbits              = USART_STOPBITS_1;
    /* 奇偶校验位 */
    usart_cfg.parity                = USART_PARITY_NONE;
    /* 传输方向 */
    usart_cfg.transfer_direction    = USART_DIRECTION_TX_RX;
    /* 时钟配置 */
    clock_cfg.output                = USART_CLK_OUTPUT;
    clock_cfg.polarity              = USART_POLARITY_LOW;
    clock_cfg.phase                 = USART_PHASE_2EDGE;
    clock_cfg.last_clk_pulse        = USART_LASTCLKPULSE_OUTPUT;

    USART_DeInit(USART1);
    USART_Init(USART1, &usart_cfg);
    USART_SetSclkOutput(USART1, ENABLE);
    USART_ClockInit(USART1, &clock_cfg);
    /* 使能串口接收非空中断 */
    USART_SetIntMode(USART1, USART_INT_RXNE, ENABLE);
    /* 配置USART1中断优级 */
    NVIC_SetPriority(USART1_IRQn, 5);
    /* 使能USART1中断 */
    NVIC_EnableIRQ(USART1_IRQn);
}

void spi_config(void)
{
    /* 使能GPIOA时钟*/
    RCC_EnablePeripheralClock(RCC_GPIOB);
    /* 使能SPI1时钟*/
    RCC_EnablePeripheralClock(RCC_SPI1);
    /* 配置PA4,PA5,PA6和PA7引脚复用为SPI1*/
    gpio_config_t gpio_cfg = {
        .pin         = GPIO_PIN_2 | GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_5,
        .mode        = GPIO_MODE_ALTERNATE,
        .speed       = GPIO_SPEED_FREQ_HIGH,
        .output_type = GPIO_OUTPUT_PUSHPULL,
        .pull        = GPIO_PULLUP,
        .alternate   = GPIO_AF1_SPI1,
    };

    GPIO_Init(GPIOB, &gpio_cfg);

    spi_config_t spi_cfg = {
        /* 从机模式 */
        .mode         = SPI_MODE_SLAVE,
        /* motorola模式 */
        .protocol     = SPI_PROTOCOL_MOTOROLA,
        /*  双线全双工  */
        .direction    = SPI_DIRECTION_2LINES,
        /* 帧长8位 */
        .frame_size   = SPI_FRAME_SIZE_8BITS,
        /*SCK低电平有效 */
        .clk_polarity = SPI_SCK_ACTIVE_LOW,
        /* SCK相位 */
        .clk_phase    = SPI_SCK_PHASE_2ND_EDGE,
        /* 软件NSS */
        .nss          = SPI_NSS_SOFT,
        /* 片选极性 */
        .nss_polarity = SPI_NSS_ACTIVE_LOW,
        /* 254分频 */
        .prescaler    = SPI_BAUDRATE_PRESCALER(254),
        /* 字节序 */
        .byte_order   = SPI_BYTE_ORDER_INTEL,
        /* 发送/接收数据时低位在前 */
        .bit_order    = SPI_LSB_FIRST,
        /* 关闭采样滤波 */
        .filter       = SPI_FILTER_DISABLE,
    };

    SPI_Init(SPI1, &spi_cfg);
    /* 使能SPI1 */
    SPI_Enable(SPI1);
    /* 使能SPI1接收中断 */
    SPI_SetIntMode(SPI1, SPI_INT_RXNE, ENABLE);
    /* 使能SPI1发送中断 */
    SPI_SetIntMode(SPI1, SPI_INT_TXE, ENABLE);
    /* 配置SPI1中断优级 */
    NVIC_SetPriority(SPI1_IRQn, 5);
    /* 使能SPI1中断 */
    NVIC_EnableIRQ(SPI1_IRQn);
}

void SPI1_IRQHandler(void)
{

    if (SPI_GetIntFlag(SPI1, SPI_FLAG_RXNE))
    {
        /* 接收数据*/
        rx_spi_buf[b] = SPI_ReceiveData(SPI1);

        SPI_ClearStatusFlag(SPI1, SPI_FLAG_RXNE);

        b++;
        if (b == 4)
        {
            b = 0;
        }
    }

    if (SPI_GetIntFlag(SPI1, SPI_FLAG_TXE))
    {
        /* 发送数据 */
        SPI_TransmitData(SPI1, tx_spi_buf[a]);

        SPI_ClearStatusFlag(SPI1, SPI_FLAG_TXE);

        a++;
        if (a == 4)
        {
            a = 0;
        }
    }
}

void USART1_IRQHandler(void)
{

    /* USART1接收数据 */
    rx_buf[c] = USART_ReceiveData(USART1);

    USART_ClearStatusFlag(USART1, USART_FLAG_RXNE);

    c++;
    if (c == 4)
    {
        c = 0;
    }
}

