/*! @file    nccm0670_btim.h
 *  @brief   BTIM固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_BTIM_H
#define NCC0670_BTIM_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_btim
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*!
 * @brief 事件生成选择
 */
typedef enum
{
    BTIM_EVENT_SRC_UG = BTIM_EGR_UG_Msk /*!< 更新生成 */
} btim_event_t;

/*!
 * @brief  BTIM中断
 */
typedef enum
{
    BTIM_INT_UIE = BTIM_IER_UIE_Msk /* 更新中断 */
} btim_interrupt_t;

/*!
 * @brief  BTIM标志
 */
typedef enum
{
    BTIM_FLAG_UIF = BTIM_SR1_UIF_Msk /* 更新标志 */
} btim_status_flag_t;

/*!
 * @brief  计数器工作模式
 */
typedef enum
{
    BTIM_COUNTER_UP,  /*!< 计数器递增计数*/
    BTIM_COUNTER_DOWN /*!< 计数器递减计数 */
} btim_counter_direction_t;

/*!
 * @brief   预分频器更新模式
 */
typedef enum
{
    BTIM_PSC_RELOAD_UPDATE,   /*!< 预分频器在下次update event事件后重新加载 */
    BTIM_PSC_RELOAD_IMMEDIATE /*!< 预分频器立即重新加载 */
} btim_psc_reload_mode_t;

/*!
 * @brief   单脉冲模式
 */
typedef enum
{
    BTIM_ONE_PULSE_MODE_SINGLE,    /*!< 单脉冲模式 */
    BTIM_ONE_PULSE_MODE_REPETITIVE /*!< 重复脉冲模式 */
} btim_one_pulse_mode_t;

/*!
 * @brief   更新请求源
 */
typedef enum
{
    BTIM_UPDATE_REQUEST_SRC_GLOBAL, /*!< 所有以下事件都会产生更新中断:
                                         - 计数器上溢/下溢
                                         - 将 UG 位置 1
                                         - 通过从模式控制器生成的更新事件 */
    BTIM_UPDATE_REQUEST_SRC_REGULAR /*!< 只有计数器上溢/下溢会生成更新中断 */
} btim_update_request_src_t;

/*!
 * @brief   定时器基本配置
 */
typedef struct
{
    uint32_t prescaler;                    /*!< 预分频系数 */
    btim_counter_direction_t counter_mode; /*!< 计数模式 */
    uint32_t period;                       /*!< 定时周期  */
} btim_config_t;

/*!
 * @brief   TRGO触发输出
 */
typedef enum
{
    BTIM_TRGO_DISABLE, /*!< 不输出 TRGO保持低电平 */
    BTIM_TRGO_RESET,   /*!< TIMx_EGR 寄存器中的 UG 位用作触发输出 */
    BTIM_TRGO_ENABLE,  /*!< 计数器使能信号 CNT_EN 用作触发输出 */
    BTIM_TRGO_UPDATE,  /*!< 选择更新事件作为触发输出 */
} btim_trigger_output_t;
/*******************************************************************************
 * API
 ******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif
void BTIM_Init(BTIM_Type *const base, const btim_config_t *const config);
void BTIM_DeInit(BTIM_Type *const base);
void BTIM_Enable(BTIM_Type *const base);
void BTIM_Disable(BTIM_Type *const base);
void BTIM_SetArrPreload(BTIM_Type *const base, bool enable);
void BTIM_SetUpdateEvent(BTIM_Type *const base, bool enable);
void BTIM_SetCounterDirection(BTIM_Type *const base, btim_counter_direction_t dir);
void BTIM_SetPrescaler(BTIM_Type *const base, uint16_t prescaler);
void BTIM_SetAutoReload(BTIM_Type *const base, uint32_t autoreload);
uint16_t BTIM_GetCounter(BTIM_Type *const base);
void BTIM_SetOnePulseMode(BTIM_Type *const base, btim_one_pulse_mode_t mode);
void BTIM_SetUpdateSource(BTIM_Type *const base, btim_update_request_src_t source);
void BTIM_GenerateEvent(BTIM_Type *const base, btim_event_t event);
bool BTIM_GetStatusFlag(BTIM_Type *const base, btim_status_flag_t flag);
void BTIM_ClearStatusFlag(BTIM_Type *const base, btim_status_flag_t flag);
void BTIM_SetIntMode(BTIM_Type *const base, btim_interrupt_t interrupt, bool enable);
bool BTIM_GetIntFlag(BTIM_Type *const base, btim_status_flag_t flag);
void BTIM_SetTriggerOutput(BTIM_Type *const base, btim_trigger_output_t timer_synchronization);

#if defined(__cplusplus)
}
#endif
/*! @} */
#endif /* NCC0670_BTIM_H*/
/*******************************************************************************
 * EOF
 ******************************************************************************/
