/*! @file    nccm0670_gpio.h
 *  @brief   GPIO固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_GPIO_H
#define NCC0670_GPIO_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_gpio
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

#define GPIO_PIN_0   0x0001U /*<! Pin 0 */
#define GPIO_PIN_1   0x0002U /*<! Pin 1 */
#define GPIO_PIN_2   0x0004U /*<! Pin 2 */
#define GPIO_PIN_3   0x0008U /*<! Pin 3 */
#define GPIO_PIN_4   0x0010U /*<! Pin 4 */
#define GPIO_PIN_5   0x0020U /*<! Pin 5 */
#define GPIO_PIN_6   0x0040U /*<! Pin 6 */
#define GPIO_PIN_7   0x0080U /*<! Pin 7 */
#define GPIO_PIN_8   0x0100U /*<! Pin 8 */
#define GPIO_PIN_9   0x0200U /*<! Pin 9 */
#define GPIO_PIN_10  0x0400U /*<! Pin 10*/
#define GPIO_PIN_11  0x0800U /*<! Pin 11*/
#define GPIO_PIN_12  0x1000U /*<! Pin 12*/
#define GPIO_PIN_13  0x2000U /*<! Pin 13*/
#define GPIO_PIN_14  0x4000U /*<! Pin 14*/
#define GPIO_PIN_15  0x8000U /*<! Pin 15*/
#define GPIO_PIN_ALL 0xFFFFU /*<! 所有 pins*/

typedef enum
{
    GPIO_OUTPUT_PUSHPULL, /*<! 推挽输出 */
    GPIO_OUTPUT_OPENDRAIN /*<! 开漏输出 */
} gpio_output_type_t;

/*!
 * @brief GPIO 模式配置
 */
typedef enum
{
    GPIO_MODE_INPUT,     /*!< 输入模式                     */
    GPIO_MODE_OUTPUT,    /*!< 通用输出模式                 */
    GPIO_MODE_ALTERNATE, /*!< 复用功能模式                 */
    GPIO_MODE_ANALOG     /*!< 模拟模式                     */
} gpio_mode_t;

/*!
 * @brief GPIO速度配置
 */
typedef enum
{
    GPIO_SPEED_FREQ_LOW,      /*<! 低速 */
    GPIO_SPEED_FREQ_MEDIUM,   /*<! 中速 */
    GPIO_SPEED_FREQ_HIGH,     /*<! 高速 */
    GPIO_SPEED_FREQ_VERY_HIGH /*<! 超高速*/
} gpio_speed_t;

/*!
 * @brief GPIO上下拉配置
 */
typedef enum
{
    GPIO_NOPULL,  /*<! 无上下拉 */
    GPIO_PULLUP,  /*<! 上拉 */
    GPIO_PULLDOWN /*<! 下拉 */
} gpio_pull_t;

/*!
 * @brief GPIO引脚复用配置
 */
typedef enum
{
    GPIO_AF0_I2C1     = 0x00, /*!< I2C1 复用引脚 */
    GPIO_AF0_MCO      = 0x00, /*!< MCO (MCO1) 复用引脚 */
    GPIO_AF0_SWJ      = 0x00, /*!< SWJ (SWD) 复用引脚 */
    GPIO_AF0_TIM10    = 0x00, /*!< TIM10 复用引脚 */
    GPIO_AF0_TIM16    = 0x00, /*!< TIM16 复用引脚 */
    GPIO_AF0_BEEP     = 0x00, /*!< BUZZER 复用引脚 */
    GPIO_AF0_USART1   = 0x00, /*!< USART1 复用引脚 */

    GPIO_AF1_IR       = 0x01, /*!< IR 复用引脚 */
    GPIO_AF1_LPUART1  = 0x01, /*!< LPUART1 复用引脚 */
    GPIO_AF1_SPI1     = 0x01, /*!< SPI1 复用引脚 */
    GPIO_AF1_I2C1     = 0x01, /*!< I2C1 复用引脚 */
    GPIO_AF1_TIM2     = 0x01, /*!< TIM2 复用引脚 */
    GPIO_AF1_USART1   = 0x01, /*!< USART1 复用引脚 */

    GPIO_AF2_LED      = 0x02  /*!< LED 复用引脚 */
} gpio_alt_t;

/*!
 * @brief GPIO配置
 */
typedef struct
{
    uint32_t pin;                   /*!< 引脚 */
    gpio_mode_t mode;               /*!< 模式 */
    gpio_speed_t speed;             /*!< 翻转速度 */
    gpio_output_type_t output_type; /*!< 输出类型 */
    gpio_pull_t pull;               /*!< 上下拉 */
    gpio_alt_t alternate;           /*!< 引脚复用 */
} gpio_config_t;

/*******************************************************************************
 * API
 ******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif

void GPIO_Init(GPIO_Type *const base, const gpio_config_t *const config);
void GPIO_DeInit(GPIO_Type *const base);
void GPIO_SetPinMode(GPIO_Type *const base, gpio_mode_t mode, gpio_pull_t pull_up_down, uint32_t pin);
void GPIO_SetPinOutput(GPIO_Type *const base, gpio_output_type_t otype, gpio_speed_t speed, uint32_t pin);
void GPIO_SetPinAf(GPIO_Type *const base, gpio_alt_t alternate, uint32_t pin);
void GPIO_SetPins(GPIO_Type *const base, uint32_t pin);
void GPIO_ClearPins(GPIO_Type *const base, uint32_t pin);
void GPIO_WritePins(GPIO_Type *const base, uint32_t pin, bool val);
void GPIO_WritePort(GPIO_Type *const base, uint16_t val);
bool GPIO_ReadPins(const GPIO_Type *const base, uint32_t pin);
uint32_t GPIO_ReadPort(const GPIO_Type *const base);
void GPIO_TogglePin(GPIO_Type *const base, uint32_t pin);
void GPIO_LockPin(GPIO_Type *const base, uint32_t pin);

#if defined(__cplusplus)
}
#endif
/*! @} */
#endif /* NCC0670_GPIO_H*/
/*******************************************************************************
 * EOF
 ******************************************************************************/
