/*! @file    nccm0670_gtim.h
 *  @brief   GTIM固件库
 *  @version 2021-04-02  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_GTIM_H
#define NCC0670_GTIM_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_gtim
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*!
 * @brief   事件生成选择
 */
typedef enum
{
    GTIM_EVENT_SRC_UG   = (uint16_t)(0x0001U), /*!< 更新生成 */
    GTIM_EVENT_SRC_TG   = (uint16_t)(0x0002U), /*!< 触发生成 */
    GTIM_EVENT_SRC_CH1G = (uint16_t)(0x0010U), /*!< 捕获/比较 1 生成 */
    GTIM_EVENT_SRC_CH2G = (uint16_t)(0x0020U), /*!< 捕获/比较 2 生成 */
    GTIM_EVENT_SRC_CH3G = (uint16_t)(0x0040U), /*!< 捕获/比较 3 生成 */
    GTIM_EVENT_SRC_CH4G = (uint16_t)(0x0080U), /*!< 捕获/比较 4 生成 */
} gtim_event_t;

/*!
 * @brief   中断标志
 */
typedef enum
{
    GTIM_INT_UIE  = GTIM_IER_UIE_Msk,   /* 更新中断 */
    GTIM_INT_TRG = GTIM_IER_TIE_Msk,   /* 触发中断 */
    GTIM_INT_CC1 = GTIM_IER_CC1IE_Msk, /* 捕获/比较 1 中断 */
    GTIM_INT_CC2 = GTIM_IER_CC2IE_Msk, /* 捕获/比较 2 中断 */
    GTIM_INT_CC3 = GTIM_IER_CC3IE_Msk, /* 捕获/比较 3 中断 */
    GTIM_INT_CC4 = GTIM_IER_CC4IE_Msk, /* 捕获/比较 4 中断 */
} gtim_interrupt_t;

/*!
 * @brief   状态标志
 */
typedef enum
{
    GTIM_FLAG_UIF    = GTIM_SR1_UIF_Msk,             /* 更新中断标志 */
    GTIM_FLAG_TRG   = GTIM_SR1_TIF_Msk,             /* 触发中断标志 */
    GTIM_FLAG_CC1   = GTIM_SR1_CC1IF_Msk,           /* 捕获/比较 1 中断标志 */
    GTIM_FLAG_CC2   = GTIM_SR1_CC2IF_Msk,           /* 捕获/比较 2 中断标志 */
    GTIM_FLAG_CC3   = GTIM_SR1_CC3IF_Msk,           /* 捕获/比较 3 中断标志 */
    GTIM_FLAG_CC4   = GTIM_SR1_CC4IF_Msk,           /* 捕获/比较 4 中断标志 */
    GTIM_FLAG_CC1OF = (GTIM_SR2_CC1OF_Msk << 16UL), /* 捕获/比较 1 重复捕获标志 */
    GTIM_FLAG_CC2OF = (GTIM_SR2_CC2OF_Msk << 16UL), /* 捕获/比较 2 重复捕获标志 */
    GTIM_FLAG_CC3OF = (GTIM_SR2_CC3OF_Msk << 16UL), /* 捕获/比较 3 重复捕获标志 */
    GTIM_FLAG_CC4OF = (GTIM_SR2_CC4OF_Msk << 16UL), /* 捕获/比较 4 重复捕获标志 */
} gtim_status_flag_t;

/*!
 * @brief   定时器时钟 (CK_INT) 频率与数字滤波器（ ETR、 TIx）所使用的采样时钟之间的分频比
 */
typedef enum
{
    GTIM_CLOCK_DIVISION_DIV1, /*!< tDTS=tCK_INT */
    GTIM_CLOCK_DIVISION_DIV2, /*!< tDTS=2*tCK_INT */
    GTIM_CLOCK_DIVISION_DIV4, /*!< tDTS=4*tCK_INT */
} gtim_clock_division_t;

/*!
 * @brief  计数器工作模式
 */
typedef enum
{
    GTIM_COUNTER_UP,  /*!< 计数器递增计数*/
    GTIM_COUNTER_DOWN /*!< 计数器递减计数 */
} gtim_counter_direction_t;

/*!
 * @brief   预分频器更新模式
 */
typedef enum
{
    GTIM_PSC_RELOAD_UPDATE,   /*!< 预分频器在下次update event事件后重新加载 */
    GTIM_PSC_RELOAD_IMMEDIATE /*!< 预分频器立即重新加载 */
} gtim_psc_reload_mode_t;

/*!
 * @brief   单脉冲模式
 */
typedef enum
{
    GTIM_ONE_PULSE_MODE_SINGLE,    /*!< 单脉冲模式 */
    GTIM_ONE_PULSE_MODE_REPETITIVE /*!< 重复脉冲模式 */
} gtim_one_pulse_mode_t;

/*!
 * @brief   更新请求源
 */
typedef enum
{
    GTIM_UPDATE_REQUEST_SRC_GLOBAL, /*!< 所有以下事件都会产生更新中断:
                                         - 计数器上溢/下溢
                                         - 将 UG 位置 1
                                         - 通过从模式控制器生成的更新事件 */
    GTIM_UPDATE_REQUEST_SRC_REGULAR /*!< 只有计数器上溢/下溢会生成更新中断 */
} gtim_update_request_src_t;

/*!
 * @brief   定时器通道
 * @note 根据定时器实际支持的通道数选择
 */
typedef enum
{
    GTIM_CH_1, /*!< 通道 1 */
    GTIM_CH_2, /*!< 通道 2 */
    GTIM_CH_3, /*!< 通道 3 */
    GTIM_CH_4, /*!< 通道 4 */
} gtim_oc_ic_channel_t;

/*!
 * @brief   输出比较模式
 */
typedef enum
{
    GTIM_OC_MODE_FROZEN,      /*!<冻结*/
    GTIM_OC_MODE_ACTIVE,      /*!<匹配时输出有效电平*/
    GTIM_OC_MODE_INACTIVE,    /*!<匹配时输出无效电平*/
    GTIM_OC_MODE_TOGGLE,      /*!<翻转*/
    GTIM_OC_MODE_FORCED_LOW,  /*!<强制变为无效电平*/
    GTIM_OC_MODE_FORCED_HIGH, /*!<强制变为有效电平*/
    GTIM_OC_MODE_PWM1,        /*!<PWM模式1*/
    GTIM_OC_MODE_PWM2,        /*!<PWM模式2*/
} gtim_oc_mode_t;

/*!
 * @brief   输出比较极性选择
 */
typedef enum
{
    GTIM_OC_POLARITY_HIGH, /*!<高电平有效*/
    GTIM_OC_POLARITY_LOW   /*!<低电平有效*/
} gtim_oc_polarity_t;

/*!
 * @brief   通道输出使能状态
 */
typedef enum
{
    GTIM_OC_OUTPUT_ENABLE, /*!<通道输出使能*/
    GTIM_OC_OUTPUT_DISABLE /*!<禁止通道输出*/
} gtim_oc_output_enable_t;

/*!
 * @brief   定时器基本配置
 */
typedef struct
{
    uint32_t prescaler;                    /*!< 预分频系数 = prescaler + 1 */
    uint32_t period;                       /*!< 定时周期  */
    gtim_counter_direction_t counter_mode; /*!< 计数模式 */
    gtim_clock_division_t
        clock_division; /*!< 定时器时钟 (CK_INT) 频率与数字滤波器（ ETR、 TIx）所使用的采样时钟之间的分频比 */
} gtim_base_config_t;

/*!
 * @brief   OC输出比较配置
 */
typedef struct
{
    gtim_oc_output_enable_t state;  /*!< 输出使能*/
    gtim_oc_output_enable_t nstate; /*!< 互补输出使能*/
    gtim_oc_polarity_t polarity;    /*!< 极性*/
    gtim_oc_polarity_t npolarity;   /*!< 互补输出极性*/
} gtim_oc_config_t;

typedef enum
{
    GTIM_IC_SELECTION_DIRECTTI = 2U, /*!< ICx映射到TIx上*/
    GTIM_IC_SELECTION_XOR /*!< ICx映射到 TXOR上，TXOR= TI ^ T2 ^ T3，TI,T2,T3异或之后产生的信号*/
} gtim_ic_selection_t;

/*!
 * @brief   IC输入捕获触发边沿
 */
typedef enum
{
    GTIM_IC_POLARITY_RISING,  /*!< 非反相/上升沿触发 */
    GTIM_IC_POLARITY_FALLING, /*!< 反相/下降沿触发 */
    GTIM_IC_POLARITY_BOTHEDGE /*!< 非反相/双边沿触发 */
} gtim_ic_polarity_t;

/*!
 * @brief   IC输入捕获输入滤波
 */
typedef enum
{
    GTIM_IC_FILTER_FDIV1,     /* 无滤波,按 fDTS 频率进行采样 */
    GTIM_IC_FILTER_FDIV1_N2,  /*!< fSAMPLING=fCK_INT, N=2 */
    GTIM_IC_FILTER_FDIV1_N4,  /*!< fSAMPLING=fCK_INT, N=4 */
    GTIM_IC_FILTER_FDIV1_N8,  /*!< fSAMPLING=fCK_INT, N=8 */
    GTIM_IC_FILTER_FDIV2_N6,  /*!< fSAMPLING=fDTS/2, N=6 */
    GTIM_IC_FILTER_FDIV2_N8,  /*!< fSAMPLING=fDTS/2, N=8 */
    GTIM_IC_FILTER_FDIV4_N6,  /*!< fSAMPLING=fDTS/4, N=6 */
    GTIM_IC_FILTER_FDIV4_N8,  /*!< fSAMPLING=fDTS/4, N=8 */
    GTIM_IC_FILTER_FDIV8_N6,  /*!< fSAMPLING=fDTS/8, N=6 */
    GTIM_IC_FILTER_FDIV8_N8,  /*!< fSAMPLING=fDTS/8, N=8 */
    GTIM_IC_FILTER_FDIV16_N5, /*!< fSAMPLING=fDTS/16, N=5 */
    GTIM_IC_FILTER_FDIV16_N6, /*!< fSAMPLING=fDTS/16, N=6 */
    GTIM_IC_FILTER_FDIV16_N8, /*!< fSAMPLING=fDTS/16, N=8 */
    GTIM_IC_FILTER_FDIV32_N5, /*!< fSAMPLING=fDTS/32, N=5 */
    GTIM_IC_FILTER_FDIV32_N6, /*!< fSAMPLING=fDTS/32, N=6 */
    GTIM_IC_FILTER_FDIV32_N8  /*!< fSAMPLING=fDTS/32, N=8 */
} gtim_ic_filter_t;

/*!
 * @brief   IC输入捕获输入滤波
 */
typedef enum
{
    GTIM_IC_PSC_DIV1, /*!< 无预分频器，捕获输入上每检测到一个边沿便执行捕获 */
    GTIM_IC_PSC_DIV2, /*!< 每发生 2 个事件便执行一次捕获 */
    GTIM_IC_PSC_DIV4, /*!< 每发生 4 个事件便执行一次捕获 */
    GTIM_IC_PSC_DIV8  /*!< 每发生 8 个事件便执行一次捕获 */
} gtim_ic_prescaler_t;

/*!
 * @brief   IC输入捕获配置
 */
typedef struct
{
    gtim_ic_polarity_t polarity;   /*!< 输入触发边沿 */
    gtim_ic_selection_t selection; /*!< 输入选择 */
    gtim_ic_prescaler_t prescaler; /*!< 预分频系数 */
    gtim_ic_filter_t filter;       /*!< 滤波器 */
} gtim_ic_config_t;

/*!
 * @brief   TRGO触发输出
 */
typedef enum
{
    GTIM_TRGO_DISABLE, /*!< 不输出 TRGO保持低电平 */
    GTIM_TRGO_RESET,   /*!< TIMx_EGR 寄存器中的 UG 位用作触发输出 */
    GTIM_TRGO_ENABLE,  /*!< 计数器使能信号 CNT_EN 用作触发输出 */
    GTIM_TRGO_UPDATE,  /*!< 选择更新事件作为触发输出 */
    GTIM_TRGO_CC1IF  = GTIM_CR2_CHSEL(0) | 0x04, /*!< 一旦某通道发生输入捕获或比较匹配事件，触发输出都会发送一个正脉冲 */
    GTIM_TRGO_OC1REF = GTIM_CR2_CHSEL(0) | 0x05, /*!< 某通道的比较输出参考信号 OCxREF用作触发输出 */
    GTIM_TRGO_CC2IF  = GTIM_CR2_CHSEL(1) | 0x04, /*!< 一旦某通道发生输入捕获或比较匹配事件，触发输出都会发送一个正脉冲 */
    GTIM_TRGO_OC2REF = GTIM_CR2_CHSEL(1) | 0x05, /*!< 某通道的比较输出参考信号 OCxREF用作触发输出 */
    GTIM_TRGO_CC3IF  = GTIM_CR2_CHSEL(2) | 0x04, /*!< 一旦某通道发生输入捕获或比较匹配事件，触发输出都会发送一个正脉冲 */
    GTIM_TRGO_OC3REF = GTIM_CR2_CHSEL(2) | 0x05, /*!< 某通道的比较输出参考信号 OCxREF用作触发输出 */
    GTIM_TRGO_CC4IF  = GTIM_CR2_CHSEL(3) | 0x04, /*!< 一旦某通道发生输入捕获或比较匹配事件，触发输出都会发送一个正脉冲 */
    GTIM_TRGO_OC4REF = GTIM_CR2_CHSEL(3) | 0x05, /*!< 某通道的比较输出参考信号 OCxREF用作触发输出 */
} gtim_trigger_output_t;

/*!
 * @brief   TRGO同步使能
 */
typedef enum
{
    GTIM_TRGO_SYNC_DISABLE, /*<不执行任何操作，
                           当前定时器与其从定时器至少有2个定时器时钟的延迟（若从定时器TS=0000，则有2个时钟的延迟；
                           若从定时器TS=0001，则有4个时钟的延迟）>*/
    GTIM_TRGO_SYNC_ENABLE /*<当前定时器的使能或复位动作被延迟，以使当前定时器与其从定时器实现完美同步（通过TRGO）的情况>*/
} gtim_trgo_sync_enable;

/*!
 * @brief   TRGI触发输入
 */
typedef enum
{
    GTIM_TRGI_NON_SYNC_ITR, /* 不经过同步逻辑的内部触发，上升沿有效 */
    GTIM_TRGI_SYNC_ITR,     /* 经过同步逻辑的内部触发，上升沿有效 */
    GTIM_TRGI_ETRF,         /* 滤波后的外部触发，触发极性由ETP控制 */
    GTIM_TRGI_CH1FP,        /* 滤波后的定时器输入 1 */
    GTIM_TRGI_CH2FP,        /* 滤波后的定时器输入 2 */
    GTIM_TRGI_CH3FP,        /* 滤波后的定时器输入 3 */
    GTIM_TRGI_CH4FP,        /* 滤波后的定时器输入 4 */
} gtim_trigger_input_t;

/*!
 * @brief   从模式选择
 */
typedef enum
{
    GTIM_SLAVE_MODE_DISABLED, /*!< 禁止从模式 */
    GTIM_SLAVE_MODE_RESET, /*!< 复位模式 - 在出现所选触发输入的有效边沿时，重新初始化计数器并生成一个寄存器更新事件 */
    GTIM_SLAVE_MODE_GATED, /*!< 门控模式 - 触发输入 (TRGI)
                              为有效电平时使能计数器时钟。只要触发输入变为无效电平，计数器立即停止计数（但不复位）。计数器的启动和停止都被控制
                            */
    GTIM_SLAVE_MODE_TRIGGER,              /*!< 触发模式 - 触发信号 TRGI 出现有效沿时启动计数器 */
    GTIM_SLAVE_MODE_TRG_ONE_PULSE_MODE_1, /*!< 触发单脉冲模式 1 -
                                              可再触发单脉冲模式,如果在上一个触发完成前发生新的触发，脉冲将延长 */
    GTIM_SLAVE_MODE_TRG_ONE_PULSE_MODE_2  /*!< 触发单脉冲模式 2 - 在触发完成前不响应新的触发 */
} gtim_slave_mode_t;

/*!
 * @brief   外部触发极性
 */
typedef enum
{
    GTIM_ETR_POLARITY_RISING,        /* 高电平或上升沿有效 */
    GTIM_ETR_POLARITY_FALLING,       /* 低电平或下升沿有效 */
    GTIM_ETR_POLARITY_BOTH_EDGE = 3, /* 上升沿或下降沿均有效 */
} gtim_etr_polarity_t;

/**
 * @brief 外部触发滤波器
 * 数字滤波器由事件计数器组成，每 N 个连续事件才视为一个有效输出边沿
 */
typedef enum
{
    GTIM_ETR_FILTER_FDIV1,     /* 无滤波,按 fDTS 频率进行采样 */
    GTIM_ETR_FILTER_FDIV1_N2,  /* fSAMPLING=fCK_INT，N=2 */
    GTIM_ETR_FILTER_FDIV1_N4,  /* fSAMPLING=fCK_INT，N=4 */
    GTIM_ETR_FILTER_FDIV1_N8,  /* fSAMPLING=fCK_INT，N=8 */
    GTIM_ETR_FILTER_FDIV2_N6,  /* fSAMPLING=fCK_DTS/2，N=6 */
    GTIM_ETR_FILTER_FDIV2_N8,  /* fSAMPLING=fCK_DTS/2，N=8 */
    GTIM_ETR_FILTER_FDIV4_N6,  /* fSAMPLING=fCK_DTS/4，N=6 */
    GTIM_ETR_FILTER_FDIV4_N8,  /* fSAMPLING=fCK_DTS/4，N=8 */
    GTIM_ETR_FILTER_FDIV8_N6,  /* fSAMPLING=fCK_DTS/8，N=6 */
    GTIM_ETR_FILTER_FDIV8_N8,  /* fSAMPLING=fCK_DTS/8，N=8 */
    GTIM_ETR_FILTER_FDIV16_N5, /* fSAMPLING=fCK_DTS/16，N=5 */
    GTIM_ETR_FILTER_FDIV16_N6, /* fSAMPLING=fCK_DTS/16，N=6 */
    GTIM_ETR_FILTER_FDIV16_N8, /* fSAMPLING=fCK_DTS/16，N=8 */
    GTIM_ETR_FILTER_FDIV32_N5, /* fSAMPLING=fCK_DTS/32，N=5 */
    GTIM_ETR_FILTER_FDIV32_N6, /* fSAMPLING=fCK_DTS/32，N=6 */
    GTIM_ETR_FILTER_FDIV32_N8  /* fSAMPLING=fCK_DTS/32，N=8 */
} gtim_etr_filter_t;

/*******************************************************************************
 * API
 ******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif
/* GTIM基本定时 API */
void GTIM_Init(GTIM_Type *const base, const gtim_base_config_t *const config);
void GTIM_DeInit(GTIM_Type *const base);
void GTIM_Enable(GTIM_Type *const base);
void GTIM_Disable(GTIM_Type *const base);
void GTIM_SetArrPreload(GTIM_Type *const base, bool enable);
void GTIM_SetUpdateEvent(GTIM_Type *const base, bool enable);
void GTIM_SetCounterDirection(GTIM_Type *const base, gtim_counter_direction_t dir);
void GTIM_SetPrescaler(GTIM_Type *const base, uint16_t prescaler);
void GTIM_SetAutoReload(GTIM_Type *const base, uint16_t autoreload);
uint32_t GTIM_GetCounter(GTIM_Type *const base);
void GTIM_SetOnePulseMode(GTIM_Type *const base, gtim_one_pulse_mode_t mode);
void GTIM_SetUpdateSource(GTIM_Type *const base, gtim_update_request_src_t source);
void GTIM_GenerateEvent(GTIM_Type *const base, gtim_event_t event);

/* OC输出比较 API */
void GTIM_OC_Init(GTIM_Type *const base, gtim_oc_ic_channel_t channel, const gtim_oc_config_t *occfg);

void GTIM_OC_SetMode(GTIM_Type *const base, gtim_oc_ic_channel_t channel, gtim_oc_mode_t ocmode);

void GTIM_OC_SetPolarity(GTIM_Type *const base, gtim_oc_ic_channel_t channel, gtim_oc_polarity_t ocpolarity);
void GTIM_OC_SetComplementaryPolarity(GTIM_Type *const base,
                                      gtim_oc_ic_channel_t channel,
                                      gtim_oc_polarity_t ocpolarity);
void GTIM_OC_SetFastMode(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable);
void GTIM_OC_SetPreload(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable);
void GTIM_OC_SetCompareValue(GTIM_Type *const base, gtim_oc_ic_channel_t channel, uint16_t value);
void GTIM_OC_SetComplementaryChannel(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable);
void GTIM_OC_SetChannel(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable);
void GTIM_OC_SetDeadTimeMode(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable);
void GTIM_OC_SetDeadTime(GTIM_Type *const base, gtim_oc_ic_channel_t channel, uint8_t revdt, uint8_t obvdt);

/* IC输入捕获 API */
void GTIM_IC_Init(GTIM_Type *const base, gtim_oc_ic_channel_t channel, gtim_ic_config_t *iccfg);
void GTIM_IC_SetPrescaler(GTIM_Type *const base, gtim_oc_ic_channel_t channel, gtim_ic_prescaler_t prescaler);
uint32_t GTIM_IC_GetCaptureValue(GTIM_Type *const base, gtim_oc_ic_channel_t channel);

/* 主从模式 API */
void GTIM_SetSlaveMode(GTIM_Type *const base, gtim_slave_mode_t mode);
void GTIM_SetTriggerInput(GTIM_Type *const base, gtim_trigger_input_t trigger_input);
void GTIM_SetTriggerOutput(GTIM_Type *const base,
                           gtim_trigger_output_t timer_synchronization,
                           gtim_trgo_sync_enable trgo_sync);
void GTIM_ConfigExternalTrigger(GTIM_Type *const base, gtim_etr_polarity_t polarity, gtim_etr_filter_t filter);

/* 状态与中断 API */
void GTIM_SetIntMode(GTIM_Type *const base, gtim_interrupt_t interrupt, bool enable);
bool GTIM_GetIntFlag(GTIM_Type *const base, gtim_status_flag_t flag);
bool GTIM_GetStatusFlag(GTIM_Type *const base, gtim_status_flag_t flag);
void GTIM_ClearStatusFlag(GTIM_Type *const base, gtim_status_flag_t flag);

/*!@}*/
#if defined(__cplusplus)
}
#endif
/*! @} */
#endif /* NCC0670_GTIM_H*/
/*******************************************************************************
 * EOF
 ******************************************************************************/
