/*! @file    nccm0670_i2c.h
 *  @brief   I2C固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_I2C_H
#define NCC0670_I2C_H
#include "nccm0670.h"

/*!
 * @addtogroup nccm0670_i2c
 * @{
 */
/*******************************************************************************
 * Definitions
 ******************************************************************************/

/**
 * @brief  I2C中断使能标志位
 */

typedef enum
{
    I2C_INT_TXIE   = I2C_CR1_TXIE_Msk,   /*!< 发送中断 */
    I2C_INT_RXIE   = I2C_CR1_RXIE_Msk,   /*!< 接收中断 */
    I2C_INT_ADDRIE = I2C_CR1_ADDRIE_Msk, /*!< 地址匹配中断 */
    I2C_INT_NACKIE = I2C_CR1_NACKIE_Msk, /*!< NACK中断 */
    I2C_INT_STOPIE = I2C_CR1_STOPIE_Msk, /*!< 停止位检测中断 */
    I2C_INT_TCIE   = I2C_CR1_TCIE_Msk,   /*!< 传输完成中断 */
    I2C_INT_ERRIE  = I2C_CR1_ERRIE_Msk   /*!< 错误中断 */
} i2c_interrupt_t;

/**
 * @brief  I2C状态标志位
 */
typedef enum
{
    I2C_FLAG_BUSY    = I2C_ISR_BUSY_Msk,    /*!< 总线繁忙 */
    I2C_FLAG_ALERT   = I2C_ISR_ALERT_Msk,   /*!< SMBus 报警 (SMBus mode) */
    I2C_FLAG_TIMEOUT = I2C_ISR_TIMEOUT_Msk, /*!< 超时检测标识 (SMBus mode) */
    I2C_FLAG_PECERR  = I2C_ISR_PECERR_Msk,  /*!< PEC错误 (SMBus mode) */
    I2C_FLAG_OVR     = I2C_ISR_OVR_Msk,     /*!< 是否过载(slave mode) */
    I2C_FLAG_ARLO    = I2C_ISR_ARLO_Msk,    /*!< 仲裁丢失 */
    I2C_FLAG_BERR    = I2C_ISR_BERR_Msk,    /*!< 总线错误 */
    I2C_FLAG_TCR     = I2C_ISR_TCR_Msk,     /*!< 传输完成，重载模式 */
    I2C_FLAG_TC      = I2C_ISR_TC_Msk,      /*!< 传输完成 (master mode) */
    I2C_FLAG_STOP    = I2C_ISR_STOPF_Msk,   /*!< STOP条件检测标识 */
    I2C_FLAG_NACK    = I2C_ISR_NACKF_Msk,   /*!< 不响应接收 */
    I2C_FLAG_ADDR    = I2C_ISR_ADDR_Msk,    /*!< 地址匹配 */
    I2C_FLAG_RXNE    = I2C_ISR_RXNE_Msk,    /*!< 接收数据空 */
    I2C_FLAG_TXIS    = I2C_ISR_TXIS_Msk,    /*!< 发送中断状态 */
    I2C_FLAG_TXE     = I2C_ISR_TXE_Msk      /*!< 发送数据空 */
} i2c_status_flag_t;

/**
 * @brief  I2C外设模式
 */
typedef enum
{
    I2C_MODE_I2C              = 0x00000000U,        /*!< I2C 主机或从机模式 */
    I2C_MODE_SMBUS_HOST       = I2C_CR1_SMBHEN_Msk, /*!< SMBus主模式 */
    I2C_MODE_SMBUS_DEVICE     = 0x00000000U,        /*!< SMBus从模式， 默认地址不响应 */
    I2C_MODE_SMBUS_DEVICE_ARP = I2C_CR1_SMBDEN_Msk  /*!< SMBus从模式， 默认地址响应 */
} i2c_mode_t;

/**
 * @brief  模拟滤波器选择
 */
typedef enum
{
    I2C_ANALOGFILTER_ENABLE, /*!< 使能模拟波波器. */
    I2C_ANALOGFILTER_DISABLE /*!< 禁止模拟波波器. */
} i2c_analogfilter_mode_t;

/**
 *  @brief Master 寻址模式
 */
typedef enum
{
    I2C_ADDRESSING_MODE_7BIT, /*!< 主机工作在7位寻址模式. */
    I2C_ADDRESSING_MODE_10BIT  /*!< 主机工作在10位寻址模式. */
} i2c_addressing_mode_t;

/**
 * @brief 双地址模式
 */
typedef enum
{
    I2C_DUAL_ADDRESS_DISABLE, /*!< 禁止双地址模式. */
    I2C_DUAL_ADDRESS_ENABLE   /*!< 使能双地址模式. */
} i2c_dual_addressing_mode_t;

/**
 * @brief 设备自身地址 2 屏蔽位
 */
typedef enum
{
    I2C_OA2_NOMASK, /*!< 比较地址2的所有位 */
    I2C_OA2_MASK01, /*!< 只比较地址2的 第7-2位 */
    I2C_OA2_MASK02, /*!< 只比较地址2的 第7-3位 */
    I2C_OA2_MASK03, /*!< 只比较地址2的 第7-4位 */
    I2C_OA2_MASK04, /*!< 只比较地址2的 第7-5位 */
    I2C_OA2_MASK05, /*!< 只比较地址2的 第7-6位 */
    I2C_OA2_MASK06, /*!< 只比较地址2的 第7位 */
    I2C_OA2_MASK07  /*!< 所有地址都接收 */
} i2c_own_address2_masks_t;

/**
 * @brief 广播呼叫模式
 */
typedef enum
{
    I2C_GENERAL_CALL_DISABLE, /*!< 禁止广播 */
    I2C_GENERAL_CALL_ENABLE   /*!< 使能广播 */
} i2c_general_call_mode_t;

/**
 * @brief 时钟延时
 */
typedef enum
{
    I2C_NO_STRETCH_DISABLE, /*!< 使能时钟延时 */
    I2C_NO_STRETCH_ENABLE   /*!< 禁止时钟延时 */
} i2c_no_stretch_mode_t;

/**
 * @brief 响应位生成
 */
typedef enum
{
    I2C_ACK, /*!< 当前字节传输后产生ACK. */
    I2C_NACK /*!< 当前地址传输后产生NACK.*/
} i2c_ack_mode_t;

/**
 * @brief 传输方向
 */
typedef enum
{
    I2C_REQUEST_WRITE, /*!< 主机写请求. */
    I2C_REQUEST_READ   /*!< 主机读请求.  */
} i2c_request_mode_t;

/**
 * @brief 传输停止模式
 */
typedef enum
{
    I2C_MODE_SOFTEND = 0x00000000U,         /*!<使能I2C不带PEC硬件比较的软件停止模式.   */
    I2C_MODE_AUTOEND = I2C_CR2_AUTOEND_Msk, /*!<使能I2C不带PEC硬件比较的自动停止模式.  */
    I2C_MODE_RELOAD  = I2C_CR2_RELOAD_Msk,
    I2C_SMBUS_SENDPEC_MODE = I2C_CR2_PECBYTE_Msk,
} i2c_autoend_mode_t;

/**
 * @brief SMBUS PEC 模式
 */
typedef enum
{
    I2C_MODE_SMBUS_RELOAD         = I2C_CR2_RELOAD_Msk,  /*!< 使能SMBUS带PEC硬件比较的重载模式.   */
    I2C_MODE_SMBUS_AUTOEND_NO_PEC = I2C_CR2_AUTOEND_Msk, /*!< 使能SMBUS不带PEC硬件比较的自动停止模式. */
    I2C_MODE_SMBUS_SOFTEND_NO_PEC = 0x00000000U,         /*!< 使能SMBUS不带PEC硬件比较的软件停止模式. */
    I2C_MODE_SMBUS_PECBYTE        = I2C_CR2_PECBYTE_Msk, /*!<使能SMBUS带PEC硬件比较的软件停止模式.   */
    I2C_MODE_SMBUS_AUTOEND_WITH_PEC =
        ((uint32_t)(I2C_CR2_AUTOEND_Msk | I2C_CR2_PECBYTE_Msk)), /*!< 使能SMBUS带PEC硬件比较的自动停止模式.    */
    I2C_MODE_SMBUS_SOFTEND_WITH_PEC = ((uint32_t)(I2C_CR2_PECBYTE_Msk)) /*!< 使能SMBUS带PEC硬件比较的软件停止模式.    */
} i2c_smbus_pecmode_t;

/**
 * @brief  起始和停止位
 */
typedef enum
{
    I2C_GENERATE_NOSTARTSTOP = 0x00000000U,      /*!< 不产生起始和停止位     */
    I2C_GENERATE_STOP        = I2C_CR2_STOP_Msk, /*!< 产生停止位             */
    I2C_GENERATE_START_READ =
        ((uint32_t)(0x80000000U | I2C_CR2_START_Msk | I2C_CR2_RD_WRN_Msk)),   /*!< 产生读请求和起始条件位. */
    I2C_GENERATE_START_WRITE = ((uint32_t)(0x80000000U | I2C_CR2_START_Msk)), /*!<产生写请求和起始条件位. */
    I2C_GENERATE_RESTART_7BIT_READ = ((uint32_t)(0x80000000U | I2C_CR2_START_Msk |
                                                 I2C_CR2_RD_WRN_Msk)), /*!< 产生读请求的重复起始条件位, 从机7位地址.*/
    I2C_GENERATE_RESTART_7BIT_WRITE =
        ((uint32_t)(0x80000000U | I2C_CR2_START_Msk)), /*!< 产生写请求的重复起始条件位, 从机7位地址. */
    I2C_GENERATE_RESTART_10BIT_READ =
        ((uint32_t)(0x80000000U | I2C_CR2_START_Msk | I2C_CR2_RD_WRN_Msk |
                    I2C_CR2_HEAD10R_Msk)), /*!< 产生读请求的重复起始条件位, 从机10位地址. */
    I2C_GENERATE_RESTART_10BIT_WRITE =
        ((uint32_t)(0x80000000U | I2C_CR2_START_Msk |
                    I2C_CR2_HEAD10R_Msk)) /*!< 产生写请求的重复起始条件位, 从机10位地址.*/
} i2c_startstop_mode_t;

/**
 * @brief  读写方向
 */
typedef enum
{
    I2C_DIRECTION_WRITE, /*!< 主机写传输请求, 从机进入接收模式.*/
    I2C_DIRECTION_READ   /*!< 主机读请求, 从机进入发送模式.*/
} i2c_directionflag_t;
/**
 * @brief  SMBus TimeoutA 模式 SCL SDA 超时
 */
typedef enum
{
    I2C_SMBUS_TIMEOUTA_MODE_SCL_LOW,     /*!<使用TimeoutA检测SCL低电平超时. */
    I2C_SMBUS_TIMEOUTA_MODE_SDA_SCL_HIGH /*!< 使用TimeoutA 检测SCL 和SDA高电平超时*/
} i2c_smbus_timeouta_mode_t;

/**
 * @brief  SMBus 超时选择
 */

typedef enum
{
    I2C_SMBUS_TIMEOUTA,   /*!< TimeoutA 使能位                       */
    I2C_SMBUS_TIMEOUTB,   /*!< TimeoutB (扩展时钟) 使能位 */
    I2C_SMBUS_ALL_TIMEOUT /*!< TimeoutA 和 TimeoutB (扩展时钟) 使能位 */
} i2c_smbus_timout_config_t;

/*!
 * @brief   I2C配置结构体
 */
typedef struct
{
    uint32_t timing;                              /*!< I2C TIMINGR寄存器配置 */
    uint32_t own_address1;                        /*!< 第一个设备地址 */
    i2c_addressing_mode_t addressing_mode;        /*!< 设备地址1长度(7位或10位) */
    i2c_dual_addressing_mode_t dual_address_mode; /* 双地址模式 */
    uint32_t own_address2;                        /*!< 第二个设备地址(7位) */
    i2c_own_address2_masks_t own_address2_masks;  /*!< 第二个设备地址屏蔽掩码 */
    i2c_general_call_mode_t general_call_mode;    /*!< 广播模式 */
    i2c_no_stretch_mode_t no_stretch_mode;        /*!< 时钟延长禁止 */
} i2c_config_t;

/**
  * @brief  配置SDA的建立时间，保持时间和SCL高低周期.
  * @param  PRESCALER 参数范围  0 ~ 0xF.
  * @param  SETUP_TIME 参数范围  0 ~ 0xF.
                         (tscldel = (SCLDEL+1)xtpresc)
  * @param  HOLD_TIME  参数范围  0 ~ 0xF.
                         (tsdadel = SDADELxtpresc)
  * @param  SCLH_PERIOD 参数范围  0 ~ 0xFF.
                          (tsclh = (SCLH+1)xtpresc)
  * @param  SCLL_PERIOD 参数范围  0 ~ 0xFF.
                            (tscll = (SCLL+1)xtpresc)
  */
#define I2C_CONVERT_TIMINGS(PRESCALER, DATA_SETUP_TIME, DATA_HOLD_TIME, CLOCK_HIGH_PERIOD, CLOCK_LOW_PERIOD)           \
    I2C_TIMINGR_PRESC(PRESCALER) | I2C_TIMINGR_SCLDEL(DATA_SETUP_TIME) | I2C_TIMINGR_SDADEL(DATA_HOLD_TIME) |          \
        I2C_TIMINGR_SCLH(CLOCK_HIGH_PERIOD) | I2C_TIMINGR_SCLL(CLOCK_LOW_PERIOD)

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif
/* I2C 通用 API */
void I2C_Enable(I2C_Type *const base);
void I2C_Disable(I2C_Type *const base);
bool I2C_IsEnabled(I2C_Type *const base);
void I2C_SetMode(I2C_Type *const base, i2c_mode_t peripheralmode);
i2c_mode_t I2C_GetMode(I2C_Type *const base);
void I2C_HandleTransfer(I2C_Type *const base,
                        uint32_t dev_adrress,
                        uint16_t transfer_size,
                        i2c_autoend_mode_t endmode,
                        i2c_startstop_mode_t request);
void I2C_DeInit(I2C_Type *const base);
void I2C_Init(I2C_Type *const base, const i2c_config_t *const config);

/* I2C 配置API */
void I2C_ConfigFilters(I2C_Type *const base, i2c_analogfilter_mode_t analogfilter, uint32_t digitalfilter);
void I2C_SetDigitalFilter(I2C_Type *const base, uint32_t digitalFilter);
uint32_t I2C_GetDigitalFilter(I2C_Type *const base);
void I2C_SetAnalogFilterMode(I2C_Type *const base, bool enable);
void I2C_SetClockStretching(I2C_Type *const base, bool enable);
void I2C_SetSlaveByteControl(I2C_Type *const base, bool enable);
void I2C_SetGeneralCall(I2C_Type *const base, bool enable);
void I2C_SetMasterAddressingMode(I2C_Type *const base, i2c_addressing_mode_t addressingmode);
i2c_addressing_mode_t I2C_GetMasterAddressingMode(I2C_Type *const base);
void I2C_SetOwnAddress1(I2C_Type *const base, uint32_t ownaddress1, i2c_addressing_mode_t ownaddrsize);
void I2C_SetOwnAddress1Mode(I2C_Type *const base, bool enable);
void I2C_SetOwnAddress2(I2C_Type *const base, uint32_t ownaddress2, uint32_t ownaddrmask);
void I2C_SetOwnAddress2mode(I2C_Type *const base, bool enable);
void I2C_SetTiming(I2C_Type *const base, uint32_t timing);
void I2C_SetWakeUpFromStop(I2C_Type *const base, bool enable);
void I2C_SetAutoEnd(I2C_Type *const base, bool enable);
void I2C_SetReloadMode(I2C_Type *const base, bool enable);
void I2C_SetTransferSize(I2C_Type *const base, uint32_t transfersize);
uint32_t I2C_GetTransferSize(I2C_Type *const base);
void I2C_AcknowledgeNextData(I2C_Type *const base, i2c_ack_mode_t type_acknowledge);
void I2C_GenerateStartCondition(I2C_Type *const base);
void I2C_GenerateStopCondition(I2C_Type *const base);
void I2C_SetAuto10BitRead(I2C_Type *const base, bool enable);
void I2C_SetTransferRequest(I2C_Type *const base, uint32_t transferrequest);
i2c_request_mode_t I2C_GetTransferRequest(I2C_Type *const base);
void I2C_SetSlaveAddr(I2C_Type *const base, uint32_t slaveaddr);
uint32_t I2C_GetSlaveAddr(I2C_Type *const base);
i2c_directionflag_t I2C_GetTransferDirection(I2C_Type *const base);
uint32_t I2C_GetAddressMatchCode(I2C_Type *const base);

/* I2C_SMBUS API */
void I2C_SMBUS_SetAlert(I2C_Type *const base, bool enable);
void I2C_SMBUS_SetPEC(I2C_Type *const base, bool enable);
void I2C_SMBUS_SetTimeoutA(I2C_Type *const base, uint32_t timeoutA);
void I2C_SMBUS_SetTimeoutAMode(I2C_Type *const base, i2c_smbus_timeouta_mode_t timeoutAmode);
void I2C_SMBUS_SetTimeoutB(I2C_Type *const base, uint32_t timeoutB);
void I2C_SMBUS_EnableTimeout(I2C_Type *const base, i2c_smbus_timout_config_t clocktimeout);
void I2C_SMBUS_DisableTimeout(I2C_Type *const base, uint32_t clocktimeout);
void I2C_SMBUS_EnablePECCompare(I2C_Type *const base);
uint32_t I2C_SMBUS_GetPEC(I2C_Type *const base);

/* I2C 数据收发API */
uint8_t I2C_ReceiveData8(I2C_Type *const base);
void I2C_TransmitData8(I2C_Type *const base, uint8_t Data);

/* I2C状态与中断 API */
bool I2C_GetStatusFlag(I2C_Type *const base, i2c_status_flag_t flag);
void I2C_ClearStatusFlag(I2C_Type *const base, i2c_status_flag_t flag);
void I2C_SetIntMode(I2C_Type *const base, i2c_interrupt_t interrupt, bool enable);
bool I2C_GetIntFlag(I2C_Type *const base, i2c_status_flag_t flag);

#if defined(__cplusplus)
}
#endif

/*! @} */

#endif /* NCC0670_I2C_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
