/*! @file    nccm0670_pwr.h
 *  @brief   PWR固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_PWR_H
#define NCC0670_PWR_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_pwr
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/


/*!
 * @brief   PWR LDO控制（Low power deepsleep）
 */
typedef enum
{
    PWR_MAINREGULATOR_ON = PWR_CR_LPDS(0) | PWR_CR_HLDO_LPMOD(0),     /*!< 进入停止模式后，LDO正常工作，不进入低功耗模式 */
    PWR_LOWPOWERREGULATOR_ON = PWR_CR_LPDS(0) | PWR_CR_HLDO_LPMOD(1), /*!< 进入停止模式后，LDO进入低功耗模式工作 */
    PWR_LOWPOWERREGULATOR_OFF = PWR_CR_LPDS(1) | PWR_CR_HLDO_LPMOD(0) /*!< 进入停止模式后，LDO关闭 */
} pwr_regulator_t;

/*!
 * @brief   进入低功耗模式方式
 */
typedef enum
{
    PWR_LOWPOWER_ENTRY_WFI, /*!< 进入低功耗模式，等待中断唤醒 */
    PWR_LOWPOWER_ENTRY_WFE  /*!< 进入低功耗模式，等待事件唤醒 */
} pwr_lowpower_entry_t;

/*!
 * @brief   欠压检测电平选择
 */
typedef enum
{
    PWR_BOD_LEVEL_NONE = 0,
    PWR_BOD_LEVEL_2V   = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(0), /*!< 2.0V */
    PWR_BOD_LEVEL_2V2  = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(1), /*!< 2.2V */
    PWR_BOD_LEVEL_2V4  = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(2), /*!< 2.4V */
    PWR_BOD_LEVEL_2V6  = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(3), /*!< 2.6V */
    PWR_BOD_LEVEL_2V8  = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(4), /*!< 2.8V */
    PWR_BOD_LEVEL_3V   = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(5), /*!< 3.0V */
    PWR_BOD_LEVEL_3V7  = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(6), /*!< 3.7V */
    PWR_BOD_LEVEL_4V3  = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(7), /*!< 4.3V */
    PWR_BOD_LEVEL_1V2  = PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL(8)  /*!< 选择外部小于1.2V */
}pwr_bod_level_t;

/*!
 * @brief   欠压复位电压阈值
 */
typedef enum
{
    PWR_BOR_LEVEL_NONE = 0,
    PWR_BOR_LEVEL_2V1  = PWR_BORCR_BOR_EN_Msk | PWR_BORCR_BOR_LEVEL(0), /*!< 2.1V */
    PWR_BOR_LEVEL_2V4  = PWR_BORCR_BOR_EN_Msk | PWR_BORCR_BOR_LEVEL(1), /*!< 2.4V */
    PWR_BOR_LEVEL_1V9  = PWR_BORCR_BOR_EN_Msk | PWR_BORCR_BOR_LEVEL(2)  /*!< 1.9V */
} pwr_bor_level_t;

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif

void PWR_DeInit(void);
void PWR_EnterSleepMode(pwr_lowpower_entry_t sleep_entry);
void PWR_EnterStopMode(pwr_regulator_t regulator, pwr_lowpower_entry_t stop_entry);
void PWR_SetAdcPowerDown(bool enable);
void PWR_SetBandgapPowerDown(bool enable);
void PWR_SetBodLevel(pwr_bod_level_t bod_level);
void PWR_SetBorLevel(pwr_bor_level_t bor_level);

#if defined(__cplusplus)
}
#endif

/*! @} */
#endif /* NCC0670_PWR_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
