/*! @file    nccm0670_usart.h
 *  @brief   USART固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_USART_H
#define NCC0670_USART_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_usart
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/**
 * @brief 奇偶校验
 */
typedef enum
{
    USART_PARITY_NONE = 0x0U,                                /*!< 关闭奇偶校验 */
    USART_PARITY_EVEN = USART_CTL_PCE_Msk,                   /*!< 偶校验 */
    USART_PARITY_ODD  = USART_CTL_PCE_Msk | USART_CTL_PS_Msk /*!< 奇校验 */
} usart_parity_t;

/**
 * @brief 数据长度
 */
typedef enum
{
    USART_DATAWIDTH_8B, /*!< 8 位数据 : 1 个起始位， 8 个数据位， n 个停止位 */
    USART_DATAWIDTH_9B, /*!< 9 位数据 : 1 个起始位， 9 个数据位， n 个停止位 */
    USART_DATAWIDTH_7B  /*!< 7 位数据 : 1 个起始位， 7 个数据位， n 个停止位 */
} usart_datawidth_t;

/**
 * @brief 停止位
 */
typedef enum
{
    USART_STOPBITS_1,      /*!< 1 个停止位 */
    USART_STOPBITS_2 = 2   /*!< 2 个停止位 */
} usart_stopbits_t;

/**
 * @brief 传输方向
 */
typedef enum
{
    USART_DIRECTION_NONE  = 0x0U,                               /*!< 关闭发送和接收 */
    USART_DIRECTION_RX    = USART_CTL_RE_Msk,                   /*!< 只接收 */
    USART_DIRECTION_TX    = USART_CTL_TE_Msk,                   /*!< 只发送 */
    USART_DIRECTION_TX_RX = USART_CTL_RE_Msk | USART_CTL_TE_Msk /*!< 发送和接收 */
} usart_direction_t;

/**
 * @brief 过采样
 */
typedef enum
{
    USART_OVERSAMPLING_16, /*!< 16 倍过采样*/
    USART_OVERSAMPLING_8   /*!< 8 倍过采样*/
} usart_oversampling_t;

/**
 * @brief 自动波特率模式
 */
typedef enum
{
    USART_AUTOBAUD_DETECT_ON_STARTBIT, /*!< 通过测量起始位检测波特率 */
    USART_AUTOBAUD_DETECT_ON_FALLINGEDGE, /*!< 下降沿到下降沿的测量（接收到的帧必须以一个等于 1 的位开头，即帧 =
                                             10xxxxxx） */
    USART_AUTOBAUD_DETECT_ON_7F_FRAME, /*!< 0x7F 帧检测 */
    USART_AUTOBAUD_DETECT_ON_55_FRAME  /*!< 0x55 帧检测 */
} usart_autobaud_mode_t;

/**
 * @brief USART 初始化配置
 */
typedef struct
{
    uint32_t baudrate;                    /*!< 波特率 */
    usart_datawidth_t datawidth;          /*!< 帧长度 */
    usart_stopbits_t stopbits;            /*!< 停止位 */
    usart_parity_t parity;                /*!< 奇偶校验位 */
    usart_direction_t transfer_direction; /*!< 传输方向 */
    usart_oversampling_t oversampling;    /*!< 过采样 */
} usart_base_cfg_t;

/**
 * @brief 传输比特顺序
 */
typedef enum
{
    USART_BITORDER_LSBFIRST, /*!< 在起始位之后，首先使用数据位 0 发送/接收数据 */
    USART_BITORDER_MSBFIRST  /*!< 在起始位之后，首先使用 MSB（位 7/8）发送/接收数据 */
} usart_bitorder_t;

/**
 * @brief 7 位地址检测/4 位地址检测
 */
typedef enum
{
    USART_ADDRESS_DETECT_4B, /*!< 4 位地址检测 */
    USART_ADDRESS_DETECT_7B  /*!< 7 位地址检测（在 8 位数据模式下） */
} usart_address_detect_t;

/**
 * @brief USART 静默模式的唤醒方法
 */
typedef enum
{
    USART_WAKEUP_IDLELINE,   /*!<  空闲线路 */
    USART_WAKEUP_ADDRESSMARK /*!<  地址标记 */
} usart_wakeup_method_t;

/**
 * @brief 时钟使能，用于同步模式时钟产生
 */
typedef enum
{
    USART_CLK_NO_OUTPUT, /*!< 禁止CK引脚 */
    USART_CLK_OUTPUT     /*!< 使能CK引脚 */
} usart_clk_output_t;

/**
 * @brief 时钟极性
 * 在同步模式下选择CK引脚上时钟输出的极性
 */
typedef enum
{
    USART_POLARITY_LOW, /*!< 空闲时CK引脚为低电平*/
    USART_POLARITY_HIGH /*!< 空闲时CK引脚为高电平 */
} usart_polarity_t;

/**
 * @brief 时钟相位
 * 在同步模式下选择CK引脚上时钟输出的相位
 */
typedef enum
{
    USART_PHASE_1EDGE, /*!< 从第一个时钟边沿开始采样数据*/
    USART_PHASE_2EDGE  /*!< 从第二个时钟边沿开始采样数据 */
} usart_phase_t;

/**
 * @brief 最后一个位时钟脉冲
 * 用于在同步模式下选择与发送的最后一个数据位(MSB)关联的时钟脉冲是否必须在CK引脚上输出
 */
typedef enum
{
    USART_LASTCLKPULSE_NO_OUTPUT, /*!< 最后一个数据位的时钟脉冲不在CK引脚上输出 */
    USART_LASTCLKPULSE_OUTPUT     /*!< 最后一个数据位的时钟脉冲在CK引脚上输出 */
} usart_last_clk_pulse_t;

/**
 * @brief USART SCLK时钟配置
 */
typedef struct
{
    usart_clk_output_t output;             /*!< CK是否输出 */
    usart_polarity_t polarity;             /*!< 时钟极性 */
    usart_phase_t phase;                   /*!< 时钟相位 */
    usart_last_clk_pulse_t last_clk_pulse; /*!< 最后一个位时钟脉冲 */
} usart_clock_cfg_t;

/**
 * @brief 选择正常模式和低功耗 IrDA 模式
 */
typedef enum
{
    USART_IRDA_POWER_NORMAL, /*!< IrDA 正常模式 */
    USART_IRDA_POWER_LOW     /*!< IrDA 低功耗模式 */
} usart_irda_power_mode_t;

/**
 * @brief 二进制数逻辑
 */
typedef enum
{
    USART_BINARY_LOGIC_POSITIVE, /*!< 正向逻辑：1=H，0=L */
    USART_BINARY_LOGIC_NEGATIVE, /*!< 反向逻辑：0=H，1=L */
} usart_binary_logic_t;

/**
 * @brief 中断 使能/标志/清除位
 */
typedef enum
{
    USART_INT_TXF    = USART_IE_TXFIE_Msk,    /*!< TX clear完成中断使能 */
    USART_INT_RXF    = USART_IE_RXFIE_Msk,    /*!< RX clear完成中断使能 */
    USART_INT_IDLE   = USART_IE_IDLEIE_Msk,   /*!< IDLE 帧中断使能 */
    USART_INT_WU     = USART_IE_WUIE_Msk,     /*!< 从静默模式唤醒中断使能 */
    USART_INT_CM     = USART_IE_CMIE_Msk,     /*!< 字符匹配中断使能 */
    USART_INT_ABRE   = USART_IE_ABREIE_Msk,   /*!< 自动波特率检测错误中断使能 */
    USART_INT_RTO    = USART_IE_RTOIE_Msk,    /*!< 接收器超时中断使能 */
    USART_INT_LBD    = USART_IE_LBDIE_Msk,    /*!< LIN 断路检测中断使能 */
    USART_INT_TXE    = USART_IE_TXEIE_Msk,    /*!< TXEIF 中断使能 */
    USART_INT_TC     = USART_IE_TCIE_Msk,     /*!< 传输完成中断使能 */
    USART_INT_RXNE   = USART_IE_RXNEIE_Msk,   /*!< RXNEIF 标志 中断使能 */
    USART_INT_RXFULL = USART_IE_RXFULLIE_Msk, /*!< RXFULL中断使能X */
    USART_INT_ORE    = USART_IE_OREIE_Msk,    /*!< 接受FIFO上溢中断使能位 */
    USART_INT_N      = USART_IE_NIE_Msk,      /*!< 噪声中断使能 */
    USART_INT_FE     = USART_IE_FEIE_Msk,     /*!< 错误中断使能 */
    USART_INT_PE     = USART_IE_PEIE_Msk      /*!< PE奇偶校验错误中断使能 */
} usart_interrupt_t;

/**
 * @brief 中断和状态 标志位
 */
typedef enum
{
    USART_FLAG_TXF    = USART_IS_TXF_Msk,    /*!< 中断：TX FIFO clear完成标志位 */
    USART_FLAG_RXF    = USART_IS_RXF_Msk,    /*!< 中断：RX FIFO clear完成标志位 */
    USART_FLAG_IDLE   = USART_IS_IDLE_Msk,   /*!< 中断：检测到空闲线路 */
    USART_FLAG_WUF    = USART_IS_WUF_Msk,    /*!< 中断：接收器检测到静默模式唤醒事件标志位 */
    USART_FLAG_RWU    = USART_IS_RWU_Msk,    /*!< 接收器静默模式状态位 */
    USART_FLAG_SBKF   = USART_IS_SBKF_Msk,   /*!< 发送断路标志 */
    USART_FLAG_CMF    = USART_IS_CMF_Msk,    /*!< 中断：字符匹配标志 */
    USART_FLAG_BUSY   = USART_IS_BUSY_Msk,   /*!< 忙标志 */
    USART_FLAG_ABRF   = USART_IS_ABRF_Msk,   /*!< 自动波特率标志 */
    USART_FLAG_ABRE   = USART_IS_ABRE_Msk,   /*!< 中断：自动波特率错误 */
    USART_FLAG_RTOF   = USART_IS_RTOF_Msk,   /*!< 中断：接收器超时 */
    USART_FLAG_LBDF   = USART_IS_LBDF_Msk,   /*!< 中断：LIN 断路检测标志 */
    USART_FLAG_TXE    = USART_IS_TXEIF_Msk,  /*!< FIFO中发送数据寄存器为空 */
    USART_FLAG_TC     = USART_IS_TC_Msk,     /*!< 中断：发送完成 */
    USART_FLAG_RXNE   = USART_IS_RXNEIF_Msk, /*!< 中断：读取数据寄存器不为空 */
    USART_FLAG_RXFULL = USART_IS_RXFULL_Msk, /*!< 中断：接收数据FIFO为满的标志位 */
    USART_FLAG_ORE    = USART_IS_ORE_Msk,    /*!< 中断：溢出错误 */
    USART_FLAG_NF     = USART_IS_NF_Msk,     /*!< 中断：START 位噪声检测标志 */
    USART_FLAG_FE     = USART_IS_FE_Msk,     /*!< 中断：帧错误 */
    USART_FLAG_PE     = USART_IS_PE_Msk      /*!< 中断：奇偶校验错误 */
} usart_status_flag_t;

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif
/* USART 通用 API */
void USART_DeInit(USART_Type *const base);
void USART_Init(USART_Type *const base, const usart_base_cfg_t *config);
void USART_SetBaudRate(USART_Type *const base, uint32_t baud);
void USART_Enable(USART_Type *const base);
void USART_Disable(USART_Type *const base);
void USART_SetTransferDirection(USART_Type *const base, usart_direction_t direction);
void USART_SetTransferBitOrder(USART_Type *const base, usart_bitorder_t bitorder);
void USART_SetOverSampling(USART_Type *const base, usart_oversampling_t oversampling);
void USART_SetOneBitSample(USART_Type *const base, bool enable);
void USART_SetRxTimeoutMode(USART_Type *const base, bool enable);
void USART_SetRxTimeout(USART_Type *const base, uint32_t timeout);
void USART_TransmitData(USART_Type *const base, uint32_t value);
uint16_t USART_ReceiveData(USART_Type *const base);
void USART_TransmitData9(USART_Type *const base, uint32_t value);
uint16_t USART_ReceiveData9(USART_Type *const base);
void USART_RequestRxDataFlush(USART_Type *const base);
void USART_RequestTxDataFlush(USART_Type *const base);
void USART_SetBinaryDataLogic(USART_Type *const base, usart_binary_logic_t data_Logic);

/* 自动波特率侦测 API */
void USART_SetAutoBaudRate(USART_Type *const base, bool enable);
void USART_SetAutoBaudRateMode(USART_Type *const base, usart_autobaud_mode_t mode);
void USART_RequestAutoBaudRate(USART_Type *const base);

/* 多机通信 API */
void USART_ConfigNodeAddress(USART_Type *const base, usart_address_detect_t address_len, uint8_t node_address);
void USART_SetMute(USART_Type *const base, bool enable);
void USART_RequestEnterMuteMode(USART_Type *const base);
void USART_SetWakeUpMethod(USART_Type *const base, usart_wakeup_method_t method);

/* LIN API */
void USART_SetLin(USART_Type *const base, bool enable);
void USART_RequestBreakSending(USART_Type *const base);

/* 半双工通信 API */
void USART_SetHalfDuplex(USART_Type *const base, bool enable);

/* 同步串口 API */
void USART_SetSclkOutput(USART_Type *const base, bool enable);
void USART_ClockInit(USART_Type *const base, const usart_clock_cfg_t *config);

/* IrDA API */
void USART_SetIrda(USART_Type *const base, bool enable);
void USART_SetIrdaPrescaler(USART_Type *const base, uint8_t value);
void USART_SetIrdaPowerMode(USART_Type *const base, usart_irda_power_mode_t mode);

/* 状态与中断 API */
bool USART_GetStatusFlag(USART_Type *const base, usart_status_flag_t flag);
void USART_SetIntMode(USART_Type *const base, usart_interrupt_t interrupt, bool enable);
bool USART_GetIntFlag(USART_Type *const base, usart_status_flag_t interrupt);
void USART_ClearStatusFlag(USART_Type *const base, usart_status_flag_t flag);

#if defined(__cplusplus)
}
#endif

/*! @} */

#endif /* NCC0670_USART_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
