/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_adc ADC
 * @brief ADC模块固件库

 * @{
 */
#include "nccm0670_adc.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief ADC反初始化
 *
 * @return None
 ******************************************************************************/
void ADC_DeInit(void)
{
    RCC_PeripheralForceReset(RCC_ADC);
}

/*!*****************************************************************************
 * @brief ADC初始化
 *
 * @param[in] config ADC配置
 * @return None
 ******************************************************************************/
void ADC_Init(const adc_config_t *config)
{
    /* 配置参考电压，分频系数，连续采样 */
    ADC->CTL &= ~(ADC_CTL_VREF_SEL_Msk | ADC_CTL_CLK_SEL_Msk | ADC_CTL_CONT_Msk | ADC_CTL_VBUF_SEL_Msk);
    ADC->CTL |= ADC_CTL_CLK_SEL(config->prescaler) | ADC_CTL_CONT(config->continuous_conv) | (uint32_t)(config->vref);

    /* 配置触发边沿和触发模式 */
    ADC->CFG &= ~(ADC_CFG_TRGS_Msk | ADC_CFG_EDGE_Msk);
    ADC->CFG |= ADC_CFG_EDGE(config->edge) | ADC_CFG_TRGS(config->trigger);
    /* 使能ADC */
    ADC->CTL |= ADC_CTL_EN_Msk;
}

/*!*****************************************************************************
 * @brief 开启ADC通道
 *
 * @param[in] channel ADC通道
 * @param[in] config ADC通道配置参数
 * @return None
 ******************************************************************************/
void ADC_StartChannel(adc_channel_t channel, const adc_channel_config_t *config)
{
    /* 选择ADC通道 */
    ADC->CTL &= ~(ADC_CTL_ECH_TYPE_SEL_Msk | ADC_CTL_ECH_SEL_Msk | ADC_CTL_ICH_SEL_Msk | ADC_CTL_SAMP_Msk);
    ADC->CTL |= (uint32_t)channel;
    /* 采样时间 */
    ADC->CTL |= ADC_CTL_SAMP(config->sampling_time);
    /* 配置触发延时 */
    ADC->CFG &= ~ADC_CFG_TRGDL_Msk;
    ADC->CFG |= ADC_CFG_TRGDL(config->trigger_delay_time);
    /* 开启转换 */
    ADC->CTL |= ADC_CTL_ADGO_Msk;
}

/*!*****************************************************************************
 * @brief 软件触发ADC转换
 ******************************************************************************/
void ADC_TrigSwConversion(void)
{
    /* 清除转换结束标志 */
    ADC->ICR = ADC_ISR_EOCIF_Msk | ADC_ISR_REFIF_Msk;
    /* 开启转换 */
    ADC->CTL |= ADC_CTL_ADGO_Msk;
}

/*!*****************************************************************************
 * @brief 关闭ADC
 ******************************************************************************/
void ADC_Stop(void)
{
    ADC->CTL &= ~ADC_CTL_EN_Msk;
}

/*!*****************************************************************************
 * @brief 设置参考电压源
 *
 * @return ADC值
 ******************************************************************************/
void ADC_SetVref(adc_voltage_reference_t vref)
{
    ADC->CTL &= ~(ADC_CTL_VBUF_SEL_Msk | ADC_CTL_VREF_SEL_Msk);
    ADC->CTL |= (uint32_t)vref;
}

/*!*****************************************************************************
 * @brief 读取ADC转换结果
 *
 * @return ADC值
 ******************************************************************************/
uint32_t ADC_GetValue(void)
{
    return ADC->DATA & ADC_DATA_DATA_Msk;
}

/*!*****************************************************************************
 * @brief ADC比较模式配置
 *
 * @param[in] config 比较模式配置参数
 * @return None
 ******************************************************************************/
void ADC_SetCompare(const adc_compare_config_t *config)
{
    /* 设置比较下限 */
    ADC->REFL = config->low_limit;
    /* 设置比较上限 */
    ADC->REFH = config->high_limit;
    /* 设置比较模式 */
    ADC->CTL &= ~(ADC_CTL_REF_CMP_Msk);
    ADC->CTL |= ADC_CTL_REF_CMP(config->mode);
}

/*!*****************************************************************************
 * @brief ADC中断开关
 * @note 若只需使能ADC比较中断，且需进比较中断处理函数进行相关操作，必须先关闭ADC转换完成中断
 * @param[in] interrupt 中断选择
 * @param[in] enabel 使能开关
 * @return None
 ******************************************************************************/
void ADC_SetIntMode(adc_interrupt_t interrupt, bool enable)
{
    if (enable)
    {
        ADC->IER |= (uint32_t)interrupt;
    }
    else
    {
        ADC->IER &= ~(uint32_t)interrupt;
    }
}

/*!*****************************************************************************
 * @brief ADC状态获取
 *
 * @param[in] flag 状态标志
 * @return None
 ******************************************************************************/
bool ADC_GetStatusFlag(adc_status_flag_t flag)
{
    return (ADC->ISR & (uint32_t)flag) > 0U;
}

/*!*****************************************************************************
 * @brief 清除状态
 *
 * @param[in] 状态状态
 *  @arg ADC_FLAG_COMPARE
 *  @arg ADC_FLAG_FINISH
 * @return None
 ******************************************************************************/
void ADC_ClearStatusFlag(adc_status_flag_t flag)
{
    ADC->ICR = (uint32_t)flag;
}

/*!*****************************************************************************
 * @brief 获取中断状态
 *
 * @param[in] flag 状态
 * @return None
 ******************************************************************************/
bool ADC_GetIntFlag(adc_status_flag_t flag)
{
    return (ADC->ISR & (uint32_t)flag) && (ADC->IER & (uint32_t)flag);
}
/*! @} */
