/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_awu AWU
 * @brief AWU模块固件库

## 详细说明 ##
- 唤醒定时器
- 可配置在STOP模式下运行并唤醒STOP
- 蜂鸣器
- BUZZER电路输出的BUZ信号范围为500~16KHz

## 使用指导 ##

@verbatim

    __disable_irq();
    SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk; //关系统嘀嗒
    NVIC_DisableIRQ(USART1_IRQn);

    exti_config_t exti_cfg;
    exti_cfg.mode    = EXTI_MODE_INTERRUPT;
    exti_cfg.trigger = EXTI_TRIGGER_RISING;
    EXTI_SetConfigLine(EXTI_LINE_20, &exti_cfg); //设置awu外部中断线12
    NVIC_EnableIRQ(AWU_IRQn);                    //开awu中断

    __enable_irq();
    RCC_EnableOSC(RCC_OSC_LSI);
    awu_config_t cfg ={
        .awu_timeout = 0,
        .buz_frequency = 0,
    };
    AWU_Init(&cfg);//唤醒时间为：16384/32768*(x+1) 秒，此处设置为500ms
    AWU_Enable(); 
    PWR_EnterStopMode(PWR_LOWPOWERREGULATOR_ON, PWR_LOWPOWER_ENTRY_WFI);
    SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk;
    NVIC_EnableIRQ(USART1_IRQn); 

@endverbatim

@verbatim

    awu_gpio_init();
    RCC_EnableOSC(RCC_OSC_LSI);
    awu_config_t cfg ={
        .awu_timeout = 0,
        .buz_frequency = 455,
    };
    AWU_Init(&cfg);
    BUZ_Enable();  
     
@endverbatim

 * @{
 */
#include "nccm0670_awu.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief AWU反初始化
 *
 * @return None
 ******************************************************************************/
void AWU_DeInit(void)
{
    RCC_PeripheralForceReset(RCC_AWU);
}

/*!*****************************************************************************
 * @brief 实时唤醒定时器/BUZ初始化
 * timeout值为0x00~0x1F
 * BUZ分频值为0~31(0x00~0x0F)，即2分频~64分频，分频值为32000/2/buz_frequency - 1，
 * 用户输入的buz_frequency值应在500HZ~16000HZ之间，否则无效
 * @param[in]  定时器/BUZ配置
 * @return None
 ******************************************************************************/
void AWU_Init(awu_config_t *config)
{
    AWU->AWUCON = AWU_AWUCON_TIMEOUT(config->awu_timeout);
    
    if (config->buz_frequency < 500)
    {
        config->buz_frequency = 500;
    }
    if (config->buz_frequency > 16000)
    {
        config->buz_frequency = 16000;
    }
    AWU->BUZCON = AWU_BUZCON_PSC(LSI_VALUE / 2 / config->buz_frequency - 1);
}

/*!*****************************************************************************
 * @brief  AWU模块使能
 *
 * @return None
 ******************************************************************************/
void AWU_Enable(void)
{
    AWU->AWUCON |= AWU_AWUCON_EN_Msk;
}

/*!*****************************************************************************
 * @brief 关闭实时唤醒定时器
 *
 * @return None
 ******************************************************************************/
void AWU_Disable(void)
{
    AWU->AWUCON &= ~AWU_AWUCON_EN_Msk;
}
/*!*****************************************************************************
 * @brief  BUZ模块使能
 *
 * @return None
 ******************************************************************************/
void BUZ_Enable(void)
{
    AWU->BUZCON |= AWU_BUZCON_EN_Msk;
}
/*!*****************************************************************************
 * @brief 关BUZ
 *
 * @return None
 ******************************************************************************/
void BUZ_Disable(void)
{
    AWU->BUZCON &= ~AWU_BUZCON_EN_Msk;
}
/*! @} */
