/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_gpio GPIO
 * @brief GPIO模块固件库

## 详细说明 ##

- 输入输出方向
- 输出速率
- 引脚复用
- 上下拉配置
- 输入输出值

## 使用指导 ##

@verbatim
    GPIO_InitStruct.pin  = GPIO_PIN_6 | GPIO_PIN_7 | GPIO_PIN_4 | GPIO_PIN_5;
    GPIO_InitStruct.mode = GPIO_MODE_OUTPUT;
    GPIO_InitStruct.pull = GPIO_NOPULL;
    GPIO_InitStruct.speed = GPIO_SPEED_FREQ_VERY_HIGH;
    GPIO_Init(GPIOA, &GPIO_InitStruct);

    GPIO_SetPins(GPIOA, GPIO_PIN_6);
@endverbatim


 * @{
 */
#include "nccm0670_gpio.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief  初始化GPIO
 *
 * @param[in] base  GPIO基址
 * @param[in] config 初始化配置表
 * @return None
 ******************************************************************************/
void GPIO_Init(GPIO_Type *const base, const gpio_config_t *const config)
{
    if ((config->mode == GPIO_MODE_OUTPUT) || (config->mode == GPIO_MODE_ALTERNATE))
    {
        GPIO_SetPinOutput(base, config->output_type, config->speed, config->pin);
    }

    if (config->mode == GPIO_MODE_ALTERNATE)
    {
        /* Speed mode configuration */
        GPIO_SetPinAf(base, config->alternate, config->pin);
    }

    /* Pin Mode configuration */
    GPIO_SetPinMode(base, config->mode, config->pull, config->pin);
}

/*!*****************************************************************************
 * @brief  反初始化GPIO
 *
 * @param[in] base  GPIO基址
 * @return None
 ******************************************************************************/
void GPIO_DeInit(GPIO_Type *const base)
{
    /* 重启外设时钟复位外设寄存器 */
    if (base == GPIOA)
    {
        RCC_PeripheralForceReset(RCC_GPIOA);
    }
    else if (base == GPIOB)
    {
        RCC_PeripheralForceReset(RCC_GPIOB);
    }
    else if (base == GPIOC)
    {
        RCC_PeripheralForceReset(RCC_GPIOC);
    }
    else
    {
        /* 防止静态分析报警 */
    }
}

/*!*****************************************************************************
 * @brief  配置工作模式
 *
 * @param[in] base  GPIO基址
 * @param[in] mode GPIO工作模式
 * @param[in] pull_up_down 上下拉
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 ******************************************************************************/
void GPIO_SetPinMode(GPIO_Type *const base, gpio_mode_t mode, gpio_pull_t pull_up_down, uint32_t pin)
{
    uint16_t i, shift;
    uint32_t moder, pupd;

    moder = base->MODER;
    pupd  = base->PUPDR;

    for (i = 0U; i < 16U; i++)
    {
        if (((1UL << i) & pin) != 0UL)
        {
            shift = 2U * i;
            /* clear the specified pin mode bits */
            moder &= ~(GPIO_MODER_MODE0_Msk << shift);
            /* set the specified pin mode bits */
            moder |= (uint32_t)mode << shift;

            /* clear the specified pin pupd bits */
            pupd &= ~(GPIO_PUPDR_PUPD0_Msk << shift);
            /* set the specified pin pupd bits */
            pupd |= (uint32_t)pull_up_down << shift;
        }
    }

    base->MODER = moder;
    base->PUPDR = pupd;
}

/*!*****************************************************************************
 * @brief  配置输出参数
 *
 * @param[in] base  GPIO基址
 * @param[in] otype 推挽输出或者开漏输出
 * @param[in] speed 输出速度
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 ******************************************************************************/
void GPIO_SetPinOutput(GPIO_Type *const base, gpio_output_type_t otype, gpio_speed_t speed, uint32_t pin)
{
    uint16_t i, shift;
    uint32_t ospeed;

    if (GPIO_OUTPUT_OPENDRAIN == otype)
    {
        base->OTYPER |= pin;
    }
    else
    {
        base->OTYPER &= ~pin;
    }

    /* get the specified pin output speed bits value */
    ospeed = base->OSPEEDR;

    for (i = 0U; i < 16U; i++)
    {
        if (((1UL << i) & pin) != 0UL)
        {
            shift = 2U * i;

            /* clear the specified pin output speed bits */
            ospeed &= ~(GPIO_OSPEEDR_OSPEED0_Msk << shift);
            /* set the specified pin output speed bits */
            ospeed |= (uint32_t)speed << shift;
        }
    }
    base->OSPEEDR = ospeed;
}

/*!*****************************************************************************
 * @brief  配置引脚复用
 *
 * @param[in] base  GPIO基址
 * @param[in] alternate 引脚复用
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 ******************************************************************************/
void GPIO_SetPinAf(GPIO_Type *const base, gpio_alt_t alternate, uint32_t pin)
{
    uint16_t i, shift;
    uint32_t afrl, afrh;

    afrl = base->AFRL;
    afrh = base->AFRH;

    for (i = 0U; i < 8U; i++)
    {
        if (((1UL << i) & pin) != 0U)
        {
            shift = 4U * i;
            /* clear the specified pin alternate function bits */
            afrl &= ~(GPIO_AFRL_AFSEL0_Msk << shift);
            afrl |= (uint32_t)alternate << shift;
        }
    }

    for (i = 8U; i < 16U; i++)
    {
        if (((1UL << i) & pin) != 0UL)
        {
            shift = 4U * (i - 8U);
            /* clear the specified pin alternate function bits */
            afrh &= ~(GPIO_AFRH_AFSEL8_Msk << shift);
            afrh |= (uint32_t)alternate << shift;
        }
    }

    base->AFRL = afrl;
    base->AFRH = afrh;
}

/*!*****************************************************************************
 * @brief  PIN置位
 *
 * @param[in] base  GPIO基址
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 ******************************************************************************/
void GPIO_SetPins(GPIO_Type *const base, uint32_t pin)
{
    base->BSRR = pin;
}

/*!*****************************************************************************
 * @brief  PIN清零
 *
 * @param[in] base  GPIO基址
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 ******************************************************************************/
void GPIO_ClearPins(GPIO_Type *const base, uint32_t pin)
{
    base->BRR = pin;
}

/*!*****************************************************************************
 * @brief  PIN写操作
 *
 * @param[in] base  GPIO基址
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 * @param[in] val true:置位，false:清零
 ******************************************************************************/
void GPIO_WritePins(GPIO_Type *const base, uint32_t pin, bool val)
{
    if (val)
    {
        base->BSRR = pin;
    }
    else
    {
        base->BRR = pin;
    }
}

/*!*****************************************************************************
 * @brief  PORT写操作
 *
 * @param[in] base  GPIO基址
 * @param[in] val true:置位，false:清零
 ******************************************************************************/
void GPIO_WritePort(GPIO_Type *const base, uint16_t val)
{
    base->ODR = val;
}

/*!*****************************************************************************
 * @brief  PIN读操作
 *
 * @param[in] base  GPIO基址
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 *
 * @return true:置位，false:清零
 ******************************************************************************/
bool GPIO_ReadPins(const GPIO_Type *const base, uint32_t pin)
{
    return (base->IDR & pin) > 0U;
}

/*!*****************************************************************************
 * @brief  PORT读操作
 *
 * @param[in] base  GPIO基址
 * @return 当前端口所有PIN输入值
 ******************************************************************************/
uint32_t GPIO_ReadPort(const GPIO_Type *const base)
{
    return base->IDR;
}

/*!*****************************************************************************
 * @brief  GPIO输出翻转
 *
 * @param[in] base  GPIO基址
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 ******************************************************************************/
void GPIO_TogglePin(GPIO_Type *const base, uint32_t pin)
{
    uint32_t odr;
    /* get current Output Data Register value */
    odr        = base->ODR;
    /* Set selected pins that were at low level, and reset ones that were high */
    base->BSRR = ((odr & pin) << 16U) | (~odr & pin);
}

/*!*****************************************************************************
 * @brief  GPIO锁定
 *
 * @param[in] base  GPIO基址
 * @param[in] pin GPIO_PIN_x(x=0..15), GPIO_PIN_ALL
 ******************************************************************************/
void GPIO_LockPin(GPIO_Type *const base, uint32_t pin)
{
    uint32_t lock = 0x00010000U;
    lock |= pin;

    /* lock key writing sequence: write 1->write 0->write 1->read 0->read 1 */
    base->LCKR = (uint32_t)lock;
    base->LCKR = (uint32_t)pin;
    base->LCKR = (uint32_t)lock;
    lock       = base->LCKR;
    lock       = base->LCKR;
}
/*! @} */
