/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_gtim GTIM
 * @brief GTIM模块固件库

## 详细说明 ##


## 使用指导 ##

@verbatim
    gtim_cfg.counter_mode   = GTIM_COUNTER_UP;
    gtim_cfg.clock_division = GTIM_CLOCK_DIVISION_DIV1;
    gtim_cfg.prescaler      = 999;
    gtim_cfg.period         = 3999;
    GTIM_Init(TIM2, &gtim_cfg);
@endverbatim


 * @{
 */
#include "nccm0670_gtim.h"
#include "nccm0670_rcc.h"

#define CC_CCMR(base, ch) (*(&base->CCMR1_Output + ch * 8))
#define CC_CCER(base, ch) (*(&base->CCER1 + ch * 8))
#define CC_CCR(base, ch)  (*(&base->CCR1 + ch * 8))
#define CC_DTR(base, ch)  (*(&base->DTR1 + ch * 8))

/*!
 * @name GTIM基本定时 API
 * @{
 */
/*!
 * @brief  GTIM初始化
 *
 * 更新事件周期计算工式如下:
 * 更新周期 = TIM_CLK/((PSC + 1)*(ARR + 1))
 *     其中: TIM_CLK = timer clock input
 *           PSC = 16-bit prescaler register
 *           ARR = 16/32-bit Autoreload register
 *
 * @param[in] base  GTIM基址
 * @param[in] config 配置结构体指针
 * @return None
 */
void GTIM_Init(GTIM_Type *const base, const gtim_base_config_t *const config)
{
    base->CR2 |= GTIM_CR2_CLKEN_Msk;
    /* 设置计数方向 */
    GTIM_SetCounterDirection(base, config->counter_mode);

    /* 设置时钟分频 */
    base->CR1 &= ~(GTIM_CR1_CKD_Msk);
    base->CR1 |= GTIM_CR1_CKD(config->clock_division);

    /* 设置自动重载值 */
    GTIM_SetAutoReload(base, config->period);

    /* 设置预分频器值 */
    GTIM_SetPrescaler(base, config->prescaler);

    /* 产生更新事件 */
    GTIM_GenerateEvent(base, GTIM_EVENT_SRC_UG);

    /*清更新标志*/
    GTIM_ClearStatusFlag(base, GTIM_FLAG_UIF);
}

/*!
 * @brief  GTIM反初始化
 *
 * @param[in] base  GTIM基址
 * @return None
 */
void GTIM_DeInit(GTIM_Type *const base)
{
    /* Force and Release reset on clock of base Port */
    if (base == TIM2)
    {
        RCC_PeripheralForceReset(RCC_TIM2);
    }
    else if (base == TIM10)
    {
        RCC_PeripheralForceReset(RCC_TIM10);
    }
    else if (base == TIM16)
    {
        RCC_PeripheralForceReset(RCC_TIM16);
    }
    else
    {
    }
}

/*!
 * @brief  GTIM模块使能
 *
 * @param[in] base  GTIM基址
 * @return None
 */
void GTIM_Enable(GTIM_Type *const base)
{
    base->CR1 |= GTIM_CR1_CEN_Msk;
}

/*!
 * @brief  GTIM模块失能
 *
 * @param[in] base  GTIM基址
 * @return None
 */
void GTIM_Disable(GTIM_Type *const base)
{
    base->CR1 &= ~GTIM_CR1_CEN_Msk;
}

/*!
 * @brief  自动重载预装载使能
 *
 * @param[in] base  GTIM基址
 * @param[in] enable 使能标志
 * @return None
 */
void GTIM_SetArrPreload(GTIM_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= GTIM_CR1_ARPE_Msk;
    }
    else
    {
        base->CR1 &= ~GTIM_CR1_ARPE_Msk;
    }
}

/*!
 * @brief  使能更新事件生成
 *
 * @param[in] base  GTIM基址
 * @param[in] enable 使能标志
 * @return None
 */
void GTIM_SetUpdateEvent(GTIM_Type *const base, bool enable)
{
    if (!enable)
    {
        base->CR1 |= GTIM_CR1_UDIS_Msk;
    }
    else
    {
        base->CR1 &= ~GTIM_CR1_UDIS_Msk;
    }
}

/*!
 * @brief  计数器计数方向
 *
 * @param[in] base  GTIM基址
 * @param[in] dir 递增or递减计数
 * @return None
 */
void GTIM_SetCounterDirection(GTIM_Type *const base, gtim_counter_direction_t dir)
{
    base->CR1 &= ~(GTIM_CR1_DIR_Msk);
    base->CR1 |= GTIM_CR1_DIR(dir);
}

/**
 * @brief  自动重载值配置
 *
 * @param[in] base  GTIM基址
 * @param[in] autoreload 自动重载值
 * @return None
 */
void GTIM_SetAutoReload(GTIM_Type *const base, uint16_t autoreload)
{
    base->ARR = autoreload;
}

/**
 * @brief  获取计数器值
 *
 * @param[in] base  GTIM基址
 * @return 计数器值
 */
uint32_t GTIM_GetCounter(GTIM_Type *const base)
{
    return base->CNT;
}

/**
 * @brief  预分频配置
 *
 * @param[in] base  GTIM基址
 * @param[in] prescaler 预分频器值
 * @return None
 */
void GTIM_SetPrescaler(GTIM_Type *const base, uint16_t prescaler)
{
    base->PSC = prescaler;
}

/**
 * @brief  单脉冲模式配置
 *
 * @param[in] base  GTIM基址
 * @param[in] mode 单脉冲模式
 * @return None
 */
void GTIM_SetOnePulseMode(GTIM_Type *const base, gtim_one_pulse_mode_t mode)
{
    base->CR1 &= ~GTIM_CR1_OWM_Msk;
    base->CR1 |= GTIM_CR1_OWM(mode);
}

/**
 * @brief  更新请求源配置
 *
 * @param[in] base  GTIM基址
 * @param[in] source 更新请求源
 * @return None
 */
void GTIM_SetUpdateSource(GTIM_Type *const base, gtim_update_request_src_t source)
{
    base->CR1 &= ~GTIM_CR1_URS_Msk;
    base->CR1 |= GTIM_CR1_URS(source);
}

/**
 * @brief  更新请求源配置
 *
 * @param[in] base  GTIM基址
 * @param[in] event 事件生成源
 * @return None
 */
void GTIM_GenerateEvent(GTIM_Type *const base, gtim_event_t event)
{
    base->EGR |= event;
}
/*!@}*/

/*!
 * @name OC输出比较 API
 * @{
 */
/**
 * @brief  输出比较功能配置
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] occfg 配置
 * @return None
 */
void GTIM_OC_Init(GTIM_Type *const base, gtim_oc_ic_channel_t channel, const gtim_oc_config_t *occfg)
{
    /* 设置输出使能 */
    GTIM_OC_SetChannel(base, channel, occfg->state == GTIM_OC_OUTPUT_ENABLE);
    /* 设置输出极性 */
    GTIM_OC_SetPolarity(base, channel, occfg->polarity);
    /* 设置互补输出使能 */
    GTIM_OC_SetComplementaryChannel(base, channel, occfg->nstate == GTIM_OC_OUTPUT_ENABLE);
    /* 设置互补输出极性 */
    GTIM_OC_SetComplementaryPolarity(base, channel, occfg->npolarity);
    /* 设置为输出比较模式 */
    CC_CCMR(base, channel) &= ~GTIM_CCMR1_Output_CC1S_Msk;
    CC_CCMR(base, channel) |= GTIM_CCMR1_Output_CC1S(1);
}

/**
 * @brief  输出模式配置
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] ocmode 输出比较模式
 * @return None
 */
void GTIM_OC_SetMode(GTIM_Type *const base, gtim_oc_ic_channel_t channel, gtim_oc_mode_t ocmode)
{
    CC_CCMR(base, channel) &= ~GTIM_CCMR1_Output_OC1M_Msk;
    CC_CCMR(base, channel) |= GTIM_CCMR1_Output_OC1M(ocmode);
}

/**
 * @brief  输出极性配置
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] ocpolarity 输出比较极性
 * @return None
 */
void GTIM_OC_SetPolarity(GTIM_Type *const base, gtim_oc_ic_channel_t channel, gtim_oc_polarity_t ocpolarity)
{
    CC_CCER(base, channel) &= ~GTIM_CCER1_CC1P_Msk;
    CC_CCER(base, channel) |= GTIM_CCER1_CC1P(ocpolarity);
}

/**
 * @brief  互补输出极性配置
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] ocpolarity 输出比较极性
 * @return None
 */
void GTIM_OC_SetComplementaryPolarity(GTIM_Type *const base,
                                      gtim_oc_ic_channel_t channel,
                                      gtim_oc_polarity_t ocpolarity)
{
    CC_CCER(base, channel) &= ~GTIM_CCER1_CC1NP_Msk;
    CC_CCER(base, channel) |= GTIM_CCER1_CC1NP(ocpolarity);
}

/**
 * @brief  输出快速使能
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] enable 使能标志
 * @return None
 */
void GTIM_OC_SetFastMode(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable)
{
    if (enable)
    {
        CC_CCMR(base, channel) |= GTIM_CCMR1_Output_OC1FE_Msk;
    }
    else
    {
        CC_CCMR(base, channel) &= ~GTIM_CCMR1_Output_OC1FE_Msk;
    }
}

/**
 * @brief  输出预装载使能
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] enable 使能标志
 * @return None
 */
void GTIM_OC_SetPreload(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable)
{
    if (enable)
    {
        CC_CCMR(base, channel) |= GTIM_CCMR1_Output_OC1PE_Msk;
    }
    else
    {
        CC_CCMR(base, channel) &= ~GTIM_CCMR1_Output_OC1PE_Msk;
    }
}

/**
 * @brief  设置输出比较值
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] value 输出比较值
 * @return None
 */
void GTIM_OC_SetCompareValue(GTIM_Type *const base, gtim_oc_ic_channel_t channel, uint16_t value)
{
    CC_CCR(base, channel) = value;
}

/**
 * @brief  使能互补输出通道
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] enable 使能标志
 * @return None
 */
void GTIM_OC_SetComplementaryChannel(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable)
{
    if (enable)
    {
        CC_CCER(base, channel) |= GTIM_CCER1_CC1NE_Msk;
    }
    else
    {
        CC_CCER(base, channel) &= ~GTIM_CCER1_CC1NE_Msk;
    }
}

/**
 * @brief  使能输出通道死区时间配置
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @return None
 */
void GTIM_OC_SetDeadTimeMode(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable)
{
    if (enable)
    {
        CC_CCER(base, channel) |= GTIM_CCER1_DTEN_Msk;
    }
    else
    {
        CC_CCER(base, channel) &= ~GTIM_CCER1_DTEN_Msk;
    }
}

/**
 * @brief  使能输出通道死区时间配置
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] revdt 反面死区值，同向输出与反向输出将同时有效时插入的延迟
 * @param[in] obvdt 正面死区值，同向输出与反向输出将同时失效时插入的延迟
 * @return None
 */
void GTIM_OC_SetDeadTime(GTIM_Type *const base, gtim_oc_ic_channel_t channel, uint8_t revdt, uint8_t obvdt)
{
    CC_DTR(base, channel) &= ~(GTIM_DTR1_REVDT_Msk | GTIM_DTR1_OBVDT_Msk);
    CC_DTR(base, channel) = GTIM_DTR1_REVDT(revdt) | GTIM_DTR1_OBVDT(obvdt);
}

/**
 * @brief  使能输出通道
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @return None
 */
void GTIM_OC_SetChannel(GTIM_Type *const base, gtim_oc_ic_channel_t channel, bool enable)
{
    if (enable)
    {
        CC_CCER(base, channel) |= GTIM_CCER1_CC1E_Msk;
    }
    else
    {
        CC_CCER(base, channel) &= ~GTIM_CCER1_CC1E_Msk;
    }
}
/*!@}*/

/*!
 * @name IC输入捕获 API
 * @{
 */
/**
 * @brief  输入捕获初始化
 *
 * @param[in] base  GTIM基址
 * @param[in] config 通道
 * @return None
 */
void GTIM_IC_Init(GTIM_Type *const base, gtim_oc_ic_channel_t channel, gtim_ic_config_t *iccfg)
{
    /* 关闭捕获通道 */
    CC_CCER(base, channel) &= ~GTIM_CCER1_CC1E_Msk;
    /* 设置捕获边沿 */
    if (iccfg->polarity == GTIM_IC_POLARITY_RISING)
    {
        CC_CCER(base, channel) &= ~(GTIM_CCER1_CC1P_Msk | GTIM_CCER1_CC1NP_Msk);
    }
    else if (iccfg->polarity == GTIM_IC_POLARITY_FALLING)
    {
        CC_CCER(base, channel) &= ~GTIM_CCER1_CC1NP_Msk;
        CC_CCER(base, channel) |= GTIM_CCER1_CC1P_Msk;
    }
    else
    {
        CC_CCER(base, channel) |= GTIM_CCER1_CC1P_Msk | GTIM_CCER1_CC1NP_Msk;
    }
    /* 选择输入信号 */
    CC_CCMR(base, channel) &= ~GTIM_CCMR1_Input_CC1S_Msk;
    CC_CCMR(base, channel) |= GTIM_CCMR1_Input_CC1S(iccfg->selection);
    /* 设置输入滤波 */
    CC_CCMR(base, channel) &= ~GTIM_CCMR1_Input_IC1F_Msk;
    CC_CCMR(base, channel) |= GTIM_CCMR1_Input_IC1F(iccfg->filter);
    /* 打开捕获通道 */
    CC_CCER(base, channel) |= GTIM_CCER1_CC1E_Msk;

    GTIM_IC_SetPrescaler(base, channel, iccfg->prescaler);
}

/**
 * @brief  设置输入捕获预分频器
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @param[in] prescaler 分频系数
 * @return None
 */
void GTIM_IC_SetPrescaler(GTIM_Type *const base, gtim_oc_ic_channel_t channel, gtim_ic_prescaler_t prescaler)
{
    CC_CCMR(base, channel) &= ~GTIM_CCMR1_Input_IC1PSC_Msk;
    CC_CCMR(base, channel) |= GTIM_CCMR1_Input_IC1PSC(prescaler);
}

/**
 * @brief  读取输入捕获值
 *
 * @param[in] base  GTIM基址
 * @param[in] channel 通道
 * @return 捕获计数值
 */
uint32_t GTIM_IC_GetCaptureValue(GTIM_Type *const base, gtim_oc_ic_channel_t channel)
{
    return CC_CCR(base, channel);
}
/*!@}*/

/*!
 * @name 主从模式 API
 * @{
 */
/**
 * @brief  设置用于同步计数器的触发输入
 *
 * @param[in] base  GTIM基址
 * @param[in] trigger_input 触发输入
 * @return None
 */
void GTIM_SetTriggerInput(GTIM_Type *const base, gtim_trigger_input_t trigger_input)
{
    base->SMCR &= ~GTIM_SMCR_TS_Msk;
    base->SMCR |= GTIM_SMCR_TS(trigger_input);
}

/**
 * @brief  设置主模式下将要发送到从定时器以实现同步的信息 (TRGO)
 *
 * @param[in] base  GTIM基址
 * @param[in] timer_synchronization 触发输出
 * @return None
 */
void GTIM_SetTriggerOutput(GTIM_Type *const base,
                           gtim_trigger_output_t timer_synchronization,
                           gtim_trgo_sync_enable trgo_sync)
{
    base->CR2 &= ~(GTIM_CR2_CHSEL_Msk | GTIM_CR2_MMS_Msk);
    base->CR2 |= timer_synchronization;
    base->SMCR &= ~GTIM_SMCR_TRGO_SYNC_Msk;
    base->SMCR |= GTIM_SMCR_TRGO_SYNC(trgo_sync);
}

/**
 * @brief  设置定时器从模式
 *
 * @param[in] base  GTIM基址
 * @param[in] mode 定时器从模式选择
 * @return None
 */
void GTIM_SetSlaveMode(GTIM_Type *const base, gtim_slave_mode_t mode)
{
    base->SMCR &= ~GTIM_SMCR_SMS_Msk;
    base->SMCR |= GTIM_SMCR_SMS(mode);
}

/**
 * @brief  设置外部触发信号极性和滤波器
 *
 * @param[in] base  GTIM基址
 * @param[in] polarity 触发信号极性
 * @param[in] filter 滤波器
 * @return None
 */
void GTIM_ConfigExternalTrigger(GTIM_Type *const base, gtim_etr_polarity_t polarity, gtim_etr_filter_t filter)
{
    base->SMCR &= ~(GTIM_SMCR_ETP_Msk | GTIM_SMCR_ETF_Msk);
    base->SMCR |= GTIM_SMCR_ETP(polarity) | GTIM_SMCR_ETF(filter);
}
/*!@}*/

/*!
 * @name 状态与中断 API
 * @{
 */
/*!
 * @brief 查询标志
 *
 * @param[in] base 基地址
 * @param[in] flag 状态标志
 * @return None
 */
bool GTIM_GetStatusFlag(GTIM_Type *const base, gtim_status_flag_t flag)
{
    uint32_t status = (base->SR1 & 0xFFFFUL) | (base->SR2 << 16U);

    return (status & flag) > 0U ? true : false;
}

/*!
 * @brief 清除标志位
 *
 * @param[in] base 基地址
 * @param[in] flag 要清除的状态位
 * @return None
 */
void GTIM_ClearStatusFlag(GTIM_Type *const base, gtim_status_flag_t flag)
{
    base->SR1 = flag & 0xFFFFUL;
    base->SR2 = (flag >> 16) & 0xFFFFUL;
}

/*!
 * @brief 中断使能或失能
 *
 * @param[in] base 基地址
 * @param[in] interrupt 中断掩码
 * @param[in] enable 使能位
 * @return None
 */
void GTIM_SetIntMode(GTIM_Type *const base, gtim_interrupt_t interrupt, bool enable)
{
    if (enable)
    {
        base->IER |= (uint32_t)interrupt;
    }
    else
    {
        base->IER &= ~(uint32_t)interrupt;
    }
}

/*!
 * @brief 获取中断使能状态
 *
 * @param[in] base 基地址
 * @param[in] flag 标志
 * @return 状态
 */
bool GTIM_GetIntFlag(GTIM_Type *const base, gtim_status_flag_t flag)
{
    uint32_t status = (base->SR1 & (uint32_t)flag) | (base->SR2 & (flag >> 16U));

    return status && (base->IER & status);
}
/*! @} */
/*! @} */
