/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * @defgroup nccm0670_i2c I2C
 * @brief I2C模块固件库

## 详细说明 ##
- 从模式和主模式
- 多主模式功能
- 标准速度模式（高达 100 kHz）
- 快速模式（高达 400 kHz）
- 超快速模式（高达 1 MHz）
- 7 位和 10 位寻址模式
- 多个 7 位从地址（2 个从设备地址寄存器，1 个具有可配置的掩码位段）
- 所有 7 位地址应答模式
- 广播呼叫
- 总线上的数据建立和保持时间可软件配置
- 方便易用的事件管理
- 可选的时钟延长
- 软件复位
- 可编程模拟和数字噪声滤波器


## 使用指导 ##

@verbatim
    i2c_config_t i2c2_cfg;
    gpio_config_t GPIO_InitStruct;

    RCC_EnablePeripheralClock(RCC_GPIOB);
    RCC_EnablePeripheralClock(RCC_I2C2);
    //I2C2 PB10 = SCL PB11 = SDA;

    GPIO_InitStruct.alternate = GPIO_AF6_I2C2 ;
    GPIO_InitStruct.mode = GPIO_MODE_ALTERNATE;
    GPIO_InitStruct.output_type = GPIO_OUTPUT_OPENDRAIN;
    GPIO_InitStruct.pin = GPIO_PIN_10 | GPIO_PIN_11 ;
    GPIO_InitStruct.pull = GPIO_PULLUP;
    GPIO_InitStruct.speed = GPIO_SPEED_FREQ_HIGH;
    GPIO_Init(GPIOB, &GPIO_InitStruct);


    i2c2_cfg.AnalogFilter = I2C_ANALOGFILTER_ENABLE;
    i2c2_cfg.DigitalFilter = 0;
    i2c2_cfg.OwnAddress1 = I2C2ADDR;
    i2c2_cfg.OwnAddrSize = I2C_OWNADDRESS1_7BIT;
    i2c2_cfg.PeripheralMode = I2C_MODE_I2C;
    i2c2_cfg.Timing = 0x00000E14 ;
    i2c2_cfg.TypeAcknowledge = I2C_ACK;
    I2C_Init(I2C2,&i2c2_cfg);

@endverbatim

 * @{
 */

#include "nccm0670_i2c.h"
#include "nccm0670_rcc.h"

#define TIMING_CLEAR_MASK (0xF0FFFFFFU) /*!< I2C TIMING clear register Mask */

/*!
 * @name I2C 通用 API
 * @{
 */
/*!*****************************************************************************
 * @brief  使能I2C外设 (PE = 1).
 * @param  base I2C实例.
 * @return None
 */
void I2C_Enable(I2C_Type *const base)
{
    base->CR1 |= I2C_CR1_PE_Msk;
}

/*!*****************************************************************************
 * @brief  禁止I2C外设 (PE = 0).
 * @note   当 PE = 0, I2C SCL 和SDA 线都被释放.
 *         内部状态机和状态位都被置为默认状态.
 *          当此位被清零, PE 必须保持为低至少3个APB clock cycles.
 * @param  base I2C实例.
 * @return None
 ******************************************************************************/
void I2C_Disable(I2C_Type *const base)
{
    base->CR1 &= ~I2C_CR1_PE_Msk;
}

/*!*****************************************************************************
 * @brief  检测I2C设备是否使能.
 * @param  base I2C实例.
 * @return 返回I2C使能状态
 *         1: 使能
 *         0: 未使能
 ******************************************************************************/
bool I2C_IsEnabled(I2C_Type *const base)
{
    return ((base->CR1) & (I2C_CR1_PE_Msk << I2C_CR1_PE_Pos)) == I2C_CR1_PE_Msk;
}

/*!*****************************************************************************
 * @brief  配置外设模式
 * @param  base I2C实例.
 * @param  peripheralmode I2C外设的工作模式(I2C,SMBUS_HOST,SMBUS_DEVICE,SMBUS_ARP等)
 * @return None
 ******************************************************************************/
void I2C_SetMode(I2C_Type *const base, i2c_mode_t peripheralmode)
{
    base->CR1 &= ~(I2C_CR1_SMBHEN_Msk | I2C_CR1_SMBDEN_Msk);
    base->CR1 |= (uint32_t)peripheralmode;
}

/*!*****************************************************************************
 * @brief  获取外设模式.
 * @param  base I2C实例.
 * @return I2C外设的工作模式(I2C,SMBUS_HOST,SMBUS_DEVICE,SMBUS_ARP等)

 ******************************************************************************/
i2c_mode_t I2C_GetMode(I2C_Type *const base)
{
    return (i2c_mode_t)((base->CR1) & (I2C_CR1_SMBHEN_Msk | I2C_CR1_SMBDEN_Msk));
}

/*!*****************************************************************************
 * @brief 当正在传输或者开始传输时处理通信 (TC 或者TCR 标志置位).
 * @param  base I2C实例.
 * @param  dev_adrress      从机地址.
 * @param  transfer_size   传输的字节数.
 * @param  endmode        停止模式（软件停止/自动停止）
 * @param  request        传输模式(起始位/停止位生成,传输方向,重复模式等)
 * @return None
 ******************************************************************************/

void I2C_HandleTransfer(I2C_Type *const base,
                        uint32_t dev_adrress,
                        uint16_t transfer_size,
                        i2c_autoend_mode_t endmode,
                        i2c_startstop_mode_t request)

{
    base->CR2 &=
        ~(I2C_CR2_SADD_Msk | (I2C_CR2_RD_WRN_Msk & ((uint32_t)request >> (31U - I2C_CR2_RD_WRN_Pos))) |
          I2C_CR2_START_Msk | I2C_CR2_STOP_Msk | I2C_CR2_RELOAD_Msk | I2C_CR2_NBYTES_Msk | I2C_CR2_AUTOEND_Msk | 
          I2C_CR2_PECBYTE_Msk);

    base->CR2 |= I2C_CR2_SADD(dev_adrress) | I2C_CR2_NBYTES(transfer_size) | (uint32_t)endmode | (uint32_t)request;
}

/*!*****************************************************************************
 * @brief I2C反初始化
 *
 * @param base I2C基地址
 * @return None
 *END**************************************************************************/
void I2C_DeInit(I2C_Type *const base)
{
    if (base == I2C1)
    {
        RCC_PeripheralForceReset(RCC_I2C1);
    }
}

/*!*****************************************************************************
 * @brief I2C初始化
 *
 * @param[in] base I2C基址
 * @param[in] config 配置结构体指针
 * @return None
 ******************************************************************************/
void I2C_Init(I2C_Type *const base, const i2c_config_t *const config)
{
    /* 关闭I2C */
    base->CR1 &= ~I2C_CR1_PE_Msk;

    /* 配置I2C时钟 */
    base->TIMINGR = config->timing & TIMING_CLEAR_MASK;

    /* 配置OA1前需先关闭OA1 */
    base->OAR1 &= ~I2C_OAR1_OA1EN_Msk;
    /* 配置自身地址1 */
    if (config->addressing_mode == I2C_ADDRESSING_MODE_7BIT)
    {
        base->OAR1 = (I2C_OAR1_OA1EN_Msk | config->own_address1);
    }
    else
    {
        base->OAR1 = (I2C_OAR1_OA1EN_Msk | I2C_OAR1_OA1MODE_Msk | config->own_address1);
    }

    /* 配置主器件寻址模式 */
    if (config->addressing_mode == I2C_ADDRESSING_MODE_10BIT)
    {
        base->CR2 = I2C_CR2_ADD10_Msk;
    }
    /* 默认开启AUTOEND和NACK(在从模式时应关闭) */
    base->CR2 |= (I2C_CR2_AUTOEND_Msk | I2C_CR2_NACK_Msk);

    /* 配置OA2前需先关闭OA2 */
    base->OAR2 &= ~I2C_OAR2_OA2EN_Msk;
    /* 配置自身地址2 */
    base->OAR2 =
        I2C_OAR2_OA2EN(config->dual_address_mode) | config->own_address2 | I2C_OAR2_OA2MSK(config->own_address2_masks);

    /* 配置广播呼叫模式和时钟延时 */
    base->CR1 = I2C_CR1_GCEN(config->general_call_mode) | I2C_CR1_NOSTRETCH(config->no_stretch_mode);

    /* 使能I2C */
    base->CR1 |= I2C_CR1_PE_Msk;
}

/*!
 * @name I2C 配置API
 * @{
 */

/*!*****************************************************************************
 * @brief  配置模拟和数字噪声滤波器.
 * @note   数字滤波器被使能时会被加到模拟滤波器后。此位需要在I2C设备禁止时更改 (PE = 0).
 *         CR1          DNF           I2C_ConfigFilters
 * @param  base I2C实例.
 * @param  analogfilter 是否启用模拟滤波器
 * @param  digitalfilter 此参数配置数字滤波器，参数值需要在 Min_Data=0x00 and Max_Data=0x0F 之间.
 * @return None
 ******************************************************************************/
void I2C_ConfigFilters(I2C_Type *const base, i2c_analogfilter_mode_t analogfilter, uint32_t digitalfilter)
{
    base->CR1 &= ~(I2C_CR1_ANFOFF_Msk | I2C_CR1_DNF_Msk);
    base->CR1 |= I2C_CR1_ANFOFF(analogfilter) | I2C_CR1_DNF(digitalfilter);
}

/*!*****************************************************************************
 * @brief  配置数字噪声滤波器.
 * @note   如果模拟噪声滤波器被使能，数字噪声滤波器只能在I2C禁止时被编程. (PE = 0).
 * @param  base I2C实例.
 * @param  digitalFilter 此参数配置数字滤波器，参数值需要在 Min_Data=0x00 and Max_Data=0x0F 之间.
 * @return None
 ******************************************************************************/
void I2C_SetDigitalFilter(I2C_Type *const base, uint32_t digitalFilter)
{
    base->CR1 &= ~I2C_CR1_DNF_Msk;
    base->CR1 |= I2C_CR1_DNF(digitalFilter);
}

/*!*****************************************************************************
 * @brief 获取当前数字滤波器配置.
 * @param  base I2C实例.
 * @return 返回值在 Min_Data=0x0 and Max_Data=0xF之间
 ******************************************************************************/
uint32_t I2C_GetDigitalFilter(I2C_Type *const base)
{
    return (uint32_t)(((base->CR1) & (I2C_CR1_DNF_Msk)) >> I2C_CR1_DNF_Pos);
}

/*!*****************************************************************************
 * @brief  使能模拟噪声滤波器.
 * @note   只能在I2C设备禁止时被编程 (PE = 0).
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetAnalogFilterMode(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 &= ~I2C_CR1_ANFOFF_Msk;
    }
    else
    {
        base->CR1 |= I2C_CR1_ANFOFF_Msk;
    }
}

/*!*****************************************************************************
 * @brief  使能时钟延长.
 * @note   只能在I2C禁止时被编程 (PE = 0)
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetClockStretching(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 &= ~I2C_CR1_NOSTRETCH_Msk;
    }
    else
    {
        base->CR1 |= I2C_CR1_NOSTRETCH_Msk;
    }
}

/*!*****************************************************************************
 * @brief  使能硬件字节控制.
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetSlaveByteControl(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= I2C_CR1_SBC_Msk;
    }
    else
    {
        base->CR1 &= ~(I2C_CR1_SBC_Msk);
    }
}

/*!*****************************************************************************
 * @brief  使能停止状态唤醒.
 * @note   可以使用 IS_I2C_WAKEUP_FROMSTOP_INSTANCE(base) 检测I2C设备是否支持该功能
 * @note   只有在数字滤波器禁止时才能使用.
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetWakeUpFromStop(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= I2C_CR1_WUPEN_Msk;
    }
    else
    {
        base->CR1 &= ~(I2C_CR1_WUPEN_Msk);
    }
}

/*!*****************************************************************************
 * @brief  使能地址广播功能.
 * @note   使能时,地址0x00会被响应.
 * @note   禁止时不对0x00地址进行响应.
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetGeneralCall(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= I2C_CR1_GCEN_Msk;
    }
    else
    {
        base->CR1 &= ~(I2C_CR1_GCEN_Msk);
    }
}

/*!*****************************************************************************
 * @brief  配置7位或者10位寻址模式
 * @note   当START=1时不允许更改此位.
 * @param  base I2C实例.
 * @param  addressingmode 7bit/10bit寻址
 * @return None
 ******************************************************************************/
void I2C_SetMasterAddressingMode(I2C_Type *const base, i2c_addressing_mode_t addressingmode)
{
    base->CR2 &= ~I2C_CR2_ADD10_Msk;
    base->CR2 |= I2C_CR2_ADD10(addressingmode);
}

/*!*****************************************************************************
 * @brief  获取寻址模式.
 * @param  base I2C实例.
 * @return 返回寻址模式(7bit/10bit)
 ******************************************************************************/
i2c_addressing_mode_t I2C_GetMasterAddressingMode(I2C_Type *const base)
{
    return (i2c_addressing_mode_t)(((base->CR2) & I2C_CR2_ADD10_Msk) >> I2C_CR2_ADD10_Pos);
}

/*!*****************************************************************************
 * @brief  设置自身地址1
 *         OAR1         OA1MODE       I2C_SetOwnAddress1
 * @param  base I2C实例.
 * @param  ownaddress1 此参数需要在 Min_Data=0 和 Max_Data=0x3FF之间.
 * @param  ownaddrsize 7bit/10bit
 * @return None
 ******************************************************************************/
 void I2C_SetOwnAddress1(I2C_Type *const base, uint32_t ownaddress1, i2c_addressing_mode_t ownaddrsize)
 {
    base->OAR1 &= ~(I2C_OAR1_OA1_0_Msk | I2C_OAR1_OA1_7_1_Msk | I2C_OAR1_OA1_8_9_Msk | I2C_OAR1_OA1MODE_Msk);
    
    base->OAR1 |= ownaddress1 | I2C_OAR1_OA1MODE(ownaddrsize);
 }

/*!*****************************************************************************
 * @brief  使能自身地址1.
 *
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetOwnAddress1Mode(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->OAR1 |= I2C_OAR1_OA1EN_Msk;
    }
    else
    {
        base->OAR1 &= ~I2C_OAR1_OA1EN_Msk;
    }
}

/*!*****************************************************************************
 * @brief  设置7位自身地址2.
 * @note   只在自身地址2被使能时有用.
 *         OAR2         OA2MSK        I2C_SetOwnAddress2
 * @param  base I2C实例.
 * @param  ownaddress2 值需要在Min_Data=0 and Max_Data=0x7F之间.
 * @param  ownaddrmask 7位自身地址2掩码
 * @return None
 ******************************************************************************/
void I2C_SetOwnAddress2(I2C_Type *const base, uint32_t ownaddress2, uint32_t ownaddrmask)
{
    base->OAR2 &= ~(I2C_OAR2_OA2_Msk | I2C_OAR2_OA2MSK_Msk);
    base->OAR2 |= (ownaddress2 | ownaddrmask);
}

/*!*****************************************************************************
 * @brief  使能自身地址2响应.
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetOwnAddress2mode(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->OAR2 |= I2C_OAR2_OA2EN_Msk;
    }
    else
    {
        base->OAR2 &= ~I2C_OAR2_OA2EN_Msk;
    }
}

/*!*****************************************************************************
 * @brief 配置SDA建立和保持时间，SCL高低电平时钟周期.
 * @note   仅在 I2C 禁止时被编程(PE = 0).
 * @param  base I2C实例.
 * @param  timing 此参数需要在Min_Data=0 and Max_Data=0xFFFFFFFF之间.
 * @return None
 ******************************************************************************/
void I2C_SetTiming(I2C_Type *const base, uint32_t timing)
{
    base->TIMINGR = (uint32_t)timing;
}

/*!*****************************************************************************
 * @brief  使能主模式下的自动停止位生成
 * @note   自动停止模式:当NBYTES数据传输完成，自动发送停止条件.
 *         此位在从机模式或者RELOAD位置位时无效.
 * @note  软件停止模式 : 当传输完NBYTES数据时TC标志置位, SCL低电平延长.
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetAutoEnd(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR2 |= I2C_CR2_AUTOEND_Msk;
    }
    else
    {
        base->CR2 &= ~(I2C_CR2_AUTOEND_Msk);
    }
}

/*!*****************************************************************************
 * @brief  使能reload模式.
 * @note   当NBYTES数据被传输，传输并未结束, 当TCR置位NBYTES将会被重置.
 * @note   当NBYTES数据传输完，传输完成。(紧跟产生STOP或者RESTART).
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetReloadMode(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR2 |= I2C_CR2_RELOAD_Msk;
    }
    else
    {
        base->CR2 &= ~(I2C_CR2_RELOAD_Msk);
    }
}

/*!*****************************************************************************
 * @brief  配置需要传输的字节数.
 * @note   当START 置位，此位不允许更改.
 * @param  base I2C实例.
 * @param  TransferSize 参数值在 Min_Data=0x00 and Max_Data=0xFF之间
 * @return None
 ******************************************************************************/
void I2C_SetTransferSize(I2C_Type *const base, uint32_t transfer_size)
{
    base->CR2 &= ~(I2C_CR2_NBYTES_Msk);
    base->CR2 |= I2C_CR2_NBYTES(transfer_size);
}

/*!*****************************************************************************
 * @brief  获取NBTYES.
 * @param  base I2C实例.
 * @return 返回值在 Min_Data=0x0 and Max_Data=0xFF之间
 ******************************************************************************/
uint32_t I2C_GetTransferSize(I2C_Type *const base)
{
    return (uint32_t)(((base->CR2) & I2C_CR2_NBYTES_Msk) >> I2C_CR2_NBYTES_Pos);
}

/*!*****************************************************************************
 * @brief  当地址匹配或者下一字节传输后，准备ACK或者NACK条件
 * @note   从模式下使用.
 * @param  base I2C实例.
 * @param  type_acknowledge I2C_ACK,I2C_NACK
 * @return None
 ******************************************************************************/
void I2C_AcknowledgeNextData(I2C_Type *const base, i2c_ack_mode_t type_acknowledge)
{
    base->CR2 &= ~(I2C_CR2_NACK_Msk);
    base->CR2 |= I2C_CR2_NACK(type_acknowledge);
}

/*!*****************************************************************************
 * @brief  产生START 或者RESTART 条件
 * @note   即使在总线繁忙时START也能产生
 * @param  base I2C实例.
 * @return None
 ******************************************************************************/
void I2C_GenerateStartCondition(I2C_Type *const base)
{
    base->CR2 |= I2C_CR2_START_Msk;
}

/*!*****************************************************************************
 * @brief  当前字节传输完成产生STOP条件
 * @param  base I2C实例.
 * @return None
 ******************************************************************************/
void I2C_GenerateStopCondition(I2C_Type *const base)
{
    base->CR2 |= I2C_CR2_STOP_Msk;
}

/*!*****************************************************************************
 * @brief  使能10位模式下的读取方向自动Restart条件生成 (主机模式).
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetAuto10BitRead(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR2 &= ~(I2C_CR2_HEAD10R_Msk);
    }
    else
    {
        base->CR2 |= I2C_CR2_HEAD10R_Msk;
    }
}

/*!*****************************************************************************
 * @brief  配置传输方向，主模式.
 * @note   当START=1 不允许更改此位.
 * @param  base I2C实例.
 * @param  TransferRequest 传输读写请求(I2C_REQUEST_WRITE I2C_REQUEST_READ)
 * @return None
 ******************************************************************************/
void I2C_SetTransferRequest(I2C_Type *const base, uint32_t transferrequest)
{
    base->CR2 &= ~(I2C_CR2_RD_WRN_Msk);
    base->CR2 |= transferrequest;
}

/*!*****************************************************************************
 * @brief  获取传输方向
 * @param  base I2C实例.
 * @return 传输模式
 ******************************************************************************/
i2c_request_mode_t I2C_GetTransferRequest(I2C_Type *const base)
{
    return (i2c_request_mode_t)(((base->CR2) & I2C_CR2_RD_WRN_Msk) >> I2C_CR2_RD_WRN_Pos);
}

/*!*****************************************************************************
 * @brief  配置数据传输的从机地址 (主模式).
 * @note   当START=1 不能更改此位.
 * @param  base I2C实例.
 * @param  SlaveAddr 此参数在 Min_Data=0x00 and Max_Data=0x3F之间.
 * @return None
 ******************************************************************************/
void I2C_SetSlaveAddr(I2C_Type *const base, uint32_t slaveaddr)
{
    base->CR2 &= ~I2C_CR2_SADD_Msk;
    base->CR2 |= I2C_CR2_SADD(slaveaddr);
}

/*!*****************************************************************************
 * @brief  获取编程的从机地址.
 * @param  base I2C实例.
 * @return 返回从机地址
 ******************************************************************************/
uint32_t I2C_GetSlaveAddr(I2C_Type *const base)
{
    return (uint32_t)((base->CR2) & I2C_CR2_SADD_Msk);
}

/*!*****************************************************************************
 * @brief  指示传输方向(从模式).
 * @note   RESET: 写传输, 从机进入接收模式.
 *         SET: 读传输, 从机进入发送模式.
 * @param  base I2C实例.
 * @return 返回读写方向,I2C_DIRECTION_WRITE/I2C_DIRECTION_READ
 ******************************************************************************/
i2c_directionflag_t I2C_GetTransferDirection(I2C_Type *const base)
{
    return (i2c_directionflag_t)((base->ISR & I2C_ISR_DIR_Msk) >> I2C_ISR_DIR_Pos);
}

/*!*****************************************************************************
 * @brief  返回从机匹配地址.
 * @param  base I2C实例.
 * @return 返回值在 Min_Data=0x00 and Max_Data=0x3F间
 ******************************************************************************/
uint32_t I2C_GetAddressMatchCode(I2C_Type *const base)
{
    return (uint32_t)(((base->ISR) & I2C_ISR_ADDCODE_Msk) >> I2C_ISR_ADDCODE_Pos << 1);
}

/*! @} */

/*!
 * @name I2C_SMBUS API
 * @{
 */

/*!*****************************************************************************
 * @brief  使能SMBus Alert (主模式或者设备模式)

 * @note   SMBus 设备模式:
 *         - SMBus Alert 引脚 被拉低并且 Alert Response 地址响应使能.
 *         SMBus 主模式:
 *         - 支持SMBus Alert 引脚管理.
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SMBUS_SetAlert(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= I2C_CR1_ALERTEN_Msk;
    }
    else
    {
        base->CR1 &= ~(I2C_CR1_ALERTEN_Msk);
    }
}

/*!*****************************************************************************
 * @brief  使能 SMBus  PEC.
 * @param  base I2C实例.
 * @param  enable 使能
 * @return None
 ******************************************************************************/
void I2C_SMBUS_SetPEC(I2C_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= I2C_CR1_PECEN_Msk;
    }
    else
    {
        base->CR1 &= ~(I2C_CR1_PECEN_Msk);
    }
}

/*!*****************************************************************************
 * @brief  配置 SMBus Clock TimeoutA (SCL 低电平超时 或者 SCL和SDA 高电平超时，取决于TimeoutA 模式).
 * @note   此参数只能在TimeoutA 禁止时被修改.
 * @param  base I2C实例.
 * @param  TimeoutA 参数取值在 Min_Data=0 and Max_Data=0xFFF间.
 * @return None
 ******************************************************************************/
void I2C_SMBUS_SetTimeoutA(I2C_Type *const base, uint32_t timeoutA)
{
    base->TIMEOUTR = timeoutA;
}


/*!*****************************************************************************
 * @brief  设置SMBus Clock TimeoutA 模式.

 * @param  base I2C实例.
 * @param  TimeoutAMode 参数值可取:
 *         @arg @ref I2C_SMBUS_TIMEOUTA_MODE_SCL_LOW
 *         @arg @ref I2C_SMBUS_TIMEOUTA_MODE_SDA_SCL_HIGH
 * @return None
 ******************************************************************************/
void I2C_SMBUS_SetTimeoutAMode(I2C_Type *const base, i2c_smbus_timeouta_mode_t timeoutAmode)
{
    base->TIMEOUTR &= ~I2C_TIMEOUTR_TIDLE_Msk;
    base->TIMEOUTR |= I2C_TIMEOUTR_TIDLE(timeoutAmode);
}

/*!*****************************************************************************
 * @brief  设置SMBus TimeoutB超时 (Master or Slave mode).

 * @note   仅当TimeoutB 禁止时编程此位.
 * @param  base I2C实例.
 * @param  TimeoutB 参数取值在 Min_Data=0 and Max_Data=0xFFF间.
 * @return None
 ******************************************************************************/
void I2C_SMBUS_SetTimeoutB(I2C_Type *const base, uint32_t timeoutB)
{
    base->TIMEOUTR = timeoutB << I2C_TIMEOUTR_TIMEOUTB_Pos;
}

/*!*****************************************************************************
 * @brief  使能 SMBus 时钟超时.
 *         TIMEOUTR     TEXTEN        I2C_SMBUS_EnableTimeout
 * @param  base I2C实例.
 * @param  clocktimeout 参数可取:
 *         @arg @ref I2C_SMBUS_TIMEOUTA
 *         @arg @ref I2C_SMBUS_TIMEOUTB
 *         @arg @ref I2C_SMBUS_ALL_TIMEOUT
 * @return None
 ******************************************************************************/
void I2C_SMBUS_EnableTimeout(I2C_Type *const base, i2c_smbus_timout_config_t clocktimeout)
{
    if (clocktimeout == I2C_SMBUS_TIMEOUTA)
    {
        base->TIMEOUTR |= I2C_TIMEOUTR_TIMOUTEN_Msk;
        base->TIMEOUTR &= ~I2C_TIMEOUTR_TEXTEN_Msk;
    }
    else if (clocktimeout == I2C_SMBUS_TIMEOUTB)
    {
        base->TIMEOUTR &= ~I2C_TIMEOUTR_TIMOUTEN_Msk;
        base->TIMEOUTR |= I2C_TIMEOUTR_TEXTEN_Msk;
    }
    else
    {
        base->TIMEOUTR |= I2C_TIMEOUTR_TIMOUTEN_Msk;
        base->TIMEOUTR |= I2C_TIMEOUTR_TEXTEN_Msk;
    }
}

/*!*****************************************************************************
 * @brief 禁止SMBus时钟超时.
 *         TIMEOUTR     TEXTEN        I2C_SMBUS_DisableTimeout
 * @param  base I2C实例.
 * @param  ClockTimeout 参数可取：
 *         @arg @ref I2C_SMBUS_TIMEOUTA
 *         @arg @ref I2C_SMBUS_TIMEOUTB
 *         @arg @ref I2C_SMBUS_ALL_TIMEOUT
 * @return None
 ******************************************************************************/
void I2C_SMBUS_DisableTimeout(I2C_Type *const base, uint32_t clocktimeout)
{
    base->TIMEOUTR &= ~(clocktimeout);
}

/*!*****************************************************************************
 * @brief  使能 SMBus PEC内部比较 (发送或者接收模式).

 * @note   当PEC字节传输完时，STOP条件产生时或者地址匹配时，该功能被硬件清零 . 当RELOAD =1，该位在 设备模式
 SBC位未置位时不起作用
 * @param  base I2C实例.
 * @return None
 ******************************************************************************/
void I2C_SMBUS_EnablePECCompare(I2C_Type *const base)
{
    base->CR2 |= I2C_CR2_PECBYTE_Msk;
}

/*!*****************************************************************************
 * @brief  获取SMBus PEC.
 * @param  base I2C实例.
 * @return 返回值在Min_Data=0x00 and Max_Data=0xFF之间
 ******************************************************************************/
uint32_t I2C_SMBUS_GetPEC(I2C_Type *const base)
{
    return (uint32_t)((base->PECR) & I2C_PECR_PEC_Msk);
}

/*! @} */

/*!
 * @name I2C 数据收发API
 * @{
 */

/*!*****************************************************************************
 * @brief  获取数据接收寄存器值
 * @param  base I2C实例.
 * @return 返回值在 Min_Data=0x00 and Max_Data=0xFF之间
 ******************************************************************************/
uint8_t I2C_ReceiveData8(I2C_Type *const base)
{
    return (uint8_t)((base->RXDR) & I2C_RXDR_RXDATA_Msk);
}

/*!*****************************************************************************
 * @brief  写数据发送寄存器 .
 * @param  base I2C实例.
 * @param  Data 值在 Min_Data=0x00 and Max_Data=0xFF之间
 * @return None
 ******************************************************************************/
void I2C_TransmitData8(I2C_Type *const base, uint8_t Data)
{
    base->TXDR = (uint8_t)Data;
}

/*! @} */

/*!
 * @name I2C 状态与中断API
 * @{
 */
/*!*****************************************************************************
 * @brief 获取I2C状态
 *
 * @param[in] base I2C基址
 * @param[in] flag掩码，一次只选择一个状态
 * @return 状态标志位
 * - ENABLE   置位
 * - DISABLE  未置位
 ******************************************************************************/
bool I2C_GetStatusFlag(I2C_Type *const base, i2c_status_flag_t flag)
{
    return ((base->ISR & flag) == (flag)) ? true : false;
}

/*!*****************************************************************************
 * @brief 清除I2C状态
 *
 * @param[in] base I2C基址
 * @param[in] flag掩码，一次只选择一个状态
 *  @arg I2C_FLAG_ADDR
 *  @arg I2C_FLAG_NACK
 *  @arg I2C_FLAG_STOP
 *  @arg I2C_FLAG_BERR
 *  @arg I2C_FLAG_ARLO
 *  @arg I2C_FLAG_OVR
 *  @arg I2C_FLAG_PEC
 *  @arg I2C_FLAG_TIMOUT
 *  @arg I2C_FLAG_ALERT
 * @return None
 ******************************************************************************/
void I2C_ClearStatusFlag(I2C_Type *const base, i2c_status_flag_t flag)
{
    base->ICR |= (uint32_t)flag;
}

/*!*****************************************************************************
 * @brief 使能I2C中断
 *
 * @param[in] base I2C基址
 * @param[in] interrupt 中断
 * @param[in] enable 使能
 * @return None
 ******************************************************************************/
void I2C_SetIntMode(I2C_Type *const base, i2c_interrupt_t interrupt, bool enable)
{
    if (enable)
    {
        base->CR1 |= (uint32_t)interrupt;
    }
    else
    {
        base->CR1 &= ~((uint32_t)interrupt);
    }
}

/*!*****************************************************************************
 * @brief 获取I2C中断状态
 * @param[in] base I2C基址
 * @param[in] flag掩码，一次只选择一个状态
 * @return 返回中断标志状态
 * - ENABLE   置位
 * - DISABLE  未置位
 ******************************************************************************/
bool I2C_GetIntFlag(I2C_Type *const base, i2c_status_flag_t flag)
{
    uint32_t cr1_reg = base->CR1;
    uint32_t isr_reg = base->ISR;
    uint32_t cr2_reg = base->CR2;
    bool retval = false;

    switch (flag)
    {
    case I2C_FLAG_ALERT:
        retval = (isr_reg & I2C_FLAG_ALERT) && (cr1_reg & I2C_INT_ERRIE);
        break;
    case I2C_FLAG_TIMEOUT:
        retval = (isr_reg & I2C_FLAG_TIMEOUT) && (cr1_reg & I2C_INT_ERRIE);
        break;
    case I2C_FLAG_PECERR:
        retval = (isr_reg & I2C_FLAG_PECERR) && (cr1_reg & I2C_INT_ERRIE);
        break;
    case I2C_FLAG_OVR:
        retval = (isr_reg & I2C_FLAG_OVR) && (cr1_reg & I2C_INT_ERRIE);
        break;
    case I2C_FLAG_ARLO:
        retval = (isr_reg & I2C_FLAG_ARLO) && (cr1_reg & I2C_INT_ERRIE);
        break;
    case I2C_FLAG_BERR:
        retval = (isr_reg & I2C_FLAG_BERR) && (cr1_reg & I2C_INT_ERRIE);
        break;
    case I2C_FLAG_TCR:
        retval = (isr_reg & I2C_FLAG_TCR) && (cr1_reg & I2C_INT_TCIE) && (cr2_reg & I2C_CR2_RELOAD_Msk);
        break;
    case I2C_FLAG_TC:
        retval = (isr_reg & I2C_FLAG_TC) && (cr1_reg & I2C_INT_TCIE);
        break;
    case I2C_FLAG_STOP:
        retval = (isr_reg & I2C_FLAG_STOP) && (cr1_reg & I2C_INT_STOPIE);
        break;
    case I2C_FLAG_NACK:
        retval = (isr_reg & I2C_FLAG_NACK) && (cr1_reg & I2C_INT_NACKIE);
        break;
    case I2C_FLAG_ADDR:
        retval = (isr_reg & I2C_FLAG_ADDR) && (cr1_reg & I2C_INT_ADDRIE);
        break;
    case I2C_FLAG_RXNE:
        retval = (isr_reg & I2C_FLAG_RXNE) && (cr1_reg & I2C_INT_RXIE);
        break;
    case I2C_FLAG_TXIS:
        retval = (isr_reg & I2C_FLAG_TXIS) && (cr1_reg & I2C_INT_TXIE);
        break;

    default:
        break;
    }
    return retval;
}


/*! @} */
/*! @} */
