/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_iwdg IWDG
 * @brief IWDG模块固件库

## 详细说明 ##
独立看门狗使用内部64KHz时钟LSI驱动，可通过关闭LSI关闭独立看门狗。
主要特性：
- 自由运行的16位递减计数器
- 当递减计数器值达到 0 时复位（如果看门狗已激活）

## 使用指导 ##

@verbatim
    const iwdg_config_t config = {
        .prescaler    = IWDG_PRESCALER_1,
        .reload_value = 0xFFFF,
    };
    IWDG_Init(&config);
    IWDG_Enable();
@endverbatim


 * @{
 */
#include "nccm0670_iwdg.h"

#define IWDG_KEY_RELOAD           0xCAFEU /*!< 重载使能   */
#define IWDG_KEY_WR_ACCESS_ENABLE 0x5A3CU /*!< 写操作使能  */

/*!*****************************************************************************
 * @brief IWDG初始化
 *
 * @param[in] config IWDG配置
 * @return None
 ******************************************************************************/
void IWDG_Init(const iwdg_config_t *const config)
{
    IWDG->KR = IWDG_KEY_WR_ACCESS_ENABLE;
    IWDG->CR = IWDG_CR_RSTTIME(config->prescaler) | IWDG_CR_RSTDAT(config->reload_value);
    IWDG->KR = IWDG_KEY_RELOAD;
}

/*!*****************************************************************************
 * @brief 打开写保护
 *
 * @return None
 ******************************************************************************/
void IWDG_Enable(void)
{
    IWDG->CR |= IWDG_CR_EN_Msk;
}

/*!*****************************************************************************
 * @brief IWDG喂狗
 *
 * @return None
 ******************************************************************************/
void IWDG_Refresh(void)
{
    IWDG->KR = IWDG_KEY_RELOAD;
}

/*!*****************************************************************************
 * @brief 打开写保护
 *
 * @return None
 ******************************************************************************/
void IWDG_EnableWriteAccess(void)
{
    IWDG->KR = IWDG_KEY_WR_ACCESS_ENABLE;
}
/*! @} */
