/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_lpuart LPUART
 * @brief LPUART模块固件库

## 详细说明 ##

- 全双工异步通信
- 采用NRZ 标准格式（标记/空格）
- 可配置为 16 倍过采样或 8 倍过采样

- 自动波特率检测
- 数据字长度可编程（7 位、 8 位或 9 位）
- 可编程的数据顺序，最先移位 MSB 或 LSB
- 停止位可配置（支持 1 个或 2 个停止位）

- 单线半双工通信

- 发射器和接收器有单独的使能位
- 支持发送和接收的信号极性翻转

- 通信控制/错误检测标志
- 奇偶校验控制：
    - 发送奇偶校验位
    - 检查接收的数据字节的奇偶性
- 十四个具有标志位的中断源

- 从静默模式唤醒（通过空闲线检测或地址标记检测）


## 使用指导 ##

@verbatim
    lpuart_config_t LPUART_InitStruct;
    LPUART_InitStruct.baudrate              = 115200;                // 波特率
    LPUART_InitStruct.datawidth             = LPUART_DATAWIDTH_8B;    // 帧长度
    LPUART_InitStruct.stopbits              = LPUART_STOPBITS_1;      // 停止位
    LPUART_InitStruct.parity                = LPUART_PARITY_NONE;     // 奇偶校验位
    LPUART_InitStruct.transfer_direction    = LPUART_DIRECTION_TX_RX; // 传输方向

    LPUART_InitStruct.oversampling          = LPUART_OVERSAMPLING_16; // 过采样
    LPUART_Init(LPUART2, &LPUART_InitStruct);
    LPUART_Enable(LPUART2);
@endverbatim


 * @{
 */
#include "nccm0670_lpuart.h"
#include "nccm0670_rcc.h"

/*!
 * @name LPUART 通用 API
 * @{
 */
/*!*****************************************************************************
 * @brief LPUART反初始化
 *
 * @param[in] base 基地址
 * @return None
 ******************************************************************************/
void LPUART_DeInit(LPUART_Type *const base)
{
    if (base == LPUART1)
    {
        RCC_PeripheralForceReset(RCC_LPUART1);
    }
}

/*!*****************************************************************************
 * @brief LPUART初始化
 *
 * @param[in] base 基地址
 * @param[in] config LPUART配置
 * @return None
 ******************************************************************************/
void LPUART_Init(LPUART_Type *const base, const lpuart_config_t *config)
{
    LPUART_SetBaudRate(base, config->baudrate);

    base->CR1 &= ~(LPUART_CR1_M0_Msk | LPUART_CR1_M1_Msk | LPUART_CR1_PCE_Msk | LPUART_CR1_PS_Msk | LPUART_CR1_TE_Msk |
                   LPUART_CR1_RE_Msk | LPUART_CR1_UESM_Msk);
    base->CR2 &= ~(LPUART_CR2_SWAP_Msk | LPUART_CR2_STOP_Msk);

    base->CR1 |= (uint32_t)(config->datawidth) | (uint32_t)(config->parity) | (uint32_t)(config->transfer_direction) |
                 LPUART_CR1_UESM(config->stop_mode);
    base->CR2 |= LPUART_CR2_STOP(config->stopbits) | LPUART_CR2_SWAP(config->rxtx_swap);
}

/*!*****************************************************************************
 * @brief 设置波特率
 *
 * @param[in] base 基地址
 * @param[in] baud 波特率值
 * @return None
 ******************************************************************************/
void LPUART_SetBaudRate(LPUART_Type *const base, uint32_t baud)
{
    uint32_t pclk;
    rcc_lpuart_source_t clocksource = RCC_GetLpuart1ClockSource();

    switch (clocksource)
    {
    case RCC_LPUART_SRC_LSE:
        pclk = (uint32_t)LSE_VALUE;
        break;
    case RCC_LPUART_SRC_LSI:
        pclk = (uint32_t)LSI_VALUE;
        break;
    case RCC_LPUART_SRC_SYSCLK:
        pclk = RCC_GetClocksFreq(RCC_CLOCK_SYSCLK);
        break;
    case RCC_LPUART_SRC_PCLK:
        pclk = RCC_GetClocksFreq(RCC_CLOCK_PCLK1);
        break;
    default:
        pclk = 0;
        break;
    }
    base->BRR = (8 * pclk / baud);
}

/*!*****************************************************************************
 * @brief LPUART使能
 *
 * @return None
 */
void LPUART_Enable(LPUART_Type *const base)
{
    base->CR1 |= LPUART_CR1_UE(1);
}

/*!*****************************************************************************
 * @brief LPUART失能
 *
 * @return None
 ******************************************************************************/
void LPUART_Disable(LPUART_Type *const base)
{
    base->CR1 &= ~LPUART_CR1_UE(1);
}

/*!*****************************************************************************
 * @brief 设置传输方向
 *
 * @return None
 ******************************************************************************/
void LPUART_SetTransferDirection(LPUART_Type *const base, lpuart_direction_t direction)
{
    base->CR1 &= ~(LPUART_CR1_TE_Msk | LPUART_CR1_RE_Msk);
    base->CR1 |= (uint32_t)(direction);
}

/*!*****************************************************************************
 * @brief 停止模式下的 LPUART 使能
 *
 * @note LPUART 能够将 MCU 从低功耗模式唤醒。当该功能激活时， LPUART 的时钟源必须是LSI 或 LSE
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 ******************************************************************************/
void LPUART_SetStopMode(LPUART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= LPUART_CR1_UESM_Msk;
    }
    else
    {
        base->CR1 &= ~LPUART_CR1_UESM_Msk;
    }
}

/*!*****************************************************************************
 * @brief 设置传输比特顺序(lsb或者msb)
 *
 * @param[in] base 基地址
 * @param[in] bitorder 比特顺序
 * @return None
 ******************************************************************************/
void LPUART_SetTransferBitOrder(LPUART_Type *const base, lpuart_bitorder_t bitorder)
{
    base->CR2 &= ~LPUART_CR2_MSBFIRST_Msk;
    base->CR2 |= LPUART_CR2_MSBFIRST(bitorder);
}

/*!*****************************************************************************
 * @brief 交换 TX/RX 引脚
 *
 * @param[in] base 基地址
 * @param[in] swap 交换配置
 * @return None
 ******************************************************************************/
void LPUART_SetTxRxSwap(LPUART_Type *const base, lpuart_pin_swap_t swap)
{
    base->CR2 &= ~LPUART_CR2_SWAP_Msk;
    base->CR2 |= LPUART_CR2_SWAP(swap);
}

/*!*****************************************************************************
 * @brief RX 引脚有效电平反向
 *
 * @param[in] base 基地址
 * @param[in] inv 交换配置
 * @return None
 ******************************************************************************/
void LPUART_SetRxPinLevel(LPUART_Type *const base, lpuart_pin_level_t inv)
{
    base->CR2 &= ~LPUART_CR2_RXINV_Msk;
    base->CR2 |= LPUART_CR2_RXINV(inv);
}

/*!*****************************************************************************
 * @brief TX 引脚有效电平反向
 *
 * @param[in] base 基地址
 * @param[in] inv 交换配置
 * @return None
 ******************************************************************************/
void LPUART_SetTxPinLevel(LPUART_Type *const base, lpuart_pin_level_t inv)
{
    base->CR2 &= ~LPUART_CR2_TXINV_Msk;
    base->CR2 |= LPUART_CR2_TXINV(inv);
}

/*!*****************************************************************************
 * @brief 接收器超时使能
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 ******************************************************************************/
void LPUART_SetRxTimeoutMode(LPUART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= LPUART_CR1_RTOEN_Msk;
    }
    else
    {
        base->CR1 &= ~LPUART_CR1_RTOEN_Msk;
    }
}

/*!*****************************************************************************
 * @brief 接收器超时值设置
 *
 * @param[in] base 基地址
 * @param[in] timeout 超时值
 * @return None
 ******************************************************************************/
void LPUART_SetRxTimeout(LPUART_Type *const base, uint32_t timeout)
{
    base->RTOR &= ~LPUART_RTOR_RTO_Msk;
    base->RTOR |= LPUART_RTOR_RTO(timeout);
}

/*!*****************************************************************************
 * @brief 发送数据值
 *
 * @param[in] base 基地址
 * @param[in] value 数据值
 * @return None
 ******************************************************************************/
void LPUART_TransmitData(LPUART_Type *const base, uint32_t value)
{
    base->TDR = value & LPUART_TDR_TDR_Msk;
}

/*!*****************************************************************************
 * @brief 接收数据值
 *
 * @param[in] base 基地址
 * @param[in] value 数据值
 * @return None
 ******************************************************************************/
uint16_t LPUART_ReceiveData(LPUART_Type *const base)
{
    return (base->RDR & LPUART_RDR_RDR_Msk);
}

/*!*****************************************************************************
 * @brief 接收数据刷新请求
 *
 * @param[in] base 基地址
 * @return None
 ******************************************************************************/
void LPUART_RequestRxDataFlush(LPUART_Type *const base)
{
    base->RQR |= LPUART_RQR_RXFRQ_Msk;
}

/*!*****************************************************************************
 * @brief 立即请求在线路上发送 BREAK
 *
 * @param[in] base 基地址
 * @return None
 ******************************************************************************/
void LPUART_RequestBreakSending(LPUART_Type *const base)
{
    base->RQR |= LPUART_RQR_SBKRQ_Msk;
}
/*! @} */

/*!
 * @name 自动波特率侦测 API
 * @{
 */
/*!*****************************************************************************
 * @brief 自动波特率使能
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 ******************************************************************************/
void LPUART_SetAutoBaudRate(LPUART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= LPUART_CR1_ABREN_Msk;
    }
    else
    {
        base->CR1 &= ~LPUART_CR1_ABREN_Msk;
    }
}

/*!*****************************************************************************
 * @brief 自动波特率请求
 *
 * @param[in] base 基地址
 * @return None
 ******************************************************************************/
void LPUART_RequestAutoBaudRate(LPUART_Type *const base)
{
    base->RQR |= LPUART_RQR_ABRRQ_Msk;
}
/*! @} */

/*!
 * @name 字符匹配 API
 * @{
 */
/*!*****************************************************************************
 * @brief 使能字符匹配模式
 *
 * @param[in] base 基地址
 * @param[in] bool 使能
 * @return None
 ******************************************************************************/
void LPUART_SetMatchMode(LPUART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= LPUART_CR1_CMEN_Msk;
    }
    else
    {
        base->CR1 &= ~LPUART_CR1_CMEN_Msk;
    }
}

/*!*****************************************************************************
 * @brief 匹配字符设置
 *
 * @param[in] base 基地址
 * @param[in] ch   匹配字符
 * @return None
 ******************************************************************************/
void LPUART_SetMatchCharacter(LPUART_Type *const base, uint8_t ch)
{
    base->CR2 &= ~LPUART_CR2_ADD_Msk;
    base->CR2 |= LPUART_CR2_ADD(ch);
}

/*! @} */

/*!
 * @name 多机通信 API
 * @{
 */
/*!*****************************************************************************
 * @brief 设置LPUART节点的地址
 *
 * @note 在多处理器通信时于静默模式或低功耗模式下使用
 * @note 4位地址节点时，ADDM7=0,b7-b4位应设置为0
 *       8位地址节点时，ADDM7=1,发送器发送字符的 MSB 应为 1
 * @note 8位地址节点还可用于正常接收和静默模式无效时的字符检测（例如，ModBus协议中的块结束检测)。
 * 这种情况下，接收到的整个字符（8位）将与 ADD[7:0] 值进行比较，如果匹配， CMF 标志将置1
 * @note 必须在CR1[UE] = 0 且 CR1[RE] = 0时才能写入地址
 * @param[in] base 基地址
 * @param[in] address_len 地址长度
 * @param[in] node_address 节点地址
 * @return None
 ******************************************************************************/
void LPUART_ConfigNodeAddress(LPUART_Type *const base, lpuart_address_detect_t address_len, uint8_t node_address)
{
    base->CR2 &= ~LPUART_CR2_ADDM7_Msk;
    base->CR2 |= LPUART_CR2_ADDM7(address_len);

    base->CR2 &= ~LPUART_CR2_ADD_Msk;
    base->CR2 |= (uint32_t)(node_address) << LPUART_CR2_ADD_Pos;
}
/*!*****************************************************************************
 * @brief 使能静默模式
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 ******************************************************************************/
void LPUART_SetMute(LPUART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= LPUART_CR1_MME_Msk;
    }
    else
    {
        base->CR1 &= ~LPUART_CR1_MME_Msk;
    }
}

/*!*****************************************************************************
 * @brief 静默模式请求
 *
 * @param[in] base 基地址
 * @return None
 ******************************************************************************/
void LPUART_RequestEnterMuteMode(LPUART_Type *const base)
{
    base->RQR |= LPUART_RQR_MMRQ_Msk;
}

/*!*****************************************************************************
 * @brief 设置LPUART静默模式的唤醒方法
 *
 * @param[in] base 基地址
 * @param[in] method 唤醒方法
 * @return None
 ******************************************************************************/
void LPUART_SetWakeUpMethod(LPUART_Type *const base, lpuart_wakeup_method_t method)
{
    base->CR1 &= ~LPUART_CR1_WAKE_Msk;
    base->CR1 |= LPUART_CR1_WAKE(method);
}
/*! @} */

/*!
 * @name 半双工通信 API
 * @{
 */
/*!*****************************************************************************
 * @brief 使能单线半双工模式
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 ******************************************************************************/
void LPUART_SetHalfDuplex(LPUART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR3 |= LPUART_CR3_HDSEL_Msk;
    }
    else
    {
        base->CR3 &= ~LPUART_CR3_HDSEL_Msk;
    }
}
/*! @} */

/*!
 * @name 状态与中断 API
 * @{
 */
/*!*****************************************************************************
 * @brief 查询标志
 *
 * @note 若多个参数按位或，只要有一个标志被置位，函数就会返回true
 * @param[in] base 基地址
 * @param[in] flag 中断使能位
 * @return None
 ******************************************************************************/
bool LPUART_GetStatusFlag(LPUART_Type *const base, lpuart_status_flag_t flag)
{
    return ((base->SR & (uint32_t)flag) != 0);
}

/*!*****************************************************************************
 * @brief 中断使能
 *
 * @param[in] base 基地址
 * @param[in] interrupt 中断位
 * @param[in] enable 使能
 * @return None
 ******************************************************************************/
void LPUART_SetIntMode(LPUART_Type *const base, lpuart_interrupt_t interrupt, bool enable)
{
    uint32_t mask = (uint32_t)interrupt;
    if ((uint32_t)interrupt & 0x80000000U)
    {
        mask &= ~0x80000000U;
        base->CR3 &= ~mask;
        base->CR3 |= enable ? mask : 0;
    }
    else
    {
        base->CR1 &= ~mask;
        base->CR1 |= enable ? mask : 0;
    }
}

/*!*****************************************************************************
 * @brief 获取中断和状态标志状态
 *
 * @param[in] base 基地址
 * @param[in] flag 状态标志
 *  @arg LPUART_FLAG_IDLE
 *  @arg LPUART_FLAG_CMF
 *  @arg LPUART_FLAG_TXE
 *  @arg LPUART_FLAG_TC
 *  @arg LPUART_FLAG_RXNE
 *  @arg LPUART_FLAG_PE
 *  @arg LPUART_FLAG_ORE
 *  @arg LPUART_FLAG_FE
 *  @arg LPUART_FLAG_WUF
 * @return 状态
 *  - true 置位
 *  - false 清零
 ******************************************************************************/
bool LPUART_GetIntFlag(LPUART_Type *const base, lpuart_status_flag_t flag)
{
    bool retval = false;

    uint32_t reg_cr1 = base->CR1;
    uint32_t reg_cr3 = base->CR3 & 0x7FFFFFFFU;
    uint32_t reg_sr = base->SR & (uint32_t)flag;
    switch (flag)
    {
    case LPUART_FLAG_IDLE:
        retval = (reg_cr1 & LPUART_INT_IDLE) && reg_sr;
        break;
    case LPUART_FLAG_CMF:
        retval = (reg_cr1 & LPUART_INT_CM) && reg_sr;
        break;
    case LPUART_FLAG_TXE:
        retval = (reg_cr1 & LPUART_INT_TXE) && reg_sr;
        break;
    case LPUART_FLAG_TC:
        retval = (reg_cr1 & LPUART_INT_TC) && reg_sr;
        break;
    case LPUART_FLAG_RXNE:
        retval = (reg_cr1 & LPUART_INT_RXNE) && reg_sr;
        break;
    case LPUART_FLAG_PE:
        retval = (reg_cr1 & LPUART_INT_PE) && reg_sr;
        break;
    case LPUART_FLAG_ORE:
        retval = (reg_cr1 & (LPUART_INT_RXNE | LPUART_INT_ERR)) && reg_sr;
        break;
    case LPUART_FLAG_FE:
        retval = (reg_cr3 & LPUART_INT_ERR) && reg_sr;
        break;
    case LPUART_FLAG_WUF:
        retval = (reg_cr3 & LPUART_INT_WUF) && reg_sr;
        break;
    default:
        break;
    }

    return retval;
}

/*!*****************************************************************************
 * @brief 清除标志位
 *
 * @param[in] base 基地址
 * @param[in] flag 要清除的状态标志
 *  @arg LPUART_FLAG_IDLE
 *  @arg LPUART_FLAG_WUF
 *  @arg LPUART_FLAG_CMF
 *  @arg LPUART_FLAG_TC
 *  @arg LPUART_FLAG_ORE
 *  @arg LPUART_FLAG_FE
 *  @arg LPUART_FLAG_PE
 * @return None
 ******************************************************************************/
void LPUART_ClearStatusFlag(LPUART_Type *const base, lpuart_status_flag_t flag)
{
    base->CLR |= (uint32_t)flag;
}
/*! @} */

/*! @} */
