/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_syscfg SYSCFG
 * @brief SYSCFG模块固件库

## 详细说明 ##

器件配有一组配置寄存器。系统配置控制器的主要用途如下：
- 配置 IR 调制信号开关及其输出极性
- 重新映射位于代码区域开头的存储空间
- ADC触发源选择
- TIM2/10/16触发源选择
- NMI中断状态标志位

 * @{
 */
#include "nccm0670_syscfg.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief  SYSCFG反初始化
 *
 * @return None
 ******************************************************************************/
void SYSCFG_DeInit(SYSCFG_Type *const base)
{
    RCC_PeripheralForceReset(RCC_SYSCFG);
}

/*!*****************************************************************************
 * @brief  存储器映射选择
 *
 * 可用于中断向量表重映射
 *
 * @param[in] memory 映射选择
 * @return None
 ******************************************************************************/
void SYSCFG_SetRemapMemory(syscfg_mem_remap_t memory)
{
    SYSCFG->CFGR1 &= ~SYSCFG_CFGR1_MEM_MODE_Msk;
    SYSCFG->CFGR1 |= SYSCFG_CFGR1_MEM_MODE(memory);
}

/*!*****************************************************************************
 * @brief  IR 输出极性选择
 *
 * @param[in] polarity 极性
 * @return None
 ******************************************************************************/
void SYSCFG_SetIrPolarity(syscfg_ir_pol_t polarity)
{
    SYSCFG->CFGR2 &= ~SYSCFG_CFGR2_IR_POL_Msk;
    SYSCFG->CFGR2 |= SYSCFG_CFGR2_IR_POL(polarity);
}

/*!*****************************************************************************
 * @brief  IR 输出使能
 *
 * @param[in] enable 使能标志
 * @return None
 ******************************************************************************/
void SYSCFG_SetIrMode(syscfg_ir_mode_t mode)
{
    SYSCFG->CFGR2 &= ~SYSCFG_CFGR2_IR_MODE_Msk;
    SYSCFG->CFGR2 |= SYSCFG_CFGR2_IR_MODE(mode);
}

/*!*****************************************************************************
 * @brief  ADC 触发源选择
 *
 * @param[in] source 触发源
 * @return None
 ******************************************************************************/
void SYSCFG_SetAdcTriggerSelection(syscfg_adc_trg_selection_t source)
{
    SYSCFG->CFGR2 &= ~SYSCFG_CFGR2_ADC_TSEL_Msk;
    SYSCFG->CFGR2 |= SYSCFG_CFGR2_ADC_TSEL(source);
}

/*!*****************************************************************************
 * @brief  TIM2 触发源选择
 *
 * @param[in] source 触发源
 * @return None
 ******************************************************************************/
void SYSCFG_SetTim2TriggerSelection(syscfg_tim2_trg_selection_t source)
{
    SYSCFG->CFGR2 &= ~SYSCFG_CFGR2_TIM2_TSEL_Msk;
    SYSCFG->CFGR2 |= SYSCFG_CFGR2_TIM2_TSEL(source);
}

/*!*****************************************************************************
 * @brief  TIM10 触发源选择
 *
 * @param[in] source 触发源
 * @return None
 ******************************************************************************/
void SYSCFG_SetTim10TriggerSelection(syscfg_tim10_trg_selection_t source)
{
    SYSCFG->CFGR2 &= ~SYSCFG_CFGR2_TIM10_TSEL_Msk;
    SYSCFG->CFGR2 |= SYSCFG_CFGR2_TIM10_TSEL(source);
}

/*!*****************************************************************************
 * @brief  TIM16 触发源选择
 *
 * @param[in] source 触发源
 * @return None
 ******************************************************************************/
void SYSCFG_SetTim16TriggerSelection(syscfg_tim16_trg_selection_t source)
{
    SYSCFG->CFGR2 &= ~SYSCFG_CFGR2_TIM16_TSEL_Msk;
    SYSCFG->CFGR2 |= SYSCFG_CFGR2_TIM16_TSEL(source);
}

/*!*****************************************************************************
 * @brief  获取NMI状态
 *
 * @param[in] flag 状态掩码，可以同时输入多个状态
 *     @arg NMI_FLAG_SCM_ERR            晶振监测异常
 * @return true:置位，false：复位
 ******************************************************************************/
bool SYSCFG_GetNmiFlag(syscfg_nmi_flag_t flag)
{
    return (SYSCFG->NMISR & (uint32_t)flag) > 0U;
}
/*! @} */
