/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_tk TK
 * @brief TK模块固件库

## 详细说明 ##
&emsp;&emsp;
基本特征
- 支持16通道自感应电容触摸按键
- 具有屏蔽电极防水功能
- 拥有2种触摸检测工作模式
- 支持寄生电容工作范围为5-50P
- 当寄生电容为20pF时，触摸灵敏度可达0.1pF
- 低功耗设计，传感器动态扫描消耗为16 uA，平均功耗为6 uA
- 可根据环境变化自适应调节
- 工作电压范围：1.8-5.5v
- 工业温度范围：-40℃到+85℃

## 使用指导 ##

@verbatim
    // 配置touch初始化
    TK_SetChannels(channel);                      // 选择触控通道
    TK_SetTrigMode(TK_TRIG_HARDWARE);             // 硬件自动触发ADC转换
    TK_SetClockSource(TK_CLK_SRC_HSI);            // 选择内部高速时钟48MHz
    TK_SetChargeCycles(99);                       // 充电次数 64 * (TK_CRGNUM[7:0] + 1)次
    TK_SetChargeInterval(TK_CHARGE_INTERVAL_1MS); // 间隔时间1ms
    TK_SetChargeFreq(TK_CHARGE_FREQ_DIV_6);       // 工作时钟6分频
    TK_SetHwTrigLeadTime(0xff);                   // 硬件自动触发ADC转换模式时提前使能ADC时间

    // 配置ADC初始化
    adc_config_t adc_cfg    = { 0 };
    adc_cfg.continuous_conv = ADC_CONV_SINGLE;      // 默认：单次转换
    adc_cfg.prescaler       = ADC_CLOCK_DIV2;       // 默认：2分频
    adc_cfg.vref            = ADC_VREF_AVDD;        // AVDD 5V
    adc_cfg.edge            = ADC_TRIG_EDGE_RISING; // 默认：上升沿触发
    adc_cfg.trigger         = ADC_TRIG_SOFT;        // 默认：内部软件触发
    ADC_Init(&adc_cfg);

    TK_Enable();

    while (1)
    {
        while (!TK_GetStatusFlag(TK_FLAG_CHARGE_FINISHED)) // 判断充电完成标志位
        {
        }
        TK_ClearStatusFlag(TK_FLAG_CHARGE_FINISHED);
        uint16_t tk_ch = TK_GetWorkingChannel();

        tk_data[tk_ch] = ADC_GetValue();
    }
@endverbatim


 * @{
 */
#include "nccm0670_tk.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief  TK反初始化
 *
 * @return None
 ******************************************************************************/
void TK_DeInit(void)
{
    RCC_PeripheralForceReset(RCC_TK);
}

/*!*****************************************************************************
 * @brief TK按键通道选择
 *
 * @param[in] channel TK通道
 * @return None
 ******************************************************************************/
void TK_SetChannels(uint16_t channel)
{
    TK->CHSEL = channel;
}

/*!*****************************************************************************
 * @brief 配置TK硬件或软件触发
 *
 * @param[in] mode TK触发模式
 * @return None
 ******************************************************************************/
void TK_SetTrigMode(tk_trig_mode_t mode)
{
    TK->CR &= ~TK_CR_TK_ADCTRG_Msk;
    TK->CR |= TK_CR_TK_ADCTRG(mode);
}

/*!*****************************************************************************
 * @brief 读取当前ADC数据对应的TK通道序号
 *
 * @return channel 通道序号
 ******************************************************************************/
uint16_t TK_GetWorkingChannel(void)
{
    return (TK->ISR & TK_ISR_TK_CHWORK_Msk) >> TK_ISR_TK_CHWORK_Pos;
}

/*!*****************************************************************************
 * @brief TK选择工作时钟
 *
 * @param[in] clk 工作时钟
 * @return None
 ******************************************************************************/
void TK_SetClockSource(tk_clk_source_t clk)
{
    TK->CR &= ~TK_CR_TK_CLKSEL_Msk;
    TK->CR |= TK_CR_TK_CLKSEL(clk);
}

/*!*****************************************************************************
 * @brief TK配置充电次数
 *
 * @param[in] times 实际充电次数, 实际充电次数为 (times + 1) * 64
 * @return None
 ******************************************************************************/
void TK_SetChargeCycles(uint8_t times)
{
    TK->CFGR1 &= ~TK_CFGR1_TK_CRGNUM_Msk;
    TK->CFGR1 |= TK_CFGR1_TK_CRGNUM(times);
}

/*!*****************************************************************************
 * @brief TK配置两通道扫描时间间隔
 *
 * @param[in] interval  两通道扫描时间间隔
 * @return None
 ******************************************************************************/
void TK_SetChargeInterval(tk_charge_interval_t interval)
{
    TK->CFGR1 &= ~TK_CFGR1_TK_ITVL_Msk;
    TK->CFGR1 |= TK_CFGR1_TK_ITVL(interval);
}

/*!*****************************************************************************
 * @brief TK配置充放电频率
 *
 * @param[in] freq 充放电频率
 * @return None
 ******************************************************************************/
void TK_SetChargeFreq(tk_charge_freq_div_t freq)
{
    TK->CFGR1 &= ~TK_CFGR1_TK_CRGFREQ_Msk;
    TK->CFGR1 |= TK_CFGR1_TK_CRGFREQ(freq);
}

/*!*****************************************************************************
 * @brief TK软件配置放电
 *
 * @param[in] enable 放电使能
 * @return None
 ******************************************************************************/
void TK_SetDischarge(bool enable)
{
    if (enable)
    {
        TK->CR |= TK_CR_TK_DIS_Msk;
    }
    else
    {
        TK->CR &= ~TK_CR_TK_DIS_Msk;
    }
}

/*!*****************************************************************************
 * @brief TK硬件自动触发ADC转换模式时提前使能ADC时间
 *
 * @param[in] time 提前使能ADC时间
 * @return None
 ******************************************************************************/
void TK_SetHwTrigLeadTime(uint16_t time)
{
    TK->CFGR2 &= ~TK_CFGR2_Tenadc_time_Msk;
    TK->CFGR2 |= TK_CFGR2_Tenadc_time(time);
}

/*!*****************************************************************************
 * @brief TK使能
 *
 * @return None
 ******************************************************************************/
void TK_Enable()
{
    TK->CR |= TK_CR_TK_EN_Msk;
}

/*!*****************************************************************************
 * @brief TK失能
 *
 * @return None
 ******************************************************************************/
void TK_Disable()
{
    TK->CR &= ~TK_CR_TK_EN_Msk;
}

/*!*****************************************************************************
 * @brief TK低功耗唤醒通道阈值设定
 *
 * @param[in] channel 触摸通道，支持同时将多个通道或操作后输入
 * @param[in] threshold 触摸通道唤醒阈值
 * @return None
 ******************************************************************************/
void TK_SetWakeupThreshold(uint16_t channel, uint32_t threshold)
{
    for (uint8_t i = 0; i < 16; i++)
    {
        if (channel & 0x0001)
        {
            TK->TH[i] = threshold;
        }
        channel >>= 1;
    }
}

/*!*****************************************************************************
 * @brief TK低功耗唤醒条件配置
 *
 * @param[in] num  比较次数
 * @param[in] mode 最低次数匹配模式选择
 * @return None
 ******************************************************************************/
void TK_SetWakeupCriterion(uint8_t num, tk_criterion_mode_t mode)
{
    TK->MATCR &= ~(TK_MATCR_TK_SWNUM_Msk | TK_MATCR_TK_MATMODE_Msk);
    TK->MATCR |= TK_MATCR_TK_SWNUM(num) | TK_MATCR_TK_MATMODE(mode);
}

/*!*****************************************************************************
 * @brief 获取状态标志位
 *
 * @param[in] flag 状态标志位
 * @return None
 ******************************************************************************/
bool TK_GetStatusFlag(tk_status_flag_t flag)
{
    return ((TK->ISR & flag) != 0);
}

/*!*****************************************************************************
 * @brief 清除状态标志
 *
 * @param[in] flag 状态标志位
 * @note TK_FLAG_WKUP_BY_CHx (x=0~15) 都可将通道匹配标志位清零
 * @return None
 ******************************************************************************/
void TK_ClearStatusFlag(tk_status_flag_t flag)
{
    if (flag == TK_FLAG_CONVERT_UNFINISHED)
    {
        TK->ICR = TK_ICR_TK_NECOVC_Msk;
    }
    else if (flag == TK_FLAG_CHARGE_FINISHED)
    {
        TK->ICR = TK_ICR_TK_CRGC_Msk;
    }
    else
    {
        TK->ICR = TK_ICR_TK_MATC_Msk;
    }
}

/*!*****************************************************************************
 * @brief  TK中断使能
 *
 * @param[in] interrupt 中断
 * @param[in] enable 使能
 * @return None
 ********************************************************************************/
void TK_SetIntMode(tk_interrupt_t interrupt, bool enable)
{
    if (enable)
    {
        TK->CR |= (uint32_t)interrupt;
    }
    else
    {
        TK->CR &= ~(uint32_t)interrupt;
    }
}

/*!*****************************************************************************
 * @brief  TK获取中断标志
 *
 * @param[in] flag 要获取的标志
 * @return true:获取到标志；false:未获取到标志
 ********************************************************************************/
bool TK_GetIntFlag(tk_status_flag_t flag)
{
    uint32_t cr_reg = TK->CR;
    bool retval = false;

    switch (flag)
    {
    case TK_FLAG_CHARGE_FINISHED:
        retval = cr_reg & TK_CR_TK_CRGIE_Msk;
        break;
    case TK_FLAG_CONVERT_UNFINISHED:
        retval = cr_reg & TK_CR_TK_NECOVIE_Msk;
        break;
    case TK_FLAG_WKUP_BY_CH0:
    case TK_FLAG_WKUP_BY_CH1:
    case TK_FLAG_WKUP_BY_CH2:
    case TK_FLAG_WKUP_BY_CH3:
    case TK_FLAG_WKUP_BY_CH4:
    case TK_FLAG_WKUP_BY_CH5:
    case TK_FLAG_WKUP_BY_CH6:
    case TK_FLAG_WKUP_BY_CH7:
    case TK_FLAG_WKUP_BY_CH8:
    case TK_FLAG_WKUP_BY_CH9:
    case TK_FLAG_WKUP_BY_CH10:
    case TK_FLAG_WKUP_BY_CH11:
    case TK_FLAG_WKUP_BY_CH12:
    case TK_FLAG_WKUP_BY_CH13:
    case TK_FLAG_WKUP_BY_CH14:
    case TK_FLAG_WKUP_BY_CH15:
        retval = cr_reg & TK_CR_TK_MATIE_Msk;
        break;
    default:
        break;
    }
    return retval && (TK->ISR & (uint32_t)flag);
}

/*! @} */
