/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_wwdg WWDG
 * @brief WWDG模块固件库

## 详细说明 ##

- 可编程的自由运行递减计数器
- 复位条件
    * 当递减计数器值小于 0x40 时复位（如果看门狗已激活）
    * 在窗口之外重载递减计数器时复位（如果看门狗已激活）
- 提前唤醒中断 (EWI)：当递减计数器等于 0x40 时触发（如果已使能且看门狗已激活）


 * @{
 */
#include "nccm0670_wwdg.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief WWDG反初始化
 *
 * @return None
 ******************************************************************************/
void WWDG_DeInit(void)
{
    RCC_PeripheralForceReset(RCC_WWDG);
}

/*!*****************************************************************************
 * @brief WWDG初始化
 *
 * @param[in] config WWDG配置
 * @return None
 ******************************************************************************/
void WWDG_Init(const wwdg_config_t *const config)
{
    /* 设置计数器 */
    WWDG->CR  = WWDG_CR_TDAT(config->counter);

    /* 设置分频，窗口，EWI */
    WWDG->CFR = WWDG_CFR_WDGSEL(config->prescaler) | WWDG_CFR_WDAT(config->window) | WWDG_CFR_EWI(config->ewi_mode);
}

/*!*****************************************************************************
 * @brief 使能窗口看门狗定时器
 *
 ******************************************************************************/
void WWDG_Enable(void)
{
    WWDG->WPR = 0x5A3CU;
    WWDG->WPR = 0xCAFEU;
    WWDG->CR |= WWDG_CR_EN_Msk;
}

/*!*****************************************************************************
 * @brief 设置窗口看门狗计数值
 *
 * @param[in] counter 计数值
 * @return None
 ******************************************************************************/
void WWDG_SetCounter(uint32_t counter)
{
    WWDG->CR = counter;
}

/*!*****************************************************************************
 * @brief 提前唤醒中断使能
 * @note 只有在WWDG模块关闭时才能使用
 ******************************************************************************/
void WWDG_SetIntMode(wwdg_interrupt_t interrupt, bool enable)
{
    if (enable)
    {
        WWDG->CFR |= (uint32_t)interrupt;
    }
    else
    {
        WWDG->CFR &= ~(uint32_t)interrupt;
    }
}

/*!*****************************************************************************
 * @brief 获取提前唤醒中断标志位状态
 ******************************************************************************/
bool WWDG_GetStatusFlag(wwdg_status_flag_t flag)
{
    return (WWDG->SR & (uint32_t)flag) > 0U;
}

/*!*****************************************************************************
 * @brief 清除提前唤醒中断标志位状态
 ******************************************************************************/
void WWDG_ClearStatusFlag(wwdg_status_flag_t flag)
{
    WWDG->SR |= (uint32_t)flag;
}
/*! @} */
