#include "i2c_smbus.h"
#include "nccm0670_i2c.h"
#include <stddef.h>

#define MAX_NBYTE_SIZE 255U

/**
 * @brief 等待状态位变化或者超时
 *
 * @param[in] i2cx             I2C基址
 * @param[in] flag             状态标志
 * @param[out] status          状态标志值
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, 1 - 超时
 */
static uint8_t i2c_wait_flag(I2C_Type *i2cx, i2c_status_flag_t flag, bool status, uint32_t timeout)
{
    while (I2C_GetStatusFlag(i2cx, flag) == status)
    {
         
        if (timeout == 0U)
        {
            I2C_AcknowledgeNextData(i2cx, I2C_NACK);
            return 1;
        }
        else
        {
            timeout--;
        }
    }
    return 0;
}
/**
 * @brief 主设备发送数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_smbus_master_transmit(I2C_Type *i2cx, uint16_t DevAddress,
                                  uint8_t *data, uint16_t Size, uint32_t timeout)
{
    uint16_t xfer_count;
    uint16_t xfer_size;
    
    /* 等待总线空闲 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 1;
    }

    xfer_count = Size;

     /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
    if (Size > MAX_NBYTE_SIZE)
    {
      xfer_size = MAX_NBYTE_SIZE;
      /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE,并发送START和写请求 */
      I2C_HandleTransfer(i2cx, DevAddress, (uint8_t)xfer_size, 
                         I2C_MODE_RELOAD | I2C_SMBUS_SENDPEC_MODE,
                         I2C_GENERATE_START_WRITE);
    }
    else
    {
        xfer_size = Size;
        /* 传输结果自动发送STOP, 并发送START和写请求 */
        I2C_HandleTransfer(i2cx, DevAddress, (uint8_t)xfer_size, 
                         I2C_MODE_AUTOEND | I2C_SMBUS_SENDPEC_MODE, I2C_GENERATE_START_WRITE); 
        /* PEC使能时，发送的最后一个字节为PEC值 */
        xfer_size--;
        xfer_count--;
    }

    while (xfer_count > 0U)
    {
        /* 等待发送完成 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout) != 0)
        {
            return 2;
        }
        /* 写入数据 */
        I2C_TransmitData8(i2cx, *data);

        data++;
        xfer_count--;
        xfer_size--;

        if ((xfer_count != 0U) && (xfer_size == 0U))
        {
            /* 等待传输完成 */
            if (i2c_wait_flag(i2cx, I2C_FLAG_TCR, false, timeout) != 0)
            {
                return 3;
            }

            if (xfer_count > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                I2C_HandleTransfer(i2cx, DevAddress, MAX_NBYTE_SIZE, 
                         I2C_MODE_RELOAD  | I2C_SMBUS_SENDPEC_MODE, I2C_GENERATE_NOSTARTSTOP);
            }
            else
            {
                xfer_size = xfer_count;
                I2C_HandleTransfer(i2cx, DevAddress, (uint8_t)xfer_size, 
                         I2C_MODE_AUTOEND | I2C_SMBUS_SENDPEC_MODE, I2C_GENERATE_NOSTARTSTOP); 
            }
        }
    }    

    /* 等待STOP位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 4;
    }

    /* 清除STOP标志位 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);
    return 0;
}

/**
 * @brief 主设备接收数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_smbus_master_receive(I2C_Type *i2cx, uint16_t dev_address, uint8_t *data,
                                uint16_t size, uint32_t timeout)
{ 
    uint16_t xfer_count;
    uint16_t xfer_size;

    /*等待总线空闲*/
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 1;
    }

    xfer_count = size;
    /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
    if (size > MAX_NBYTE_SIZE)
    {
        xfer_size = MAX_NBYTE_SIZE;
        /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE,并发送START和读请求 */
        I2C_HandleTransfer(i2cx, dev_address, (uint8_t)xfer_size, 
                                 I2C_MODE_RELOAD  | I2C_SMBUS_SENDPEC_MODE, 
                                 I2C_GENERATE_START_READ); 
    }
    else
    {
        xfer_size = xfer_count;
        /* 传输结果自动发送STOP, 并发送START和读请求 */
        I2C_HandleTransfer(i2cx, dev_address, (uint8_t)xfer_size, 
                         I2C_MODE_AUTOEND | I2C_SMBUS_SENDPEC_MODE, 
                         I2C_GENERATE_START_READ); 
    }

    while (xfer_count > 0U)
    {
        /* 等待RXNE置位 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_RXNE, false, timeout))
        {
            return 2;
        }

        /* 接收数据 */
        *data = I2C_ReceiveData8(i2cx);

        data++;

        xfer_size--;
        xfer_count--;

        if ((xfer_count != 0U) && (xfer_size == 0U))
        {
            /* 等待传输完成 */
            if (i2c_wait_flag(i2cx, I2C_FLAG_TCR, false, timeout))
            {
                return 3;
            }

            /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
            if (xfer_count > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE */
                I2C_HandleTransfer(i2cx, dev_address, (uint8_t)xfer_size, 
                                         I2C_MODE_RELOAD | I2C_SMBUS_SENDPEC_MODE,
                                         I2C_GENERATE_NOSTARTSTOP);
            }
            else
            {
                xfer_size = xfer_count;
                 /* 传输结果自动发送STOP */
                I2C_HandleTransfer(i2cx, dev_address, (uint8_t)xfer_size,
                                         I2C_MODE_AUTOEND | I2C_SMBUS_SENDPEC_MODE,
                                         I2C_GENERATE_NOSTARTSTOP);
            }
        }
    }

    /* 等待STOP位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 4;
    }

    /* 清除STOP位 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);

    return 0;
}

/**
 * @brief 从设备发送数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_smbus_slave_transmit(I2C_Type *i2cx, uint8_t *data, uint16_t size, uint32_t timeout)
{
    uint16_t xfer_size;
    uint16_t xfer_count;

    /* 使能从设备下的字节控制 */
    I2C_SetSlaveByteControl(i2cx, ENABLE);
    
    /* 开启ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_ACK);

    /* 等待地址匹配 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_ADDR, false, timeout))
    {
        return 1;
    }

    /* 传输方向判断 */
    if (I2C_GetTransferDirection(i2cx) != I2C_DIRECTION_READ)
    {
        I2C_AcknowledgeNextData(i2cx, I2C_NACK);
        return 2;
    }

    xfer_count = size;

    /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
    if (size > MAX_NBYTE_SIZE)
    {
        xfer_size = MAX_NBYTE_SIZE;
        /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE */
        I2C_HandleTransfer(i2cx, 0, (uint8_t)xfer_size, 
								 I2C_MODE_RELOAD | I2C_SMBUS_SENDPEC_MODE, 
                                 I2C_GENERATE_NOSTARTSTOP);
    }
    else
    {
        xfer_size = size;
        I2C_HandleTransfer(i2cx, 0, (uint8_t)xfer_size, 
                                 I2C_SMBUS_SENDPEC_MODE, 
                                 I2C_GENERATE_NOSTARTSTOP);

        xfer_size--;
        xfer_count--;
        
    }
		
	/* 清除地址匹配标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_ADDR);

   while (xfer_count > 0U)
    {
        /* 等待允许写发送 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout))
        {
            return 3;
        }

        /* Write data to TXDR */
        I2C_TransmitData8(i2cx, *data);
        data++;
        xfer_size--;
        xfer_count--;
        if ((xfer_count != 0U) && (xfer_size == 0U))
        {
            /* 等待传输完成 */
            if (i2c_wait_flag(i2cx, I2C_FLAG_TCR, false, timeout))
            {
                return 4;
            }
            if (size > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                I2C_HandleTransfer(i2cx, 0, (uint8_t)xfer_size, 
                                         I2C_MODE_RELOAD | I2C_SMBUS_SENDPEC_MODE, 
                                         I2C_GENERATE_NOSTARTSTOP);
            }
            else
            {
                xfer_size = size;
                I2C_HandleTransfer(i2cx, 0, (uint8_t)xfer_size, 
                                         I2C_SMBUS_SENDPEC_MODE, 
                                         I2C_GENERATE_NOSTARTSTOP);

                xfer_size--;
                xfer_count--;                
            }
        }
    }

    /* 等待STOP位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 4;
    }

    /* 清除STOP位 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);

    /* 等待总线空闲 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 5;
    }

    /* 关闭ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_NACK);
    return 0;
}
/**
 * @brief 从设备接收数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_smbus_slave_receive(I2C_Type *i2cx, uint8_t *data, uint16_t size,
                                             uint32_t timeout)
{
    /* 使能从设备硬件字节控制 */
    I2C_SetSlaveByteControl(i2cx, ENABLE);

    /* 开启ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_ACK);

    /* 等待地址匹配 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_ADDR, false, timeout))
    {
        return 1;
    }
    /* 判断传输方向 */
    if (I2C_GetTransferDirection(i2cx) != I2C_DIRECTION_WRITE)
    {
        I2C_AcknowledgeNextData(i2cx, I2C_NACK);
        return 2;
    }

    /* 设置接收的字节数 */
    if ((size == 2U) || (size == 1U))
    {
        I2C_HandleTransfer(i2cx, 0, (uint8_t)size, 
                                 I2C_SMBUS_SENDPEC_MODE, 
                                 I2C_GENERATE_NOSTARTSTOP);                        
    }
    else
    {
        I2C_HandleTransfer(i2cx, 0, 1, 
                                 I2C_MODE_RELOAD | I2C_SMBUS_SENDPEC_MODE, 
                                 I2C_GENERATE_NOSTARTSTOP);                  
    }

    /* 清除地址匹配标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_ADDR);

    while (size > 0U)
    {
        /* 等待RXNE位 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_RXNE, false, timeout))
        {
            return 3;
        }
        /* 读取接收的数据 */
        *data = I2C_ReceiveData8(i2cx);
        data++;
        size--;

        if(size == 1)
        {
            I2C_HandleTransfer(i2cx, 0, 1, 
                                     I2C_SMBUS_SENDPEC_MODE, 
                                     I2C_GENERATE_NOSTARTSTOP);              
        }
        else
        {
            if(size != 0)
            {
                I2C_HandleTransfer(i2cx, 0, 1, 
                                         I2C_MODE_RELOAD | I2C_SMBUS_SENDPEC_MODE, 
                                         I2C_GENERATE_NOSTARTSTOP);                      
                I2C_AcknowledgeNextData(i2cx, I2C_ACK);                
            }
        }
    }

    /* 等待STOP位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 4;
    }
    
    /* 清除STOP位 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);

    /* 等待总线空闲 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 5;
    }

    /* 关闭ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_NACK);
    return 0;
}
